/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.plugins;

/**
 * <p>
 * Una extensin es el mecanismo en la que los plugins pueden extender Andami.
 * En la aplicacin habr una instancia de cada extensin, las cuales tienen
 * unos mtodos que sern ejecutados en respuesta a acciones del usuario. Las
 * extensiones pueden aadir controles a la interfaz de usuario.  Pueden
 * aadir mens y herramientas de la barra de herramientas, de forma que al
 * ser seleccionadas se ejecute cdigo de la extensin. Adems de implementar esta
 * interfaz, una extensin ha de tener un constructor vaco, el cual ser invocado
 * para crear la instancia de la interfaz
 * </p>
 * 
 * <p>
 * En un plugin correctamente implementado se crear una instancia de cada
 * extensin. Tras la creacin se invocar al mtodo inicializar y luego con
 * la aplicacin en marcha se invocar a los mtodos isEnabled(), isVisible(),
 * repetidas veces para controlar la activacin y visualizacin de los
 * controles relacionados con la  extensin. Cuando se ejecuta una extensin
 * se ejecuta el metodo execute. Dicho mtodo se ejecuta en el thread de la
 * interfaz de usuario, por lo que dejar la interfaz bloqueada. Si se quiere
 * dejar activa la interfaz se deber lanzar un thread a parte. Al finalizar la
 * extensin, se ejecutar el mtodo finalize.
 * </p>
 * 
 * <p>
 * Esta clase est "deprecated". En un futuro de deben sustituir las apariciones
 * de esta interfaz por la clase abstracta Extension
 * </p>
 */
public interface IExtension {
	/**
     * Cdigo de inicializacin de la extension.
     */
    public void initialize();

    /**
     * Mtodo que se ejecuta al finalizar la extensin
     *
     */
    public void terminate();
    
    /**
     * Mtodo que se ejecuta cuando el usuario acciona un control de la
     * extensin
     *
     * @param status DOCUMENT ME!
     * @param actionCommand DOCUMENT ME!
     *
     * @throws Throwable DOCUMENT ME!
     */
    public void execute(String actionCommand);

    /**
     * Este mtodo es invocado por la aplicacin para habilitar o desabilitar
     * el estado de los controles (mens y botones de la barra de
     * herramientas) que ejecutan el cdigo de la extensin.
     *
     * @return true si el cdigo debe estar habilitado y false en caso
     *         contrario
     */
    public boolean isEnabled();

    /**
     * Este mtodo es invocado por la aplicacin para mostrar o esconder los
     * controles (mens y botones de la barra de herramientas)  que ejecutan
     * el cdigo de la extensin.
     *
     * @return true si el cdigo debe estar visibles y false en caso contrario
     */
    public boolean isVisible();
}
