/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiManager;

import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import com.iver.andami.PluginServices;
import com.iver.utiles.XMLEntity;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class WindowInfo {
    /** DOCUMENT ME! */
    public static final int RESIZABLE = 1;

    /** DOCUMENT ME! */
    public static final int MAXIMIZABLE = 2;

    /** DOCUMENT ME! */
    public static final int ICONIFIABLE = 4;

    /** DOCUMENT ME! */
    public static final int MODALDIALOG = 8;
    public static final int MODELESSDIALOG = 16;
    public static final int PALETTE = 32;
    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    /** DOCUMENT ME! */
    private boolean resizable = false;

    /** DOCUMENT ME! */
    private boolean maximizable = false;

    /** DOCUMENT ME! */
    private boolean iconifiable = false;

    /** DOCUMENT ME! */
    private boolean modal = false;
    private boolean modeless = false;
    
    /**
     * Do we want to persist the geometry of this window in the project file?
     */
    private boolean persistWindow = true;

    /**
     * Se usa para poner una ventana de tipo paleta, por
     * encima de las dems. Equivale a poner usar
     * JDesktopPane.PALETTE_LAYER
     */
    private boolean palette = false;

    /** These properties store the dimension and position of the frame */
    private int width = -1;
    private int height = -1;
    private int x = 0;
    private int y = 0;
    /**
     * These properties store the position and dimension of the frame when it is not maximized
     * (so that it can be restored to its original size). They are equal to the not-normal properties
     * when the frame is not maximized, and different when the frame is maximized.
     */
    private int normalX = 0;
    private int normalY = 0;
    private int normalHeight = 0;
    private int normalWidth = 0;
    /* Whether the window is maximized */
    private boolean isMaximized = false;
    private boolean visible = true;
    /* Whether the window is closed */
    private boolean isClosed = false;


    /** DOCUMENT ME! */
    private String title;

    private int id;

    /**
     * ActionCommand del tool seleccionado. Lo usamos
     * para activar el tool que estaba seleccionado en
     * la vista, cuando volvemos a ella.
     */
    private String selectedTool;

    /**
     * DOCUMENT ME!
     *
     * @return Returns the x.
     */
    public int getX() {
        return x;
    }

    /**
     * DOCUMENT ME!
     *
     * @param x The x to set.
     */
    public void setX(int x) {
    	support.firePropertyChange("x", this.x, x);
    	this.x = x;
    	if (!isMaximized)
    		this.normalX = x;
    }

    /**
     * DOCUMENT ME!
     *
     * @return Returns the y.
     */
    public int getY() {
        return y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param y The y to set.
     */
    public void setY(int y) {
    	support.firePropertyChange("y", this.y, y);
        this.y = y;
        if (!isMaximized)
        	this.normalY = y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        support.addPropertyChangeListener(listener);
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        support.removePropertyChangeListener(listener);
    }

    /**
     * Establece las propiedades de la vista
     *
     * @param code Bit-or de las propiedades de la vista
     *
     * @throws IllegalStateException DOCUMENT ME!
     */
    public WindowInfo(int code) {
        resizable = (code % 2) > 0;
        code = code / 2;
        maximizable = (code % 2) > 0;
        code = code / 2;
        iconifiable = (code % 2) > 0;
        code = code / 2;
        modal = (code % 2) > 0;
        code = code / 2;
        modeless = (code % 2) > 0;
        code = code / 2;
        palette = (code % 2) > 0;

        if (modal && modeless) {
            throw new IllegalStateException("modal && modeless");
        }
    }

    public WindowInfo(){

    }

    /**
     * Si es iconificable
     *
     * @return
     */
    public boolean isIconifiable() {
        return iconifiable;
    }

    /**
     * Si es maximizable
     *
     * @return
     */
    public boolean isMaximizable() {
        return maximizable;
    }

    /**
     * Si es resizable
     *
     * @return
     */
    public boolean isResizable() {
        return resizable;
    }

    /**
     * Devuelve si el dilogo es modal
     *
     * @return
     */
    public boolean isModal() {
        return modal;
    }

    /**
     * Obtiene la altura de la vista
     *
     * @return
     */
    public int getHeight() {
        return height;
    }

    /**
     * Obtiene la anchura de la vista
     *
     * @return
     */
    public int getWidth() {
        return width;
    }

    /**
     * establece la propiedad altura
     *
     * @param h
     */
    public void setHeight(int h) {
    	support.firePropertyChange("height", this.height, h);
        height = h;
        if (!isMaximized)
        	this.normalHeight = h;
    }

    /**
     * establece la propiedad anchura
     *
     * @param i
     */
    public void setWidth(int w) {
    	support.firePropertyChange("width", this.width, w);
        width = w;
        if (!isMaximized)
        	this.width = w;
    }

    /**
     * obtiene la propiedad titulo
     *
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * establece la propiedad titulo
     *
     * @param string
     */
    public void setTitle(String string) {
    	support.firePropertyChange("title", this.title, string);
        title = string;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isModeless() {
        return modeless;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isVisible() {
        return visible;
    }

    public boolean isPalette()
    {
        return palette;
    }

    public void setId(int id){
    	this.id = id;
    }

    public int getId(){
    	return id;
    }

    public void setWindowInfo(WindowInfo vi){
    	this.resizable = vi.resizable;
    	this.maximizable = vi.maximizable;
    	this.iconifiable = vi.iconifiable;
    	this.modal = vi.modal;
    	this.modeless = vi.modeless;
    	this.width = vi.width;
    	this.height = vi.height;
    	this.x = vi.x;
    	this.y = vi.y;
    	this.visible = vi.visible;
    	this.title = vi.title;
    	this.id = vi.id;
    	this.normalHeight = vi.normalHeight;
    	this.normalWidth = vi.normalWidth;
    	this.normalX = vi.normalX;
    	this.normalY = vi.normalY;
    	this.isClosed = vi.isClosed;
    	this.persistWindow = vi.persistWindow;
    }
    public void toPalette(boolean b){
    	this.palette=b;
    }

    public String getSelectedTool() {
        return selectedTool;
    }

    public void setSelectedTool(String selectedTool) {
        if (selectedTool != null)
            this.selectedTool = selectedTool;
    }

    /* Finds out whether a view is open (showing) or closed */
    public boolean isClosed() {
    	return isClosed;
    }

    /* Specifies whether a view is open (showing) or closed */
    public void setClosed(boolean closed) {
    	support.firePropertyChange("closed", this.isClosed, closed);
    	this.isClosed = closed;
    }

    public void setNormalX(int normalX) {
    	support.firePropertyChange("normalX", this.normalX, normalX);
    	this.normalX = normalX;
    }

    public void setNormalY(int normalY) {
    	support.firePropertyChange("normalY", this.normalY, normalY);
    	this.normalY = normalY;
    }

    public void setNormalHeight(int normalHeight) {
    	support.firePropertyChange("normalHeight", this.normalHeight, normalHeight);
    	this.normalHeight = normalHeight;
    }

    public void setNormalWidth(int normalWidth) {
    	support.firePropertyChange("normalWidth", this.normalWidth, normalWidth);
    	this.normalWidth = normalWidth;
    }


    public void setMaximized(boolean maximized) {
    	support.firePropertyChange("maximized", this.isMaximized, maximized);
    	this.isMaximized = maximized;
    }
    
    public void setMaximizable(boolean maximizable) {
    	this.maximizable = maximizable;
    }

    private void setBounds(int x, int y, int width, int height) {
    	support.firePropertyChange("x", this.height, x);
    	this.x = x;
    	support.firePropertyChange("y", this.height, y);
    	this.y = y;
    	support.firePropertyChange("width", this.width, width);
    	this.width = width;
    	support.firePropertyChange("height", this.height, height);
    	this.height = height;
    }

    public void setNormalBounds(int x, int y, int width, int height) {
    	support.firePropertyChange("normalX", this.normalX, x);
    	this.normalX = x;
    	support.firePropertyChange("normalY", this.normalY, y);
    	this.normalY = y;
    	support.firePropertyChange("width", this.width, width);
    	this.normalWidth = width;
    	support.firePropertyChange("height", this.height, height);
    	this.normalHeight = height;
    }

    public Rectangle getNormalBounds() {
    	return new Rectangle(getNormalX(), getNormalY(), getNormalWidth(), getNormalHeight());
    }

    public void setNormalBounds(Rectangle normalBounds) {
    	support.firePropertyChange("normalX", this.normalX, normalBounds.x);
    	normalX = normalBounds.x;
    	support.firePropertyChange("normalY", this.normalY, normalBounds.y);
    	normalY = normalBounds.y;
    	support.firePropertyChange("normalHeight", this.normalHeight, normalBounds.height);
    	normalHeight = normalBounds.height;
    	support.firePropertyChange("normalWidth", this.normalWidth, normalBounds.width);
    	normalWidth = normalBounds.width;
    }

    public void setBounds(Rectangle bounds) {
    	support.firePropertyChange("x", this.x, bounds.x);
    	x = bounds.x;
    	support.firePropertyChange("y", this.y, bounds.y);
    	y = bounds.y;
    	support.firePropertyChange("height", this.height, bounds.height);
    	height = bounds.height;
    	support.firePropertyChange("width", this.width, bounds.width);
    	width = bounds.width;
    }

    public Rectangle getBounds() {
    	return new Rectangle(x, y, width, height);
    }

    public int getNormalX() {
    	if (normalX!=0)
    		return this.normalX;
    	else
    		return x;
    }

    public int getNormalY() {
    	if (normalY!=0)
    		return this.normalY;
    	else
    		return y;
    }

    public int getNormalHeight() {
    	if (normalHeight!=0)
    		return this.normalHeight;
    	else
    		return height;
    }

    public int getNormalWidth() {
    	if (normalWidth!=0)
    		return this.normalWidth;
    	else
    		return width;
    }

    public boolean isMaximized() {
    	return this.isMaximized;
    }
    
    /**
     * Checks if the geometry of this window should be persisted in the
     * project file. This is set by the persistWindow(boolean) method,
     * and the default value is true.
     * 
     * @return True if the geometry of this window should be persisted
     * in the project files, false otherwise.
     */
    public boolean checkPersistence() {
    	return this.persistWindow;
    }
    
    /**
     * Set whether the geometry of this window should be persisted in the
     * project files.
     * 
     * @param persist
     */
    public void setPersistence(boolean persist) {
    	this.persistWindow = persist;
    }
    
    /**
	 * Gets the window properties in an XMLEntity object, suitable
	 * for persistence.
	 *
	 * @return An XMLEntity object containing the window properties, or null if the window was set as
	 * not persistent
     * @throws SaveException
	 * @throws XMLException
	 * @throws SaveException
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.setName("ViewInfoProperties");
		xml.putProperty("X", this.getX());
		xml.putProperty("Y", this.getY());
		xml.putProperty("Width", this.getWidth());
		xml.putProperty("Height", this.getHeight());
		xml.putProperty("isVisible", this.isVisible());
		xml.putProperty("isResizable", this.isResizable());
		xml.putProperty("isMaximizable", this.isMaximizable());
		xml.putProperty("isModal", this.isModal());
		xml.putProperty("isModeless", this.isModeless());
		xml.putProperty("isClosed", this.isClosed());
		if (this.isMaximized()==true) {
			xml.putProperty("isMaximized", this.isMaximized());
			xml.putProperty("normalX", this.getNormalX());
			xml.putProperty("normalY", this.getNormalY());
			xml.putProperty("normalWidth", this.getNormalWidth());
			xml.putProperty("normalHeight", this.getNormalHeight());
		}
		return xml;
	}
	
	
	public static WindowInfo createFromXMLEntity(XMLEntity xml)
	{
		WindowInfo result = new WindowInfo();
		result.setX(xml.getIntProperty("X"));
		result.setY(xml.getIntProperty("Y"));
		result.setHeight(xml.getIntProperty("Height"));
		result.setWidth(xml.getIntProperty("Width"));
		result.setClosed(xml.getBooleanProperty("isClosed"));
		if (xml.contains("isMaximized")) {
			boolean maximized = xml.getBooleanProperty("isMaximized");
			result.setMaximized(maximized);
			if (maximized==true) {
				result.setNormalBounds(xml.getIntProperty("normalX"), xml.getIntProperty("normalY"), xml.getIntProperty("normalWidth"), xml.getIntProperty("normalHeight"));
			}
			else {
				result.setNormalBounds(result.getBounds());
			}
		}

		return result;
	}
	
	public void getPropertiesFromXMLEntity(XMLEntity xml)
	{
		this.x = xml.getIntProperty("X");
		this.y = xml.getIntProperty("Y");
		this.height = xml.getIntProperty("Height");
		this.width = xml.getIntProperty("Width");
		this.isClosed = xml.getBooleanProperty("isClosed");
		if (xml.contains("isMaximized")) {
			boolean maximized = xml.getBooleanProperty("isMaximized");
			this.isMaximized = maximized;
			if (maximized==true) {
				this.setNormalBounds(xml.getIntProperty("normalX"), xml.getIntProperty("normalY"), xml.getIntProperty("normalWidth"), xml.getIntProperty("normalHeight"));
			}
		}
	}
}
