package es.gva.cit.catalogClient.protocols;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: Z3950Connection.java,v 1.1.2.3 2006/10/02 07:13:43 jorpiell Exp $
 * $Log: Z3950Connection.java,v $
 * Revision 1.1.2.3  2006/10/02 07:13:43  jorpiell
 * Comprobada una NullPointerException que generaba una excepcin
 *
 * Revision 1.1.2.2  2006/09/25 06:48:38  jorpiell
 * Aadida la funcionalidad de mostrar el documento de texto cuando no es un XML
 *
 * Revision 1.1.2.1  2006/09/20 12:01:07  jorpiell
 * Se ha cambiado la versin de la jzkit, se ha incorporado la funcionalidad de cargar arcims
 *
 * Revision 1.1  2006/09/20 11:20:17  jorpiell
 * Se ha cambiado la versin de la librera jzkit de la 1 a la 2.0
 *
 *
 */
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import org.jzkit.a2j.codec.util.OIDRegister;
import org.jzkit.search.provider.iface.IRQuery;
import org.jzkit.search.provider.iface.SearchException;
import org.jzkit.search.provider.iface.Searchable;
import org.jzkit.search.util.RecordModel.ArchetypeRecordFormatSpecification;
import org.jzkit.search.util.RecordModel.ExplicitRecordFormatSpecification;
import org.jzkit.search.util.RecordModel.InformationFragment;
import org.jzkit.search.util.RecordModel.RecordFormatSpecification;
import org.jzkit.search.util.RecordModel.SUTRS;
import org.jzkit.search.util.RecordModel.SurrogateDiagnostic;
import org.jzkit.search.util.RecordModel.XMLRecord;
import org.jzkit.search.util.ResultSet.IRResultSet;
import org.jzkit.search.util.ResultSet.IRResultSetException;
import org.jzkit.search.util.ResultSet.IRResultSetStatus;
import org.jzkit.z3950.client.SynchronousOriginBean;
import org.jzkit.z3950.gen.v3.Z39_50_APDU_1995.InitializeResponse_type;
import org.w3c.dom.Document;

import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.metadataXML.XMLTreeNumberOfRecordsAnswer;
import es.gva.cit.catalogClient.utils.Strings;

public class Z3950Connection extends SynchronousOriginBean
{
  public static final char ISO2709_RS = 035;
  public static final char ISO2709_FS = 036;
  public static final char ISO2709_IDFS = 037;
  private static final String PREFIX_QUERY_TYPE = "PREFIX";
  private static final String CCL_QUERY_TYPE = "CCL";

  private int auth_type = 0; // 0=none, 1=anonymous, 2=open, 3=idpass
  private String principal = null;
  private String group = null;
  private String credentials = null;
  
  //Added by Jorge Piera
  private InitializeResponse_type resp = null;
  private URL url = null;
  private String host = null;
  private int port = 0;
  private String database = null;
  
  
  public Z3950Connection(URL url){
	  super(new OIDRegister("/a2j.properties"));
	  this.url = url;
	  this.host = url.getHost();
	  this.port = url.getPort();
	  StringTokenizer sti = new StringTokenizer(url.getPath(), "/");
      if (sti.countTokens() == 0) {
          this.database = "geo";
      } else {
          this.database = sti.nextToken();
      }
  }
  
  /**
   * 
   * @param url
   * @return
   */
  public String connect(){
	   	if (resp == null){
	    	resp = connect(host,port,auth_type,principal,group,credentials);
	    }
	    StringBuffer string = new StringBuffer();	    
	    if (resp == null){
	    	return null;
	    }
	    if (resp.result.booleanValue() == true) {
	    	if (resp.referenceId != null) {
	    		string.append("Reference ID : " + new String(resp.referenceId) +
	    		"\n");
	    	}
	    	string.append("Implementation ID : " +
	    			resp.implementationId + "\n");
	    	string.append("Implementation Name : " +
	    			resp.implementationName + "\n");
	    	string.append("Implementation Version : " +
	    			resp.implementationVersion + "\n");
	    } else {
	    	System.out.println(" Failed to establish association");
	    	return null;
	    }
	    
	    return string.toString();
  }

  public Collection search(String sQuery,int firstRecord){
	  Collection results = new ArrayList();
	  XMLNode[] nodes = null;
	  
	  try{
		  IRQuery query = new IRQuery();
		  query.collections = new Vector();
		  query.collections.add(database);
		  query.query = new org.jzkit.search.util.QueryModel.PrefixString.PrefixString(sQuery);
		  
		  Searchable s = Z3950ConnectionFactory.getSearchable(url);
		  IRResultSet result = s.evaluate(query);
		  result.waitForStatus(IRResultSetStatus.COMPLETE|IRResultSetStatus.FAILURE,0);
		  		  
		  int lastRecord;
		  int numOfResults = result.getFragmentCount();
		  if (numOfResults > 10) {
			  nodes = new XMLNode[11];
			  if ((numOfResults - firstRecord) < 10) {
				  lastRecord = numOfResults;
			  } else {
				  lastRecord = firstRecord + 9;
			  }
		  } else {
			  nodes = new XMLNode[numOfResults + 1];
			  lastRecord = numOfResults;
		  }
		  nodes[0] = XMLTreeNumberOfRecordsAnswer.getNode(numOfResults,
				  firstRecord, lastRecord);
		  
		  RecordFormatSpecification rfs = new ExplicitRecordFormatSpecification(null,null,"f");
		  InformationFragment[] fragment = result.getFragment(firstRecord,10,rfs);

		  for (int i=0 ; i<fragment.length ; i++){
			  try {
				  String answer = "";
				  if (fragment[i] instanceof XMLRecord){
					  XMLRecord xml = (XMLRecord)fragment[i];				 	 
					  answer = Strings.replace(xml.toString(),
							  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
					  "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
				  }else {
					  SUTRS sutr = (SUTRS)fragment[i];
					  answer = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" +
					  "<" + XMLNode.ISNOTXML + ">" + 
					  sutr.toString() + 
					  "</" + XMLNode.ISNOTXML + ">";
				  }
				  
				  InputStream buffer = new ByteArrayInputStream(answer.getBytes());
				  nodes[i + 1] = XMLTree.xmlToTree(buffer);
			  } catch (Exception e1) {
				  // TODO Auto-generated catch block
				  e1.printStackTrace();
			  }
		  }	
	  }catch(SearchException e){
		  e.printStackTrace();
	  } catch (IRResultSetException e) {
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  } catch (Exception e) {
		  // TODO Auto-generated catch block
		  e.printStackTrace();
	  }
	  for (int i=0 ; i<nodes.length ; i++){
          results.add(nodes[i]);
      }
      return results;
  } 
}
