/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/* CVS MESSAGES:
 *
 * $Id: ViewSelectionControls.java,v 1.7.4.3 2006/09/25 16:08:30 caballero Exp $
 * $Log: ViewSelectionControls.java,v $
 * Revision 1.7.4.3  2006/09/25 16:08:30  caballero
 * invert selection
 *
 * Revision 1.7.4.2  2006/09/20 13:22:30  jaume
 * *** empty log message ***
 *
 * Revision 1.7.4.1  2006/09/12 07:22:07  jmvivo
 * Aadido comando InvertSelection. Portados cambios desde la rama HEAD.
 *
 * Revision 1.9  2006/09/11 16:14:15  fjp
 * Invertir seleccion. Falta subirlo al branch de la 1.0
 *
 * Revision 1.8  2006/09/11 06:53:13  fjp
 * Antes del parto
 *
 * Revision 1.7  2006/09/04 16:26:54  fjp
 * Seleccionar por polgono
 *
 * Revision 1.6  2006/08/29 07:56:27  cesar
 * Rename the *View* family of classes to *Window* (ie: SingletonView to SingletonWindow, ViewInfo to WindowInfo, etc)
 *
 * Revision 1.5  2006/08/29 07:21:08  cesar
 * Rename com.iver.cit.gvsig.fmap.Fmap class to com.iver.cit.gvsig.fmap.MapContext
 *
 * Revision 1.4  2006/08/29 07:13:53  cesar
 * Rename class com.iver.andami.ui.mdiManager.View to com.iver.andami.ui.mdiManager.IWindow
 *
 * Revision 1.3  2006/08/18 08:40:04  jmvivo
 * Actualizado para que el isEnabled tenga en cuenta que las capas esten 'avialable'
 *
 * Revision 1.2  2006/07/27 06:20:28  jaume
 * *** empty log message ***
 *
 * Revision 1.1  2006/07/26 09:53:05  jaume
 * separadas las herramientas de seleccin del viewControls al ViewSelectionControls
 *
 *
 */
package com.iver.cit.gvsig;

import org.apache.log4j.Logger;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.selectionByTheme.DefaultSelectionByThemeModel;
import com.iver.cit.gvsig.gui.selectionByTheme.MySelectionByThemeListener;
import com.iver.cit.gvsig.gui.selectionByTheme.SelectionByTheme;
import com.iver.cit.gvsig.project.ProjectView;

/**
 * Extension that handles the selection tools, selection tools have sense on
 * vectorial layers only.
 *
 * @author jaume dominguez faus - jaume.dominguez@iver.es
 *
 */
public class ViewSelectionControls extends Extension {
	private static Logger logger = Logger.getLogger(ViewControls.class
			.getName());

	public void initialize() {
		// TODO Auto-generated method stub

	}

	public void execute(String actionCommand) {
		View vista = (View) PluginServices.getMDIManager().getActiveWindow();
		ProjectView model = vista.getModel();
		MapContext mapa = model.getMapContext();
		MapControl mapCtrl = vista.getMapControl();
		logger.debug("Comand : " + actionCommand);
		if (actionCommand.equals("SELRECT")) {
			mapCtrl.setTool("rectSelection");
		} else if (actionCommand.equals("SELPOINT")) {
			mapCtrl.setTool("pointSelection");
		} else if (actionCommand.equals("SELPOL")) {
			mapCtrl.setTool("polSelection");
		} else if (actionCommand.equals("SELECTIONBYSHAPE")) {
			SelectionByTheme dlg = new SelectionByTheme();
			// FLayer[] layers = mapa.getLayers().getActives();
			// int count = 0;
			dlg.setModel(new DefaultSelectionByThemeModel());
			dlg.addSelectionListener(new MySelectionByThemeListener());
			PluginServices.getMDIManager().addWindow(dlg);
		} else if (actionCommand.equals("INVERT_SELECTION")) {
			for (int i = 0; i < mapa.getLayers().getActives().length; i++) {
				FLayer lyr = mapa.getLayers().getActives()[i];
				if (lyr.isAvailable() && lyr instanceof FLyrVect) {
					FLyrVect lyrVect = (FLyrVect) lyr;
					SelectableDataSource sds;
					try {
						sds = lyrVect.getRecordset();
						FBitSet selectedRows = sds.getSelection();
						selectedRows.flip(0, (int)sds.getRowCount());
						sds.setSelection(selectedRows);
					} catch (DriverException e) {
						e.printStackTrace();
						NotificationManager.addError(e);
					} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
						e.printStackTrace();
						NotificationManager.addError(e);
					}

				}
			}
		}
	}

	public boolean isEnabled() {
		com.iver.andami.ui.mdiManager.IWindow f = PluginServices
				.getMDIManager().getActiveWindow();
		if (f instanceof View) {

			View vista = (View) f;
			ProjectView model = vista.getModel();
			MapContext mapa = model.getMapContext();

			for (int i = 0; i < mapa.getLayers().getActives().length; i++) {
				FLayer lyr = mapa.getLayers().getActives()[i];
				if (lyr.isAvailable() && lyr instanceof FLyrVect) {
					return true;
				}
			}
			return false;
		}
		return true;
	}

	public boolean isVisible() {
		IWindow f = PluginServices.getMDIManager().getActiveWindow();

		if (f != null && f instanceof View) {
			View vista = (View) f;
			ProjectView model = vista.getModel();
			MapContext mapa = model.getMapContext();

			return hasVectorLayers(mapa.getLayers());

		}
		return false;
	}

	private boolean hasVectorLayers(FLayers layers) {
		for (int i = 0; i < layers.getLayersCount(); i++) {
			FLayer lyr =layers.getLayer(i);
			if (lyr instanceof FLayers)
				return hasVectorLayers((FLayers) lyr);

			else if (lyr instanceof FLyrVect) {
				return true;
			}
		}
		return false;
	}
}
