/*
 * Created on 27-jul-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.layout.dialogs.FAlignDialog;
import com.iver.cit.gvsig.gui.layout.dialogs.FBorderDialog;
import com.iver.cit.gvsig.gui.layout.dialogs.FPositionDialog;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameGraphics;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameGroup;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameLegend;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameNorth;
import com.iver.cit.gvsig.gui.layout.fframes.FFramePicture;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameScaleBar;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameTable;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameText;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameView;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrame;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrameUseFMap;


/**
 * Operaciones realizadas sobre el conjunto de FFrames.
 *
 * @author Vicente Caballero Navarro
 */
public class FLayoutGraphics {
	private Layout layout;
	private FAlignDialog m_alignLayout = null;
	private FBorderDialog borderdialog = null;
	private FPositionDialog positiondialog = null;

	/**
	 * Crea un nuevo FLayoutGraphics.
	 *
	 * @param l Referencia al Layout.
	 */
	public FLayoutGraphics(Layout l) {
		layout = l;
	}

	/**
	 * Transforma un FFrameLegend a FFrames de tipo FFrameSymbol y FFrameText.
	 */
	public void simplify() {
		layout.getEFS().startComplexCommand();
		IFFrame[] fframes=layout.getFFrames();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe = fframes[i];

			if (fframe instanceof FFrameLegend) {
				if (fframe.getSelected() != IFFrame.NOSELECT) {
					((FFrameLegend) fframe).toFFrames(layout);
				}
			}
		}
		layout.getEFS().endComplexCommand(PluginServices.getText(this,"simplify"));
		layout.refresh();
	}

	/**
	 * Agrupar en un FFrameGroup todos los FFrames seleccionados.
	 */
	public void grouping() {
		//		Se debe controlar de alguna forma si hay varios seleccionados.
		IFFrame fframegroup = new FFrameGroup();
		IFFrame[] fframes=layout.getFFrames();
		if (fframes.length > 1) {
			ArrayList selecList = new ArrayList();

			for (int i = fframes.length - 1; i >= 0; i--) {
				IFFrame fframe = fframes[i];

				if (fframe.getSelected() != IFFrame.NOSELECT) {
					selecList.add(fframe);
					layout.delFFrame(fframe);
				}
			}

			for (int i = selecList.size() - 1; i >= 0; i--) {
				((FFrameGroup) fframegroup).addFFrame((IFFrame) selecList.get(i));
			}

			((FFrameGroup) fframegroup).setAt(layout.getAT());

			Rectangle2D.Double rd = ((FFrameGroup) fframegroup).getRectangle(layout.getAT());

			Rectangle2D.Double rd1 = FLayoutUtilities.toSheetRect(rd,
					layout.getAT());

			fframegroup.setBoundBox(rd1);
			fframegroup.setSelected(true);
			layout.addFFrame(fframegroup, true,true);
			layout.refresh();
		}
	}

	/**
	 * Desagrupar los FFrames que estan contenidos dentro del FFrameGroup en
	 * FFrames individuales.
	 */
	public void ungrouping() {
		layout.getEFS().startComplexCommand();
		IFFrame[] fframes=layout.getFFrames();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				if (fframe instanceof FFrameGroup) {
					FFrameGroup fframegroup = (FFrameGroup) fframe;
					ArrayList selecList = new ArrayList();

					for (int j = fframegroup.getFFrames().length - 1; j >= 0;
							j--) {
						selecList.add(fframegroup.getFFrames()[j]);
					}

					for (int j = selecList.size() - 1; j >= 0; j--) {
						IFFrame frame=(IFFrame) selecList.get(j);
						frame.setRotation(frame.getRotation()+fframe.getRotation());
						layout.addFFrame(frame, false,true);
					}
					layout.delFFrame(fframegroup);
				}
			}
		}
		layout.getEFS().endComplexCommand(PluginServices.getText(this,"ungroup"));
		layout.refresh();
	}

	/**
	 * Abre el dilogo para alinear los FFrames.
	 */
	public void aligning() {
		m_alignLayout = new FAlignDialog(layout);
		PluginServices.getMDIManager().addWindow(m_alignLayout);
	}
	/**
	 * Posiciona los FFrames seleccionados delante de los no seleccionados.
	 */
	public void before() {
		layout.getEFS().startComplexCommand();
		IFFrame[] fframes=layout.getFFrames();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe = fframes[i];
			if (fframe.getSelected() != IFFrame.NOSELECT) {
				if (fframe instanceof FFrameGroup) {
					((FFrameGroup) fframe).setAt(layout.getAT());
				}

				IFFrame fframeAux=fframe.cloneFFrame(layout);
				fframeAux.setLevel(layout.getNumBefore());
				layout.getEFS().modifyFFrame(fframe,fframeAux);
				fframeAux.getBoundingBox(layout.getAT());
			}
		}
		layout.getEFS().endComplexCommand(PluginServices.getText(this,"change_before"));
		layout.updateFFrames();
		layout.refresh();
	}

	/**
	 * Posiciona los FFrames seleccionados detrs de los FFrames no
	 * seleccionados.
	 */
	public void behind() {
		layout.getEFS().startComplexCommand();
		IFFrame[] fframes=layout.getFFrames();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe = fframes[i];
			if (fframe.getSelected() != IFFrame.NOSELECT) {
				if (fframe instanceof FFrameGroup) {
					((FFrameGroup) fframe).setAt(layout.getAT());
				}

				IFFrame fframeAux=fframe.cloneFFrame(layout);
				fframeAux.setLevel(layout.getNumBehind());
				layout.getEFS().modifyFFrame(fframe,fframeAux);
				fframeAux.getBoundingBox(layout.getAT());
			}
		}
		layout.getEFS().endComplexCommand(PluginServices.getText(this,"change_behind"));
		layout.updateFFrames();
		layout.refresh();
	}

	/**
	 * Abre el dilogo adecuadao a las propiedades del FFrame seleccionado,
	 * ahora mismo solo se abre cuando hay un solo FFrame seleccionado.
	 */
	public void openFFrameDialog() {
		IFFrame[] selecList = layout.getFFrameSelected();

		if (selecList.length == 1) {
			IFFrame frame=selecList[0];
			int toolaux = layout.getTool();
			layout.setTool(getType(frame));
			IFFrame fframeAux=frame.cloneFFrame(layout);
			if (layout.openFFrameDialog(fframeAux)) {
				if (fframeAux instanceof IFFrameUseFMap)
					((IFFrameUseFMap)fframeAux).refresh();
				layout.getEFS().modifyFFrame(frame,fframeAux);
				fframeAux.getBoundingBox(layout.getAT());
				layout.updateFFrames();
				layout.setIsReSel(true);
			}
			layout.setTool(toolaux);
		}
	}

	/**
	 * Devuelve un entero que representa el tipo de FFrame  que se le pasa como
	 * parmetro.
	 *
	 * @param fframe FFrame que se pasa como parmetro, para saber de que tipo
	 * 		  es.
	 *
	 * @return entero.
	 */
	private int getType(IFFrame fframe) {
		if (fframe instanceof FFrameView) {
			return Layout.RECTANGLEVIEW;
		} else if (fframe instanceof FFrameText) {
			return Layout.RECTANGLETEXT;
		} else if (fframe instanceof FFrameScaleBar) {
			return Layout.RECTANGLESCALEBAR;
		} else if (fframe instanceof FFrameNorth) {
			return Layout.RECTANGLENORTH;
		} else if (fframe instanceof FFrameTable) {
			return Layout.RECTANGLEBOX;
		} else if (fframe instanceof FFramePicture) {
			return Layout.RECTANGLEPICTURE;
		} else if (fframe instanceof FFrameLegend) {
			return Layout.RECTANGLELEGEND;
		} else if (fframe instanceof FFrameGraphics) {
			return Layout.GRAPHICS;
		} else if (fframe instanceof FFrameGroup) {
			return Layout.RECTANGLEGROUP;
		}

		return Layout.SELECT;
	}

	/**
	 * Seleccin de todos los FFrames del Layout.
	 */
	public void selecAll() {
		IFFrame[] fframes=layout.getFFrames();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe =fframes[i];
			fframe.setSelected(true);
		}
		layout.refresh();
	}

	/**
	 * Abre el dilogo para aadir un borde a los fframes.
	 */
	public void border() {
		borderdialog = new FBorderDialog(layout);
		PluginServices.getMDIManager().addWindow(borderdialog);
	}

	/**
	 * Abre el dilogo de cambio de posicin y tamao del FFrame.
	 */
	public void position() {

		IFFrame[] fframes=layout.getFFrameSelected();
		if (fframes.length!=0){
			for (int i=0;i<fframes.length;i++){
				positiondialog = new FPositionDialog(layout,fframes[i]);
				PluginServices.getMDIManager().addWindow(positiondialog);
			}
		}
	}
}
