/*
 * Created on 12-ago-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.layout.fframes.FFrameLegend;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrame;


/**
 * Popupmenu de botn derecho que se abre sobre el Layout.
 *
 * @author Vicente Caballero Navarro
 */
public class Popupmenu extends JPopupMenu {
    private JMenuItem m_properties=null;
    private JMenuItem m_selecAll = null;
    private JMenuItem m_behind = null;
    private JMenuItem m_before = null;
	private JMenuItem m_position = null;
    private JMenuItem m_simplify = null;
    private JMenuItem m_refresh = null;
    private JMenuItem m_terminate=null;
    private FLayoutGraphics m_flg = null;
    private Layout m_layout = null;
    private Point m_p = null;

    /**
     * Crea un nuevo Popupmenu.
     *
     * @param layout Layout sobre el que se abre el Popupmenu.
     * @param p punto donde se abre el popupmenu.
     */
    public Popupmenu(Layout layout, Point p) {
        m_layout = layout;
        m_flg = new FLayoutGraphics(m_layout);
        m_p = p;
        initialize();
    }

    /**
     * Inicializa los componentes del Popupmenu.
     */
    private void initialize() {
    	m_properties=new JMenuItem(PluginServices.getText(this,"propiedades"));
        m_selecAll = new JMenuItem(PluginServices.getText(this,"seleccionar_todos"));
        m_behind = new JMenuItem(PluginServices.getText(this,"colocar_detras"));
        m_before = new JMenuItem(PluginServices.getText(this,"colocar_delante"));
        m_position = new JMenuItem(PluginServices.getText(this,"tamano_posicion"));
        m_simplify = new JMenuItem(PluginServices.getText(this,"simplificar_leyenda"));
        m_refresh = new JMenuItem(PluginServices.getText(this,"refrescar"));
        m_terminate=new JMenuItem(PluginServices.getText(this,"terminar"));
        setEnables();
        add(m_properties);
        addSeparator();
        add(m_selecAll);
        addSeparator();
        add(m_behind);
        add(m_before);
        add(m_position);
		addSeparator();
		add(m_simplify);
        addSeparator();
        add(m_refresh);
        addSeparator();
        add(m_terminate);
		m_properties.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					m_flg.openFFrameDialog();
				}
			});
        m_selecAll.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_flg.selecAll();
                }
            });
        m_behind.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_flg.behind();
                }
            });
        m_before.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_flg.before();
                }
            });
		m_position.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					m_flg.position();
				}
			});
        m_simplify.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_flg.simplify();
                }
            });
        m_refresh.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    m_layout.refresh();
                }
            });
        m_terminate.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	m_layout.endGraphic();
            }
        });
        this.show(m_layout, m_p.x, m_p.y);
    }

	private void setEnables() {
		if ((m_layout.getTool()==Layout.POLYLINE ||
				m_layout.getTool()==Layout.POLYGON)&&
				m_layout.getGeometryAdapter().getPoints().length>0){

			m_refresh.setVisible(false);
			m_selecAll.setVisible(false);
			m_properties.setVisible(false);
			m_behind.setVisible(false);
			m_before.setVisible(false);
			m_position.setVisible(false);
			m_simplify.setVisible(false);
			m_terminate.setVisible(true);
			return;
		}
		m_terminate.setVisible(false);
		IFFrame[] fframes=m_layout.getFFrameSelected();
		m_selecAll.setEnabled(m_layout.isEditable());
		if (fframes.length==0 || !m_layout.isEditable()) {
        	m_properties.setEnabled(false);
        	m_behind.setEnabled(false);
        	m_before.setEnabled(false);
        	m_position.setEnabled(false);
        	m_simplify.setEnabled(false);
        }else {
        	m_properties.setEnabled(true);
        	m_behind.setEnabled(true);
        	m_before.setEnabled(true);
        	m_position.setEnabled(true);
        	boolean simplify=false;
        	for (int i =0;i<fframes.length;i++) {
        		if (fframes[i] instanceof FFrameLegend) {
        			simplify=true;
        		}
        	}
        	m_simplify.setEnabled(simplify);
        }
	}
}
