/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.project;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;
import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;

import org.gvsig.gui.beans.swing.JButton;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DBDriver;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.driver.FileDriver;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.SingletonWindow;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.DataBaseOpenDialog;
import com.iver.cit.gvsig.gui.FOpenDialog;
import com.iver.cit.gvsig.gui.FileOpenDialog;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectElement;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.ProjectMap;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.cit.gvsig.project.ProjectView;
import com.iver.utiles.DefaultListModel;


/**
 * Clase principal del proyecto donde se puede operar para crear vistas,
 * tablas, mapas, ...
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectWindow extends JPanel implements PropertyChangeListener,
	IWindow, SingletonWindow {
	private JPanel jPanel = null;
	private JRadioButton btnVistas = null;
	private JRadioButton btnTablas = null;
	private JRadioButton btnMapas = null;
	private ButtonGroup grupo = new ButtonGroup();
	private JPanel jPanel1 = null;
	private JList lstDocs = null;
	private JPanel jPanel2 = null;
	private JButton btnNuevo = null;
	private JButton btnPropiedades = null;
	private JButton btnAbrir = null;
	private JButton btnBorrar = null;
	private JButton btnRenombrar = null;
	private JPanel jPanel3 = null;
	private JLabel jLabel = null;
	private JLabel lblNombreSesion = null;
	private JLabel jLabel1 = null;
	private JLabel lblGuardado = null;
	private JLabel jLabel2 = null;
	private JLabel lblFecha = null;
	private JButton btnImportar = null;
	private JButton btnExportar = null;
	private JButton btnEditar = null;
	// The properties of the Project Manager window (size, position, etc):
	private WindowInfo m_viewInfo=null;

	/** Proyecto representado en la vista */
	private Project p;

	/**
	 * Extension con la informacin, entre otras, sobre las extensiones para
	 * las tablas
	 */
	private ProjectExtension projectExtension;
	private JScrollPane jScrollPane = null;
	private JPanel jPanel4 = null;

	/**
	 * This is the default constructor
	 *
	 * @param project Extension
	 */
	public ProjectWindow(ProjectExtension project) {
		super();

		initialize();

		projectExtension = project;

		refreshControls();
	}

	/**
	 * This is the default constructor
	 *
	 * @param project Extension
	 */
	public ProjectWindow() {
		super();

		initialize();
		refreshControls();
	}

	public void setProjectExtension(ProjectExtension p) {
		projectExtension = p;
	}


	/**
	 * Asigna el proyecto a la ventana
	 *
	 * @param p Proyecto con el que se operar a travs de este dilogo
	 */
	public void setProject(Project p) {
		this.p = p;
		p.addPropertyChangeListener(this);
		refreshControls();
	}

	/**
	 * Activa los botones de la botonera del medio o los desactiva en funcin
	 * de que est seleccionado o no un elemento de proyecto en la lista del
	 * medio
	 */
	private void activarBotones() {
		if (lstDocs.getSelectedIndex() != -1) {
			btnAbrir.setEnabled(true);
			btnBorrar.setEnabled(true);
			if (lstDocs.getSelectedIndices().length==1) {
				btnRenombrar.setEnabled(true);
				btnPropiedades.setEnabled(true);
			}else {
				btnRenombrar.setEnabled(false);
				btnPropiedades.setEnabled(false);
			}
		} else {
			btnAbrir.setEnabled(false);
			btnBorrar.setEnabled(false);
			btnRenombrar.setEnabled(false);
			btnPropiedades.setEnabled(false);
		}
	}

	/**
	 * Refresca la lista de elementos de proyecto con vistas, mapas o tablas,
	 * segn la opcin activada
	 */
	private void refreshList() {
		if (p != null) {
			DefaultListModel model;
			String tituloMarco = PluginServices.getText(this, "documentos_existentes");

			if (btnMapas.isSelected()) {
				model = new DefaultListModel(p.getMaps());
				tituloMarco = PluginServices.getText(this, "mapas");

				//				getBtnNuevo().setEnabled(true);
			} else if (btnTablas.isSelected()) {
				model = new DefaultListModel(p.getTables());
				tituloMarco = PluginServices.getText(this, "tablas");

				//                getBtnNuevo().setEnabled(false);
			} else {
				model = new DefaultListModel(p.getViews());
				tituloMarco = PluginServices.getText(this, "vistas");

				//				getBtnNuevo().setEnabled(true);
			}

			lstDocs.setModel(model);
			((TitledBorder)getJPanel1().getBorder()).setTitle(tituloMarco);
			getJPanel1().repaint(1);

			activarBotones();
		}
	}

	/**
	 * Refresca las etiquetas con la informacin del proyecto
	 */
	private void refreshProperties() {
		if (p != null) {
			String aux;

			lblNombreSesion.setText(p.getName());

			String path = ProjectExtension.getPath();

			if (path != null) {
				File f = new File(path);
				lblGuardado.setText(f.toString());
			} else {
				lblGuardado.setText("");
			}

			lblFecha.setText(p.getCreationDate());
		}
	}

	/**
	 * Refresca todo el dilogo
	 */
	public void refreshControls() {
		refreshList();
		refreshProperties();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() {
		GridBagLayout layout = new GridBagLayout();
		setLayout(layout);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy=0;
		c.insets = new Insets(2, 5, 2, 5);

		layout.setConstraints(getJPanel(),c);
		add(getJPanel(), null);

		c.fill = GridBagConstraints.BOTH;
		c.gridy=1;
		c.weightx = 1.0;
		c.weighty = 1.0;
		layout.setConstraints(getJPanel1(),c);
		add(getJPanel1(), null);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0;
		c.weighty = 0;
		c.gridy=2;
		layout.setConstraints(getJPanel3(),c);
		add(getJPanel3(), null);

		this.setSize(430, 435);
		this.setPreferredSize(new java.awt.Dimension(430, 430));
		this.setMinimumSize(new java.awt.Dimension(430, 430));
		grupo.add(btnVistas);
		grupo.add(btnTablas);
		grupo.add(btnMapas);

		btnVistas.setSelected(true);
	}
	private String ask(String title, String content){
		JOptionPane pane = new JOptionPane();
		pane.setMessage(content);
		pane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		pane.setWantsInput(true);
		pane.setInitialSelectionValue("");

		JDialog dlg = pane.createDialog((Component) PluginServices.getMainFrame(),
				title);
		dlg.setModal(true);
		dlg.show();

		return pane.getInputValue().toString().trim();
	}

	/**
	 * Crea un nuevo project element
	 *
	 * @throws Exception DOCUMENT ME!
	 */
	private void newProjectElement() throws Exception {

		if (btnMapas.isSelected()) {
			ProjectMap map = ProjectFactory.createMap(PluginServices.getText(
						this, "untitled"));
			map.setModel(new Layout());
			map.getModel().setProjectMap(map);
			p.addMap(map);
		} else if (btnTablas.isSelected()) {
			try {
				FOpenDialog fopen = new FOpenDialog(PluginServices.getText(this,"Nueva_tabla"));
				FileOpenDialog fod = new FileOpenDialog(new Class[] {
							FileDriver.class}, false, PluginServices.getText(this,"Tablas"));
				DataBaseOpenDialog dbod = new DataBaseOpenDialog();
				dbod.setClasses(new Class[] { DBDriver.class });
				fopen.addTab(PluginServices.getText(this,"Fichero"), fod);
				fopen.addTab(PluginServices.getText(this,"base_datos"), dbod);
				PluginServices.getMDIManager().addWindow(fopen);

				if (fopen.isAccepted()) {
					JPanel panel = fopen.getSelectedTab();
					ProjectExtension ext = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);

					if (panel instanceof FileOpenDialog) {
						File[] files = fod.getFiles();
						String[] driverNames = fod.getDriverNames();

						for (int i = 0; i < files.length; i++) {
							String name = files[i].getName();

							LayerFactory.getDataSourceFactory()
										.addFileDataSource(driverNames[i],
								name, files[i].getAbsolutePath());

							DataSource dataSource = LayerFactory.getDataSourceFactory()
																.createRandomDataSource(name, DataSourceFactory.AUTOMATIC_OPENING);
							SelectableDataSource sds = new SelectableDataSource(dataSource);
							EditableAdapter auxea=new EditableAdapter();
		                	auxea.setOriginalDataSource(sds);
		                	// TODO: fjp: ESTO HAY QUE REVISARLO.
		                	// Por ahora, para obtener un driver que sirva para esta
		                	// fuente de datos, compruebo que implementa IWriteable.
		                	// IWriter writer = (IWriter) LayerFactory.getWM().getWriter(driverNames[i]);
		                	Driver drv = LayerFactory.getDM().getDriver(driverNames[i]);
//		                	if (drv instanceof IWriter)
//		                	{
//		                		auxea.setWriter((IWriter) drv);
//		                	}
							ProjectTable projectTable = ProjectFactory.createTable(name,
									auxea);
							ext.getProject().addTable(projectTable);
							Project.openTable(projectTable);
						}
					} else if (panel instanceof DataBaseOpenDialog) {
						String driverName = dbod.getDriverName();
						int port = -1;

						try {
							port = Integer.parseInt(dbod.getPort());
						} catch (NumberFormatException e) {
						}

						String name = dbod.getHost() + "/" +
							dbod.getDataBase();

						if (port != -1) {
							name = dbod.getHost() + ":" + port + "/" +
								dbod.getDataBase();
						}

						String user = dbod.getUser().trim();
						String password = dbod.getPassword();

						if (user.equals("")) {
							user = null;
							password = null;
						}

						LayerFactory.getDataSourceFactory().addDBDataSourceByTable(name,
							dbod.getHost(), port, user, password,
							dbod.getDataBase(), dbod.getTable(), driverName);

						DataSource dataSource = LayerFactory.getDataSourceFactory()
															.createRandomDataSource(name, DataSourceFactory.AUTOMATIC_OPENING);
						SelectableDataSource sds = new SelectableDataSource(dataSource);
						EditableAdapter auxea=new EditableAdapter();
	                	// TODO: fjp: ESTO HAY QUE REVISARLO.
	                	// Por ahora, para obtener un driver que sirva para esta
	                	// fuente de datos, compruebo que implementa IWriteable.
	                	// IWriter writer = (IWriter) LayerFactory.getWM().getWriter(driverNames[i]);
	                	Driver drv = LayerFactory.getDM().getDriver(driverName);
//	                	if (drv instanceof IWriter)
//	                	{
//	                		auxea.setWriter((IWriter) drv);
//	                	}

	                	auxea.setOriginalDataSource(sds);
						ProjectTable projectTable = ProjectFactory.createTable(name,
								auxea);
						ext.getProject().addTable(projectTable);

						Table t = new Table();
						t.setModel(projectTable);
						projectTable.setAndamiWindow(t);
						PluginServices.getMDIManager().addWindow(t);
					}
				}
			} catch (DriverLoadException e) {
				NotificationManager.addError("Error al cargar los drivers", e);
			}
		} else {
			String viewName = "";
			String aux = PluginServices.getText(this, "untitled");
			viewName = aux + " - " + ProjectView.numViews++;
			// Buscamos si alguna vista ya tena este nombre:
			while (existViewName(viewName))
			{
				viewName = aux + " - " + ProjectView.numViews++;
			}
			ProjectView vista = ProjectFactory.createView(viewName);
			p.addView(vista);
		}
	}

	private boolean existViewName(String viewName) {
		ArrayList viewList = p.getViews();
		for (int i=0; i < viewList.size(); i++)
		{
			ProjectView pv = (ProjectView) viewList.get(i);
			String title = pv.getName();
			if (title.compareTo(viewName) == 0)
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * Abre la ventana de un nuevo project element
	 */
	private void abrir() {
		int[] indexes = lstDocs.getSelectedIndices();
		for (int i=indexes.length-1;i>=0;i--) {
			int index=indexes[i];
			if (btnMapas.isSelected()) {
				if (index == -1) {
					return;
				}
	            ProjectMap projectMap = (ProjectMap) p.getMaps().get(index);
				Layout l = projectMap.getModel();
	            // System.out.println("l.getName " + l.getName() + " projectMap.getName " + projectMap.getName());
	            projectMap.setName(l.getName());
	            l.setProjectMap(projectMap);
				l.fullRect();
	            l.getWindowInfo().setTitle(PluginServices.getText(this,
	            "Mapa") + " : " +l.getName());
				PluginServices.getMDIManager().addWindow(l);
			} else if (btnTablas.isSelected()) {
				if (index == -1) {
					return;
				}
				ProjectTable pt=(ProjectTable) p.getTables().get(index);
				Project.openTable(pt);
			} else {
				com.iver.cit.gvsig.gui.View vista = new com.iver.cit.gvsig.gui.View();
				if (index == -1) {
					return;
				}

				vista.setModel((ProjectView) p.getViews().get(index));
				PluginServices.getMDIManager().addWindow(vista);
			}
		}
	}

	/**
	 * Cambia el nombre de un project element
	 */
	private void renombrar() {
		ProjectElement elem = null;
		int index = lstDocs.getSelectedIndex();

		if (index == -1) {
			return;
		}

		if (btnMapas.isSelected()) {
			elem = (ProjectElement) p.getMaps().get(index);
		} else if (btnTablas.isSelected()) {
			elem = (ProjectElement) p.getTables().get(index);
		} else {
			elem = (ProjectElement) p.getViews().get(index);
		}

		if (elem.isLocked()) {
			JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element_it_cannot_be_renamed"));
			return;
		}

		JOptionPane pane = new JOptionPane();
		pane.setMessage(PluginServices.getText(this, "introduce_nombre"));
		pane.setMessageType(JOptionPane.QUESTION_MESSAGE);
		pane.setWantsInput(true);
		pane.setInitialSelectionValue(elem.getName());
        pane.setInputValue(elem.getName());
		JDialog dlg = pane.createDialog((Component) PluginServices.getMainFrame(),
				PluginServices.getText(this, "renombrar"));
		dlg.setModal(true);
		dlg.show();

		String nuevoNombre = pane.getInputValue().toString().trim();

		if (nuevoNombre.length() == 0) {
			return;
		}

		if (nuevoNombre == null) {
			return;
		}

		for (int i=0; i<lstDocs.getModel().getSize(); i++){
			if (i==index) continue;
			if (((ProjectElement)lstDocs.getModel().getElementAt(i)).getName().equals(nuevoNombre)){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), PluginServices.getText(this, "elemento_ya_existe"));
				return;
			}

		}
		elem.setName(nuevoNombre);

		refreshList();
	}

	/**
	 * Borra un project element
	 */
	private void borrar() {
		int res = JOptionPane.showConfirmDialog((Component) PluginServices.getMainFrame(),
				PluginServices.getText(this, "confirmar_borrar"),
				PluginServices.getText(this, "borrar"),
				JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE);

		if (res == JOptionPane.YES_OPTION) {
			int[] indexes = lstDocs.getSelectedIndices();
			for (int i=indexes.length-1;i>=0;i--) {
				int index=indexes[i];
				if (btnMapas.isSelected()) {
					if (((ProjectElement) p.getMaps().get(index)).isLocked()) {
						JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element_it_cannot_be_deleted"));
						return;
					}
					PluginServices.getMDIManager().closeSingletonWindow(p.getMaps()
																	   .get(index));
					p.delMap(index);
				} else if (btnTablas.isSelected()) {
					if (((ProjectElement) p.getTables().get(index)).isLocked()) {
						JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element_it_cannot_be_deleted"));
						return;
					}
					PluginServices.getMDIManager().closeSingletonWindow(p.getTables()
																	   .get(index));
					p.delTable(index);
				} else {
					if (((ProjectElement) p.getViews().get(index)).isLocked()) {
						JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element_it_cannot_be_deleted"));
						return;
					}
					PluginServices.getMDIManager().closeSingletonWindow(p.getViews()
																	   .get(index));
					p.delView(index);
				}
			}
			refreshList();
		}
	}

	/**
	 * Muestra el dilogo de propiedades de un project element
	 */
	private void propiedades() {
		int index = lstDocs.getSelectedIndex();

		if (index == -1) {
			return;
		}

		IWindow dlg = null;

		if (btnMapas.isSelected()) {
			if (((ProjectElement) p.getMaps().get(index)).isLocked()) {
				JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element"));
				return;
			}
			dlg = new MapProperties((ProjectMap) p.getMaps().get(index));
		} else if (btnTablas.isSelected()) {
			if (((ProjectElement) p.getTables().get(index)).isLocked()) {
				JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element"));
				return;
			}
			try {
                dlg = new TableProperties((ProjectTable) p.getTables().get(index));
            } catch (DriverException e) {
                NotificationManager.addError(e.getMessage(), e);
            }
		} else {
			if (((ProjectElement) p.getViews().get(index)).isLocked()) {
				JOptionPane.showMessageDialog(this,	PluginServices.getText(this, "locked_element"));
				return;
			}

			dlg = new ViewProperties((ProjectView) p.getViews().get(index));
		}

		PluginServices.getMDIManager().addWindow(dlg);

		refreshControls();
		lstDocs.setSelectedIndex(index);
	}

	/**
	 * This method initializes jPanel
	 *
	 * @return JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();

			java.awt.FlowLayout layFlowLayout1 = new java.awt.FlowLayout();
			layFlowLayout1.setHgap(15);
			jPanel.setLayout(layFlowLayout1);
			jPanel.add(getBtnVistas(), null);
			jPanel.add(getBtnTablas(), null);
			jPanel.add(getBtnMapas(), null);
			jPanel.setName("tipoDocPanel");
			jPanel.setPreferredSize(new java.awt.Dimension(430, 125));
			jPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "tipos_de_documentos"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}

		return jPanel;
	}

	/**
	 * This method initializes btnVistas
	 *
	 * @return JRadioButton
	 */
	private JRadioButton getBtnVistas() {
		if (btnVistas == null) {
			btnVistas = new JRadioButton();
			btnVistas.setIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/Vista.png")));
			btnVistas.setSelectedIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/Vista_sel.png")));
			btnVistas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnVistas.setText(PluginServices.getText(this, "vistas"));
			btnVistas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnVistas.setName("vistasBtn");
			btnVistas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnVistas;
	}

	/**
	 * This method initializes btnTablas
	 *
	 * @return JRadioButton
	 */
	private JRadioButton getBtnTablas() {
		if (btnTablas == null) {
			btnTablas = new JRadioButton();
			btnTablas.setIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/tablas.png")));
			btnTablas.setSelectedIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/tablas_sel.png")));
			btnTablas.setText(PluginServices.getText(this, "tablas"));
			btnTablas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnTablas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnTablas.setName("tablasBtn");
			btnTablas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnTablas;
	}

	/**
	 * This method initializes btnMapas
	 *
	 * @return JRadioButton
	 */
	private JRadioButton getBtnMapas() {
		if (btnMapas == null) {
			btnMapas = new JRadioButton();
			btnMapas.setIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/mapas.png")));
			btnMapas.setSelectedIcon(new ImageIcon(
					this.getClass().getClassLoader().getResource("images/mapas_sel.png")));
			btnMapas.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			btnMapas.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
			btnMapas.setText(PluginServices.getText(this, "mapas"));
			btnMapas.addChangeListener(new javax.swing.event.ChangeListener() {
					public void stateChanged(javax.swing.event.ChangeEvent e) {
						refreshList();
					}
				});
		}

		return btnMapas;
	}

	/**
	 * This method initializes jPanel1
	 *
	 * @return JPanel
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();

			GridBagLayout layout2 = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();

			jPanel1.setLayout(layout2);
			c.insets = new Insets(2, 5, 2, 5);
			c.anchor = GridBagConstraints.WEST;
			c.fill = GridBagConstraints.BOTH;
			c.weightx= 1.0;
			c.weighty = 1.0;
			layout2.setConstraints(getJScrollPane(),c);
			jPanel1.add(getJScrollPane());

			c.anchor = GridBagConstraints.EAST;


			c.fill = GridBagConstraints.NONE;
			c.weightx= 0;
			c.weighty = 0;
			layout2.setConstraints(getJPanel2(),c);
			jPanel1.add(getJPanel2());

			//jPanel1.setPreferredSize(new java.awt.Dimension(430,170));
			jPanel1.setMinimumSize(new java.awt.Dimension(430,170));
			jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null,
					PluginServices.getText(this, "documentos_existentes"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		}

		return jPanel1;
	}

	/**
	 * This method initializes lstDocs
	 *
	 * @return JList
	 */
	private JList getLstDocs() {
		if (lstDocs == null) {
			lstDocs = new JList();
			lstDocs.addMouseListener(new java.awt.event.MouseAdapter() {
					public ProjectElement[] getSelecteds() {
						if (lstDocs.getSelectedIndex() < 0) {
							return new ProjectElement[0];
						}
						Object[] seleteds = lstDocs.getSelectedValues();
						ProjectElement[] returnValue = new ProjectElement[seleteds.length];
						System.arraycopy(seleteds,0,returnValue,0,seleteds.length);
						return returnValue;
					}

					public ProjectElement getItem(java.awt.event.MouseEvent e) {
						if (lstDocs.getSelectedIndex() < 0) {
							return null;
						}
						Component c = lstDocs.getComponentAt(e.getPoint());

						return null;
					}

					public void mouseClicked(java.awt.event.MouseEvent e) {
						if (e.getButton() == e.BUTTON3) {
							String docType;
							ProjectElement[] selecteds = this.getSelecteds();

							if (selecteds == null) return;
							if (btnMapas.isSelected()) {
								docType = "Map";
							} else if (btnTablas.isSelected()) {
								docType = "Table";
							} else {
								docType = "View";
							}
							DocumentPopupMenu menu = new DocumentPopupMenu(docType,this.getItem(e),(ProjectElement[])selecteds);
							if (menu.getActionsVisibles() < 1) return;
							lstDocs.add(menu);
							menu.show(e.getComponent(), e.getX(), e.getY());
							return;
						}
						if (e.getClickCount() == 2) {
							abrir();
						}
					}
				});
			lstDocs.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
					public void valueChanged(
						javax.swing.event.ListSelectionEvent e) {
						activarBotones();
					}
				});
		}

		return lstDocs;
	}

	/**
	 * This method initializes jPanel2
	 *
	 * @return JPanel
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();

			//FlowLayout layout = new FlowLayout();
			GridLayout layout = new GridLayout(5,1);
			layout.setVgap(7);

			jPanel2.setLayout(layout);
			jPanel2.add(getBtnNuevo(), null);
			jPanel2.add(getBtnAbrir(), null);
			jPanel2.add(getBtnRenombrar(), null);
			jPanel2.add(getBtnBorrar(), null);
			jPanel2.add(getBtnPropiedades(), null);
			jPanel2.setPreferredSize(new java.awt.Dimension(100, 150));
			jPanel2.setMinimumSize(new java.awt.Dimension(100, 150));
			jPanel2.setMaximumSize(new java.awt.Dimension(100, 150));
		}

		return jPanel2;
	}

	/**
	 * This method initializes btnNuevo
	 *
	 * @return JButton
	 */
	private JButton getBtnNuevo() {
		if (btnNuevo == null) {
			btnNuevo = new JButton();
			btnNuevo.setName("btnNuevo");
			btnNuevo.setText(PluginServices.getText(this, "nuevo"));
			btnNuevo.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnNuevo.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						try {
							newProjectElement();
						} catch (Exception e1) {
							NotificationManager.addError(e1.getLocalizedMessage(),
								e1);
						}
					}
				});
		}

		return btnNuevo;
	}

	/**
	 * This method initializes btnPropiedades
	 *
	 * @return JButton
	 */
	private JButton getBtnPropiedades() {
		if (btnPropiedades == null) {
			btnPropiedades = new JButton();
			btnPropiedades.setText(PluginServices.getText(this, "propiedades"));
			btnPropiedades.setName("btnPropiedades");
			btnPropiedades.setEnabled(false);
			btnPropiedades.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnPropiedades.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						propiedades();
					}
				});
		}

		return btnPropiedades;
	}

	/**
	 * This method initializes btnAbrir
	 *
	 * @return JButton
	 */
	private JButton getBtnAbrir() {
		if (btnAbrir == null) {
			btnAbrir = new JButton();
			btnAbrir.setName("btnAbrir");
			btnAbrir.setText(PluginServices.getText(this, "abrir"));
			btnAbrir.setEnabled(false);
			btnAbrir.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnAbrir.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						abrir();
					}
				});
		}

		return btnAbrir;
	}

	/**
	 * This method initializes btnBorrar
	 *
	 * @return JButton
	 */
	private JButton getBtnBorrar() {
		if (btnBorrar == null) {
			btnBorrar = new JButton();
			btnBorrar.setText(PluginServices.getText(this, "borrar"));
			btnBorrar.setName("btnBorrar");
			btnBorrar.setEnabled(false);
			btnBorrar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnBorrar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						borrar();
					}
				});
		}

		return btnBorrar;
	}

	/**
	 * This method initializes btnRenombrar
	 *
	 * @return JButton
	 */
	private JButton getBtnRenombrar() {
		if (btnRenombrar == null) {
			btnRenombrar = new JButton();
			btnRenombrar.setName("btnRenombrar");
			btnRenombrar.setText(PluginServices.getText(this, "renombrar"));
			btnRenombrar.setEnabled(false);
			btnRenombrar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnRenombrar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						renombrar();
					}
				});
		}

		return btnRenombrar;
	}

	/**
	 * This method initializes jPanel3
	 *
	 * @return JPanel
	 */
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();

			GridBagLayout layout3 = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			jPanel3.setLayout(layout3);
			c.insets = new Insets(2, 5, 0, 5);
			c.anchor = GridBagConstraints.WEST;
			c.gridx =0;
			c.gridy =0;
			layout3.setConstraints(getJLabel(),c);
			jPanel3.add(getJLabel(), null);
			c.fill = GridBagConstraints.HORIZONTAL;
			c.weightx=1.0;
			c.gridx =1;
			layout3.setConstraints(getLblNombreSesion(),c);
			jPanel3.add(getLblNombreSesion(), null);

			c.gridx =0;
			c.gridy =1;
			c.fill = GridBagConstraints.NONE;
			c.weightx=0.0;
			layout3.setConstraints(getJLabel1(),c);
			jPanel3.add(getJLabel1(), null);
			c.fill = GridBagConstraints.HORIZONTAL;
			c.weightx=1.0;
			c.gridx =1;
			layout3.setConstraints(getLblGuardado(),c);
			jPanel3.add(getLblGuardado(), null);

			c.gridx =0;
			c.gridy =2;
			c.fill = GridBagConstraints.NONE;
			c.weightx=0.0;
			layout3.setConstraints(getJLabel2(),c);
			jPanel3.add(getJLabel2(), null);

			c.fill = GridBagConstraints.HORIZONTAL;
			c.weightx=1.0;
			c.gridx =1;
			layout3.setConstraints(getLblFecha(),c);
			jPanel3.add(getLblFecha(), null);
			jPanel3.setPreferredSize(new java.awt.Dimension(430, 125));
			jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(
					null, PluginServices.getText(this, "propiedades_sesion"),
					javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
					javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			c.gridx =1;
			c.gridy =4;
			c.anchor = GridBagConstraints.EAST;
			layout3.setConstraints(getJPanel4(),c);
			jPanel3.add(getJPanel4(), null);
		}

		return jPanel3;
	}

	/**
	 * This method initializes jLabel
	 *
	 * @return JLabel
	 */
	private JLabel getJLabel() {
		if (jLabel == null) {
			jLabel = new JLabel();
			jLabel.setText(PluginServices.getText(this, "nombre_sesion") + ":");
			jLabel.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel;
	}

	/**
	 * This method initializes lblNombreSesion
	 *
	 * @return JLabel
	 */
	private JLabel getLblNombreSesion() {
		if (lblNombreSesion == null) {
			lblNombreSesion = new JLabel();
			lblNombreSesion.setText("");
			lblNombreSesion.setName("lblNombreSesion");
			lblNombreSesion.setPreferredSize(new java.awt.Dimension(200, 16));
			lblNombreSesion.setAutoscrolls(true);
		}

		return lblNombreSesion;
	}

	/**
	 * This method initializes jLabel1
	 *
	 * @return JLabel
	 */
	private JLabel getJLabel1() {
		if (jLabel1 == null) {
			jLabel1 = new JLabel();
			jLabel1.setText(PluginServices.getText(this, "guardado") + ":");
			jLabel1.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel1;
	}

	/**
	 * This method initializes lblGuardado
	 *
	 * @return JLabel
	 */
	private JLabel getLblGuardado() {
		if (lblGuardado == null) {
			lblGuardado = new JLabel();
			lblGuardado.setText("");
			lblGuardado.setPreferredSize(new java.awt.Dimension(200, 16));
			lblGuardado.setAutoscrolls(true);
		}

		return lblGuardado;
	}

	/**
	 * This method initializes jLabel2
	 *
	 * @return JLabel
	 */
	private JLabel getJLabel2() {
		if (jLabel2 == null) {
			jLabel2 = new JLabel();
			jLabel2.setText(PluginServices.getText(this, "creation_date") +
				":");
			jLabel2.setPreferredSize(new java.awt.Dimension(150, 16));
		}

		return jLabel2;
	}

	/**
	 * This method initializes lblFecha
	 *
	 * @return JLabel
	 */
	private JLabel getLblFecha() {
		if (lblFecha == null) {
			lblFecha = new JLabel();
			lblFecha.setText("");
			lblFecha.setPreferredSize(new java.awt.Dimension(200, 16));
			lblFecha.setAutoscrolls(true);
		}

		return lblFecha;
	}

	/**
	 * This method initializes btnImportar
	 *
	 * @return JButton
	 */
	private JButton getBtnImportar() {
		if (btnImportar == null) {
			btnImportar = new JButton();
			btnImportar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnImportar.setText(PluginServices.getText(this, "importar"));
			btnImportar.setName("btnImportar");
			btnImportar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			// TODO implement import
			btnImportar.setVisible(false);
		}

		return btnImportar;
	}

	/**
	 * This method initializes btnExportar
	 *
	 * @return JButton
	 */
	private JButton getBtnExportar() {
		if (btnExportar == null) {
			btnExportar = new JButton();
			btnExportar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnExportar.setText(PluginServices.getText(this, "exportar"));
			btnExportar.setName("btnExportar");
			btnExportar.setMargin(new java.awt.Insets(2, 2, 2, 2));
			// TODO implement export
			btnExportar.setVisible(false);
		}

		return btnExportar;
	}

	/**
	 * This method initializes btnEditar
	 *
	 * @return JButton
	 */
	private JButton getBtnEditar() {
		if (btnEditar == null) {
			btnEditar = new JButton();
			btnEditar.setPreferredSize(new java.awt.Dimension(80, 23));
			btnEditar.setText(PluginServices.getText(this, "propiedades"));

			btnEditar.setName("btnEditar");
			btnEditar.setMargin(new java.awt.Insets(2, 2, 2, 2));

			btnEditar.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						ProjectProperties dlg = new ProjectProperties(p);
						PluginServices.getMDIManager().addWindow(dlg);
						refreshProperties();
					}
				});
		}

		return btnEditar;
	}

	/**
	 * This method initializes jScrollPane
	 *
	 * @return JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getLstDocs());
			jScrollPane.setPreferredSize(new java.awt.Dimension(200, 100));
			jScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		}

		return jScrollPane;
	}

	/**
	 * @see com.iver.mdiApp.ui.MDIManager.SingletonWindow#getWindowModel()
	 */
	public Object getWindowModel() {
		return p;
	}

    /**
     * This method is used to get <strong>an initial</strong> ViewInfo object
     * for this Project Manager window. It is not intended to retrieve the
     * ViewInfo object in a later time.
     * <strong>Use PluginServices.getMDIManager().getViewInfo(view)
     * to retrieve the ViewInfo object at any time after the creation of the
     * object.
     *
     * @see com.iver.mdiApp.ui.MDIManager.IWindow#getWindowInfo()
     */
	public WindowInfo getWindowInfo() {
		//if (m_viewInfo==null) {
			m_viewInfo = new WindowInfo(WindowInfo.MAXIMIZABLE);
			//m_viewInfo = new ViewInfo(ViewInfo.MAXIMIZABLE+ViewInfo.RESIZABLE);
			m_viewInfo.setWidth(this.getWidth());
			m_viewInfo.setHeight(this.getHeight());
			m_viewInfo.setNormalWidth(this.getPreferredSize().width);
			m_viewInfo.setNormalHeight(this.getPreferredSize().height);

			m_viewInfo.setTitle(PluginServices.getText(this, "titulo"));
		//}
		return m_viewInfo;
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		refreshControls();
	}

	/**
	 * @see com.iver.mdiApp.ui.MDIManager.IWindow#windowActivated()
	 */
	public void viewActivated() {
	}

	/**
	 * This method initializes jPanel4
	 *
	 * @return JPanel
	 */
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new JPanel();

			jPanel4.setLayout(new FlowLayout(FlowLayout.RIGHT));
			jPanel4.setPreferredSize(new java.awt.Dimension(414, 30));
			jPanel4.setComponentOrientation(java.awt.ComponentOrientation.LEFT_TO_RIGHT);
			jPanel4.add(getBtnImportar(), null);
			jPanel4.add(getBtnExportar(), null);
			jPanel4.add(getBtnEditar(), null);
		}

		return jPanel4;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
