/*
 * Created on 02-mar-2004
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.toc;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.lang.reflect.Array;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;

import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.tree.DefaultMutableTreeNode;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrDefault;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.ClassifiableVectorial;
import com.iver.cit.gvsig.fmap.rendering.SingleSymbolLegend;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.panels.AdjustTransparencyPanel;
import com.iver.cit.gvsig.gui.thememanager.FThemeManagerWindow;
import com.iver.cit.gvsig.project.IContextMenuAction;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.utiles.extensionPoints.ExtensionPoint;
import com.iver.utiles.extensionPoints.ExtensionPoints;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;

/**
 * Realiza el cambio de color si se pulsa OK
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
class FSymbolChangeColorTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group1"; //FIXME
	}

	public int getGroupOrder() {
		return 10;
	}

	public int getOrder() {
		return 0;
	}

	public String getText() {
		return PluginServices.getText(this, "Cambio_Color");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		boolean bShow = false;
		if (isTocItemBranch(item))
		{
			FLayer lyr = getNodeLayer(item);
    		if ((lyr instanceof ClassifiableVectorial))
    		{
    		    ClassifiableVectorial lyrVect = (ClassifiableVectorial) lyr;
    		    if (lyrVect.getLegend() instanceof SingleSymbolLegend)
    		        bShow = true;
    		}
		} else if (isTocItemLeaf(item)) {
		    //bShow = true;

		}
		return bShow;
	}

	public void execute(ITocItem item, FLayer[] selectedItems) {

		boolean showDialog=false;
		if (isTocItemBranch(item)) {
			FLayer[] actives = getMapContext().getLayers().getActives();
			if (actives.length > 0) {
                for (int i=0;i<actives.length;i++){
                    if ((actives[i] instanceof ClassifiableVectorial))
                    {
                        ClassifiableVectorial lyrVect = (ClassifiableVectorial) actives[i];
                        if (lyrVect.getLegend() instanceof SingleSymbolLegend && actives[i].isAvailable() )
                        {
                            showDialog=true;
                            break;
                        }
                    }
                }
			}
		}

		if (!showDialog) return;

        Color newColor = JColorChooser.showDialog(null,
                PluginServices.getText(this, "Elegir_Color"),
                null);
        if (newColor != null)
        {
			if (isTocItemBranch(item))
			{
				//FLayer lyr = getNodeLayer();
                FLayer[] actives = getMapContext().getLayers().getActives();
                for (int i=0;i<actives.length;i++){
                    if ((actives[i] instanceof ClassifiableVectorial))
                    {
                        ClassifiableVectorial lyrVect = (ClassifiableVectorial) actives[i];
                        if (lyrVect.getLegend() instanceof SingleSymbolLegend)
                        {
                            SingleSymbolLegend leg = (SingleSymbolLegend) lyrVect.getLegend();
                            leg.getDefaultSymbol().setColor(newColor);
                        }
                    }
                }
			}
			else
			{
    	        TocItemLeaf leaf = (TocItemLeaf) item;
    	        FSymbol sym = leaf.getSymbol();
    	        sym.setColor(newColor);

			}
	        // TRUCO PARA REFRESCAR.
	        getMapContext().invalidate();

        }

	}
}
/**
 * Realiza un cambio de nombre en la capa seleccionada
 *
 * @author Vicente Caballero Navarro
 */
class ChangeNameTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group1"; //FIXME
	}

	public int getGroupOrder() {
		return 10;
	}

	public int getOrder() {
		return 1;
	}

	public String getText() {
		return PluginServices.getText(this, "cambio_nombre");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return selectedItems.length == 1;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		return isTocItemBranch(item);
	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		FLayer lyr = getNodeLayer(item);
		if (!lyr.isAvailable()) return;
		ChangeName chn=new ChangeName(lyr.getName());
		PluginServices.getMDIManager().addWindow(chn);
		lyr.setName(chn.getName());
   }
}
/**
 * Muestra el men de propiedades del tema.
 *
 * @author jmorell
 */


class FLyrVectEditPropertiesTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group2"; //FIXME
	}

	public int getGroupOrder() {
		return 20;
	}

	public int getOrder() {
		return 0;
	}

	public String getText() {
		return PluginServices.getText(this, "propiedades");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return selectedItems.length == 1;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)) {
			FLayer lyr = getNodeLayer(item);
            if ((lyr instanceof ClassifiableVectorial)) {
            	if (!((lyr instanceof FLyrVect) &&
            			!((FLyrVect)lyr).isPropertiesMenuVisible())){
            		return true;
            	}

            }
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		FLayer[] actives = selectedItems;
		FThemeManagerWindow fThemeManagerWindow;
    	if (actives.length==1) {
    		if (!actives[0].isAvailable()) return;
    		System.out.println("openPliegoThemeProperties(): Una sola capa. Abrimos el panel para la capa " + actives[0]);
    		fThemeManagerWindow = new FThemeManagerWindow();
    		try {
    			fThemeManagerWindow.setMapContext(getMapContext());
    		} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
    			e1.printStackTrace();
    		}
            //PluginServices.getMDIManager().addView(fThemeManagerWindow);
    		if (PluginServices.getMainFrame() == null) {
    			JDialog dlg = new JDialog();
    			fThemeManagerWindow.setPreferredSize(fThemeManagerWindow.getSize());
    			dlg.getContentPane().add(fThemeManagerWindow);
    			dlg.setModal(false);
    			dlg.pack();
    			dlg.show();
    		} else {
    			PluginServices.getMDIManager().addWindow(fThemeManagerWindow);
    		}
    	} else {
        	for (int i = 0; i < actives.length; i++){
        		if (actives[0].isAvailable()) {
        			System.out.println("openPliegoThemeProperties(): Muchas capas. Abrimos el panel para la capa " + actives[i]);
        			fThemeManagerWindow = new FThemeManagerWindow(actives[i], getMapContext());
        			//PluginServices.getMDIManager().addView(fThemeManagerWindow);
        			if (PluginServices.getMainFrame() == null) {
        				JDialog dlg = new JDialog();
        				fThemeManagerWindow.setPreferredSize(fThemeManagerWindow.getSize());
        				dlg.getContentPane().add(fThemeManagerWindow);
        				dlg.setModal(false);
        				dlg.pack();
        				dlg.show();
        			} else {
        				PluginServices.getMDIManager().addWindow(fThemeManagerWindow);
        			}
        		}
        	}
    	}
	}

}

class FLyrRasterAdjustTransparencyTocMenuEntry extends TocMenuEntry {
	private JMenuItem transparency;
	FLayer lyr = null;
	public void initialize(FPopupMenu m) {
		super.initialize(m);

		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    		    transparency = new JMenuItem(PluginServices.getText(this, "adjust_transparency"));
    			getMenu().add( transparency );
    			transparency.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    	        transparency.addActionListener(this);
     		}
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
        AdjustTransparencyPanel transPanel = new AdjustTransparencyPanel((FLyrDefault) lyr);
		transPanel.openJDialog();
        //getMapContext().invalidate();
	}
}

class ZoomAlTemaTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group2"; //FIXME
	}

	public int getGroupOrder() {
		return 20;
	}

	public int getOrder() {
		return 1;
	}

	public String getText() {
		return PluginServices.getText(this, "Zoom_a_la_capa");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item) && ! (selectedItems == null || selectedItems.length <= 0)) {
			return true;
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {


		// 050209, jmorell: Para que haga un zoom a un grupo de capas seleccionadas.

		if (selectedItems.length==1) {
	        try {
	        	if (!selectedItems[0].isAvailable()) return;
	        	getMapContext().getViewPort().setExtent(selectedItems[0].getFullExtent());
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		} else {
			try {
				Rectangle2D maxExtent = setMaxExtent(selectedItems);
				getMapContext().getViewPort().setExtent(maxExtent);
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		}
	}

	private Rectangle2D setMaxExtent(FLayer[] actives) throws DriverException {
		Rectangle2D extRef = actives[0].getFullExtent();
		double minXRef = extRef.getMinX();
		double maxYRef = extRef.getMaxY();
		double maxXRef = extRef.getMaxX();
		double minYRef = extRef.getMinY();
		for (int i=0;i<actives.length;i++) {
			if (actives[i].isAvailable()) {
				Rectangle2D extVar = actives[i].getFullExtent();
				double minXVar = extVar.getMinX();
				double maxYVar = extVar.getMaxY();
				double maxXVar = extVar.getMaxX();
				double minYVar = extVar.getMinY();
				if (minXVar<=minXRef) minXRef=minXVar;
				if (maxYVar>=maxYRef) maxYRef=maxYVar;
				if (maxXVar>=maxXRef) maxXRef=maxXVar;
				if (minYVar<=minYRef) minYRef=minYVar;
				extRef.setRect(minXRef, minYRef, maxXRef-minXRef, maxYRef-minYRef);
			}
		}
		return extRef;
	}
}

class EliminarCapaTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group3"; //FIXME
	}

	public int getGroupOrder() {
		return 30;
	}

	public int getOrder() {
		return 0;
	}

	public String getText() {
		return PluginServices.getText(this, "eliminar_capa");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)) {
			return true;
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
    	// 050209, jmorell: Para poder borrar todas las capas seleccionadas desde el FPopUpMenu.
		//					Es necesario pulsar Maysculas a la vez que el botn derecho.

    	FLayer[] actives = selectedItems;
    	int option=-1;
    	if (actives.length>0) {
    		option=JOptionPane.showConfirmDialog((Component)PluginServices.getMainFrame(),PluginServices.getText(this,"desea_borrar_la_capa"));
    	}
    	if (option!=JOptionPane.OK_OPTION)
    		return;
    	for (int i = actives.length-1; i>=0; i--){
        	try {
				actives[i].getParentLayer().removeLayer(actives[i]);
				FLayers lyrs=getMapContext().getLayers();
				lyrs.addLayer(actives[i]);
				actives[i].getParentLayer().removeLayer(actives[i]);

                if (actives[i] instanceof AlphanumericData){
                    Project project = ((ProjectExtension)PluginServices.getExtension(ProjectExtension.class)).getProject();
                    ProjectTable pt = project.getTable((AlphanumericData) actives[i]);

                    ArrayList tables = project.getTables();
                    for (int j = 0; j < tables.size(); j++) {
                        if (tables.get(j) == pt){
                            project.delTable(j);
                            break;
                        }
                    }

                    PluginServices.getMDIManager().closeSingletonWindow(pt);
                }

				//if (getMapContext().getLayers().getLayersCount()==0)
            	    PluginServices.getMainFrame().enableControls();
    		} catch (CancelationException e1) {
    			e1.printStackTrace();
    		}
    	}
		// 050209, jmorell: As solo borra una capa (sobre la que pulsas).
    	/*FLayer lyr = getNodeLayer();
        try {
        	getMapContext().getLayers().removeLayer(lyr);
        	if (getMapContext().getLayers().getLayersCount()==0)PluginServices.getMainFrame().enableControls();
		} catch (CancelationException e1) {
			e1.printStackTrace();
		}*/
    }
}

/**
 * Realiza una agrupacin de capas, a partir de las capas que se encuentren activas.
 *
 * @author Vicente Caballero Navarro
 */
class LayersGroupTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group4"; //FIXME
	}

	public int getGroupOrder() {
		return 40;
	}

	public int getOrder() {
		return 0;
	}

	public String getText() {
		return PluginServices.getText(this, "agrupar_capas");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return selectedItems.length > 1;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		FLayer lyr = getNodeLayer(item);

		if (selectedItems.length > 0)
			return true;

		if (!(lyr instanceof FLayers) || (lyr instanceof FLayers && lyr.getParentLayer() == null)){
			return false;
		}
		return true;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		//ITocItem tocItem = (ITocItem) getNodeUserObject();
		ChangeName changename=new ChangeName(null);
		PluginServices.getMDIManager().addWindow(changename);
		String nombre=changename.getName();

		if (nombre != null){
			if (isTocItemBranch(item)){
				FLayer[] actives = selectedItems;
				FLayers allLayers = getMapContext().getLayers();
				FLayer[] layers=new FLayer[allLayers.getLayersCount()];
				for (int i=0;i<allLayers.getLayersCount();i++) {
					layers[i]=allLayers.getLayer(i);
				}
				if (actives.length>1) {

					FLayers layerGroup=new FLayers(getMapContext(),getNodeLayer(item).getParentLayer());
					boolean first=true;
					for (int i=0;i<layers.length;i++) {
						//for (int i=0;i<actives.length;i++){
						FLayer layer=layers[i];
						if (layer.isActive()) {
							if (first) {
								getMapContext().getLayers().addLayer(i,layerGroup);
								first=false;
							}
							if(layer instanceof FLyrRaster)
								((FLyrRaster)layer).setRemoveRasterFlag(false);

							layer.getParentLayer().removeLayer(layer);
							layerGroup.addLayer(layer);
						}
					}
					layerGroup.setName(nombre);


				}

			}
			// TRUCO PARA REFRESCAR.
			getMapContext().invalidate();
		}
	}
}

/**
 * Realiza una desagrupacin de capas, a partir de las capas que se encuentren activas.
 *
 * @author Vicente Caballero Navarro
 */
class LayersUngroupTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group4"; //FIXME
	}

	public int getGroupOrder() {
		return 40;
	}

	public int getOrder() {
		return 1;
	}

	public String getText() {
		return PluginServices.getText(this, "desagrupar_capas");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (selectedItems.length == 0)
			return false;

		if (isTocItemBranch(item)){
			FLayer lyr = getNodeLayer(item);
			if (!(lyr instanceof FLayers)){
				return false;
			}
		}
		return true;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)){
			FLayers agrupa = (FLayers)getNodeLayer(item);
			FLayers parent=agrupa.getParentLayer();
			if (parent!=null){
				FLayers allLayers=getMapContext().getLayers();
				for (int i=0;i<allLayers.getLayersCount();i++) {
					FLayer layer=allLayers.getLayer(i);
					if (layer.equals(agrupa)) {
						for (int j=0;j<agrupa.getLayersCount();j++){
							parent.addLayer(i+j,agrupa.getLayer(j));
						}
						parent.removeLayer(agrupa);
					}
				}
			}
		}
		// TRUCO PARA REFRESCAR.
		getMapContext().invalidate();
	}
}
/**
 * Cambia la posicin actual del layer a la primera posicin.
 *
 * @author Vicente Caballero Navarro
 */
class FirstLayerTocMenuEntry extends AbstractTocContextMenuAction {
	public String getGroup() {
		return "group4"; //FIXME
	}

	public int getGroupOrder() {
		return 40;
	}

	public int getOrder() {
		return 2;
	}

	public String getText() {
		return PluginServices.getText(this, "colocar_delante");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)){
			return true;
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
      	if (isTocItemBranch(item)){
            if (getMapContext().getLayers().getActives().length == 1) {
                FLayer layer=((TocItemBranch)item).getLayer();
                FLayers layers=layer.getParentLayer();
                for (int i=0;i<layers.getLayersCount();i++){
                    if(layers.getLayer(i).equals(layer)){
                        layers.removeLayer(i);
                        layers.addLayer(layer);
                    }
                }
            } else if (selectedItems.length > 1) {
                FLayer[] actives =selectedItems;
                FLayers layers=actives[0].getParentLayer();
                for (int i=0;i<actives.length;i++){
                    for (int j=0;j<layers.getLayersCount();j++){
                        if(layers.getLayer(j).equals(actives[i])){
                            layers.removeLayer(j);
                            layers.addLayer(actives[i]);
                        }
                    }
                }
            }
		}
		// TRUCO PARA REFRESCAR.
        getMapContext().invalidate();
	}
}
/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de zoom a un
 * pixel.
 */
class ZoomPixelTocMenuEntry extends TocMenuEntry {
	public static final int ZOOM_TO_IMAGE_CENTER = 0x1;
	public static final int ZOOM_TO_VIEW_CENTER = 0x2;
	private JMenuItem properties;
	FLayer lyr = null;
	public int zoomType = ZOOM_TO_VIEW_CENTER;


	public void initialize(FPopupMenu m) {
		super.initialize(m);

		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "Zoom_pixel"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);

    			properties.addActionListener(this);
     		}
		}
	}

	public void actionPerformed(ActionEvent e) {

		FLayer[] actives = getMapContext().getLayers().getActives();
		if (actives.length==1) {
	        try {
	        	Rectangle2D r = actives[0].getFullExtent();

	        	ArrayList attr = ((FLyrRaster)actives[0]).getSource().getAttributes();
	        	int width = 0, height = 0;
	       		for (int i=0; i<attr.size(); i++) {
	    			Object[] a = (Object []) attr.get(i);
	    			if (a[0].toString().equals("Width"))
	    				width = ((Integer)a[1]).intValue();
	    			if (a[0].toString().equals("Height"))
	    				height = ((Integer)a[1]).intValue();
	    		}
	        	ViewPort v = getMapContext().getViewPort();

	        	if (zoomType == ZOOM_TO_IMAGE_CENTER) {
		        	//A centro de imagen
		        	/*double centroX = width/2;
					double centroY = height/2;
					int minX = (int)Math.round(centroX - (v.getImageWidth()/2));
					int minY = (int)Math.round(centroY - (v.getImageHeight()/2));
					int maxX = (int)Math.round(minX + v.getImageWidth());
					int maxY = (int)Math.round(minY + v.getImageHeight());
					double wcOriginWidth = r.getMaxX() - r.getMinX();
					double wcOriginHeight = r.getMaxY() - r.getMinY();

					double wcDstMinX = r.getMinX()+((minX*wcOriginWidth)/width);
					double wcDstMinY = r.getMinY()+((minY*wcOriginHeight)/height);
					double wcDstMaxX = r.getMinX()+((maxX*wcOriginWidth)/width);
					double wcDstMaxY = r.getMinY()+((maxY*wcOriginHeight)/height);

					double wcDstWidth = wcDstMaxX - wcDstMinX;
					double wcDstHeight = wcDstMaxY - wcDstMinY;*/
	        	} else if (zoomType == ZOOM_TO_VIEW_CENTER) {
		        	//A centro de vista
		        	double wcOriginCenterX = v.getExtent().getMinX()+((v.getExtent().getMaxX()-v.getExtent().getMinX())/2);
		        	double wcOriginCenterY = v.getExtent().getMinY()+((v.getExtent().getMaxY()-v.getExtent().getMinY())/2);

		        	//Hallamos la relacin entre el pixel y las WC a partir de la imagen de la capa
					double relacionPixelWcWidth =  (r.getMaxX() - r.getMinX())/width;
					double relacionPixelWcHeight = (r.getMaxY() - r.getMinY())/height;
					//double desplazamientoX = r.getMinX();
					//double desplazamientoY = r.getMinY();

					double wcOriginX = wcOriginCenterX - ((v.getImageWidth()*relacionPixelWcWidth)/2);
					double wcOriginY = wcOriginCenterY - ((v.getImageHeight()*relacionPixelWcHeight)/2);

					double wcDstMinX = wcOriginX;
					double wcDstMinY = wcOriginY;
					double wcDstMaxX = wcDstMinX + (v.getImageWidth()*relacionPixelWcWidth);
					double wcDstMaxY = wcDstMinY + (v.getImageHeight()*relacionPixelWcHeight);

					double wcDstWidth = wcDstMaxX - wcDstMinX;
					double wcDstHeight = wcDstMaxY - wcDstMinY;

		        	r = new Rectangle2D.Double(wcDstMinX, wcDstMinY, wcDstWidth, wcDstHeight);
		        	getMapContext().getViewPort().setExtent(r);
	        	}
			} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
				e1.printStackTrace();
			}
		}
		//View vista = (View) PluginServices.getMDIManager().getActiveView();
		//MapControl mapCtrl = vista.getMapControl();
	}
}

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de zoom a un
 * pixel centrado en el cursor.
 */
class ZoomPixelCursorTocMenuEntry extends AbstractTocContextMenuAction {
	public static final int ZOOM_TO_IMAGE_CENTER = 0x1;
	public static final int ZOOM_TO_VIEW_CENTER = 0x2;
	FLayer lyr = null;
	public int zoomType = ZOOM_TO_VIEW_CENTER;

	public String getGroup() {
		return "group2"; //FIXME
	}

	public int getGroupOrder() {
		return 20;
	}

	public int getOrder() {
		return 2;
	}

	public String getText() {
		return PluginServices.getText(this, "Zoom_pixel");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return true;
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)) {
			return getNodeLayer(item) instanceof FLyrRaster;
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		if (selectedItems.length==1) {
	        lyr = getNodeLayer(item);
	    	View vista = (View) PluginServices.getMDIManager().getActiveWindow();
	    	MapControl mapCtrl = vista.getMapControl();
	    	mapCtrl.setTool("zoom_pixel_cursor");

		}
	}
}

/**
 *
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 * Entrada de men para la activacin de la funcionalidad de salvar
 * a raster una parte de la vista.
 */
class SaveRasterTocMenuEntry extends TocMenuEntry{
	private JMenuItem properties;
	FLayer lyr = null;

	public void initialize(FPopupMenu m) {
		super.initialize(m);

		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "salvar_raster"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			properties.addActionListener(this);
     		}
		}
	}

	public void actionPerformed(ActionEvent e) {
		lyr = getNodeLayer();
		View vista = (View) PluginServices.getMDIManager().getActiveWindow();
		MapControl mapCtrl = vista.getMapControl();
		mapCtrl.setTool("saveRaster");
	}
}

/**
*
* @author Jose Manuel Viv (Chema)
*
* Entrada de men para recargar una capa.
*/
class ReloadLayerTocMenuEntry extends AbstractTocContextMenuAction{
	public String getGroup() {
		return "group3"; //FIXME
	}

	public int getGroupOrder() {
		return 30;
	}

	public int getOrder() {
		return 1;
	}

	public String getText() {
		return PluginServices.getText(this, "recargar");
	}

	public boolean isEnabled(ITocItem item, FLayer[] selectedItems) {
		return !getNodeLayer(item).isAvailable();
	}

	public boolean isVisible(ITocItem item, FLayer[] selectedItems) {
		if (isTocItemBranch(item)) {
			return true;
		}
		return false;

	}


	public void execute(ITocItem item, FLayer[] selectedItems) {
		try {
			getNodeLayer(item).reload();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}



class OldTocContextMenuAction extends AbstractTocContextMenuAction {
	private TocMenuEntry entry;

	public int getGroupOrder() {
		return 999;
	}

	public String getGroup() {
		return "zzzOldMenus";
	}

	public void execute(ITocItem item, FLayer[] selectedItems) {
		return; //No se llega a ejecutar nunca
	}

	public String getText() {
		return null; //No se llega a ejecutar nunca
	}

	public void initializeElement(FPopupMenu menu) {
		this.entry.initialize(menu);
	}

	public void setEntry(TocMenuEntry entry) {
		this.entry = entry;
	}

	public TocMenuEntry getEntry() {
		return this.entry;
	}
}


/**
 * Menu de botn derecho para el TOC.
 * Se pueden aadir entradas facilmente desde una extensin,
 * creando una clase derivando de TocMenuEntry, y aadiendola en
 * esttico (o en tiempo de carga de la extensin) a FPopupMenu.
 * (Las entradas actuales estn hechas de esa manera).
 *
 * @author vcn To change the template for this generated type comment go to
 *         Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and
 *         Comments
 */

public class FPopupMenu extends JPopupMenu {
	//private static ArrayList menuEntrys = new ArrayList();
    private DefaultMutableTreeNode nodo;
    protected MapContext mapContext;
    private ExtensionPoint extensionPoint;
    private FLayer[] selecteds;
    //private JMenuItem capa;
    // Lo de fijar la fuente es porque en linux se vea mal si no se fija.
    // TODO: Esto no funcionar para idiomas como el chino. Hay que cambiarlo.
    public final static Font theFont = new Font("SansSerif", Font.PLAIN, 10);

    public static void registerExtensionPoint() {
    	ExtensionPoints extensionPoints = ExtensionPointsSingleton.getInstance();
    	extensionPoints.add("View_TocActions","FSymbolChangeColor",new FSymbolChangeColorTocMenuEntry());
    	extensionPoints.add("View_TocActions","ChangeName",new ChangeNameTocMenuEntry());
    	extensionPoints.add("View_TocActions","FLyrVectEditProperties",new FLyrVectEditPropertiesTocMenuEntry());
    	extensionPoints.add("View_TocActions","ZoomAlTema",new ZoomAlTemaTocMenuEntry());
    	extensionPoints.add("View_TocActions","ZoomPixelCursor",new ZoomPixelCursorTocMenuEntry());
    	extensionPoints.add("View_TocActions","EliminarCapa",new EliminarCapaTocMenuEntry());
    	extensionPoints.add("View_TocActions","ReloadLayer",new ReloadLayerTocMenuEntry());
    	extensionPoints.add("View_TocActions","LayersGroup",new LayersGroupTocMenuEntry());
       	extensionPoints.add("View_TocActions","LayersUngroup",new LayersUngroupTocMenuEntry());
       	extensionPoints.add("View_TocActions","FirstLayer",new FirstLayerTocMenuEntry());


    }
    static {
    	/* Cambiados
    	FPopupMenu.addEntry(new FSymbolChangeColorTocMenuEntry());
    	FPopupMenu.addEntry(new ChangeNameTocMenuEntry());
    	FPopupMenu.addEntry(new FLyrVectEditPropertiesTocMenuEntry());


    	FPopupMenu.addEntry(new ZoomAlTemaTocMenuEntry());
    	FPopupMenu.addEntry(new ZoomPixelCursorTocMenuEntry());


    	FPopupMenu.addEntry(new EliminarCapaTocMenuEntry());
    	FPopupMenu.addEntry(new ReloadLayerTocMenuEntry());

    	FPopupMenu.addEntry(new LayersGroupTocMenuEntry());
        FPopupMenu.addEntry(new LayersUngroupTocMenuEntry());


    	FPopupMenu.addEntry(new FirstLayerTocMenuEntry());
    	    	    		    	*/
    	FPopupMenu.addEntry(new FLyrRasterAdjustPropertiesTocMenuEntry());
    }


    /**
     * @deprecated
     */
    public static void addEntry(TocMenuEntry entry) {

    	ExtensionPoints extensionPoints = ExtensionPointsSingleton.getInstance();

    	OldTocContextMenuAction action = new OldTocContextMenuAction();
    	action.setEntry(entry);
    	String name =entry.getClass().getName();
    	name = name.substring(name.lastIndexOf(".")+1);
    	extensionPoints.add("View_TocActions",name,action);

    }

    /**
     * @deprecated
     */
    public static Object getEntry(String className) {
    	ExtensionPoints extensionPoints = ExtensionPointsSingleton.getInstance();
    	OldTocContextMenuAction action = null;
    	try {
			action = (OldTocContextMenuAction)((ExtensionPoint)extensionPoints.get("View_TocActions")).create(className);
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassCastException e) {
			action = null;
		}
    	if (action != null) {
    		return action.getEntry();
    	} else {
    		return null;
    	}
    }

    /**
     * Creates a new FPopupMenu object.
     *
     * @param nodo DOCUMENT ME!
     * @param vista DOCUMENT ME!
     */
    public FPopupMenu(MapContext mc, DefaultMutableTreeNode node) {
        super();
        this.initialize(mc,node);
    }

    private void initialize(MapContext mc, DefaultMutableTreeNode node) {
        this.mapContext = mc;
        this.nodo = node;

        //salir = new MenuItem("Salir");
		this.extensionPoint = (ExtensionPoint)ExtensionPointsSingleton.getInstance().get("View_TocActions");
		this.selecteds = this.mapContext.getLayers().getActives();

		IContextMenuAction[] actions = this.getActionList();

		this.createMenuElements(actions);

		this.loadOldStileOptions();


    }

    public MapContext getMapContext() { return mapContext; }

	public ITocItem getNodeUserObject() {
		if (nodo == null) return null;
		return (ITocItem)nodo.getUserObject();
	}

	public DefaultMutableTreeNode getNode() {
		return this.nodo;
	}

    private IContextMenuAction[] getActionList() {
		ArrayList actionArrayList = new ArrayList();
		Iterator iter = this.extensionPoint.keySet().iterator();
		AbstractTocContextMenuAction action;
		while (iter.hasNext()) {
			action = null;
			try {
				action = (AbstractTocContextMenuAction)this.extensionPoint.create((String)iter.next());
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if (action != null && !(action instanceof OldTocContextMenuAction)) {
				action.setMapContext(this.mapContext);
				if (action.isVisible((ITocItem)this.getNodeUserObject(),this.selecteds)) {
					actionArrayList.add(action);
				}
			}

		}
		IContextMenuAction[] result = (IContextMenuAction[])Array.newInstance(IContextMenuAction.class,actionArrayList.size());
		System.arraycopy(actionArrayList.toArray(),0,result,0,actionArrayList.size());
		Arrays.sort(result,new CompareAction());

		return result;
    }

	public class CompareAction implements Comparator{
		public int compare(Object o1, Object o2) {
			return this.compare((IContextMenuAction)o1,(IContextMenuAction)o2);
		}

		public int compare(IContextMenuAction o1, IContextMenuAction o2) {
			//FIXME: flata formatear los enteros!!!!
			NumberFormat formater = NumberFormat.getInstance();
			formater.setMinimumIntegerDigits(3);
			String key1= ""+formater.format(o1.getGroupOrder())+o1.getGroup()+formater.format(o1.getOrder());
			String key2= ""+formater.format(o2.getGroupOrder())+o2.getGroup()+formater.format(o2.getOrder());
			return key1.compareTo(key2);
		}
	}

	private void createMenuElements(IContextMenuAction[] actions) {
		String group = null;
		for (int i=0;i < actions.length;i++) {
			IContextMenuAction action = actions[i];
			MenuItem item = new MenuItem(action.getText(),action);
			item.setFont(theFont);
			item.setEnabled(action.isEnabled(this.getNodeUserObject(),this.selecteds));
			if (!action.getGroup().equals(group)) {
				if (group != null) this.addSeparator();
				group = action.getGroup();
			}
			this.add(item);
		}

	}


	public class MenuItem extends JMenuItem implements ActionListener{
		private IContextMenuAction action;
		public MenuItem(String text,IContextMenuAction documentAction) {
			super(text);
			this.action = documentAction;
			String tip = this.action.getDescription();
			if (tip != null && tip.length() > 0) {
				this.setToolTipText(tip);
			}
			this.addActionListener(this);
		}

		public void actionPerformed(ActionEvent e) {
			this.action.execute(FPopupMenu.this.getNodeUserObject(), FPopupMenu.this.selecteds);
		}
	}

	private void loadOldStileOptions() {
		boolean first = true;
		Iterator iter = this.extensionPoint.keySet().iterator();
		AbstractTocContextMenuAction action;
		while (iter.hasNext()) {
			action = null;
			try {
				action = (AbstractTocContextMenuAction)this.extensionPoint.create((String)iter.next());
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if (action != null && (action instanceof OldTocContextMenuAction)) {
				if (first) {
					this.addSeparator();
					first = false;
				}
				action.setMapContext(this.mapContext);
				((OldTocContextMenuAction)action).initializeElement(this);
			}
		}
		//comprobamos si el ultimo elemento es un seprardor
		if (this.getComponent(this.getComponentCount()-1) instanceof Separator) {
			//Si lo es lo eliminamos
			this.remove(this.getComponentCount()-1);
		}


	}


}
