/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.TreeMap;

import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.SourceInfo;
import com.hardcode.gdbms.engine.data.db.DBSourceInfo;
import com.hardcode.gdbms.engine.data.db.DBTableSourceInfo;
import com.hardcode.gdbms.engine.data.file.FileSourceInfo;
import com.hardcode.gdbms.engine.data.object.ObjectSourceInfo;
import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.SingletonWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.operations.selection.LinkSelectionListener;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.ProjectWindow;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.utiles.NotExistInXMLEntity;
import com.iver.utiles.PostProcessSupport;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;


/**
 * Clase que representa un proyecto de openSIG
 *
 * @author Fernando Gonzlez Corts
 */
public class Project implements Serializable, PropertyChangeListener {
	public static String VERSION = "1.0";
	static private IProjection defaultProjection = CRSFactory.getCRS(
			"EPSG:23030");
	private PropertyChangeSupport change;
	boolean modified = false;
	private String name;
	//private String path;
	private String creationDate;
	private String modificationDate;
	private String owner;
	private String comments;
	private Color selectionColor = new Color(255, 255, 0);
	/**
	 * Stores the list of ProjectViews in the project
	 */
	private ArrayList views = new ArrayList();
	private ArrayList tables = new ArrayList();
	private ArrayList maps = new ArrayList();
	private ArrayList extents = new ArrayList();
	private int signatureAtStartup;
	// used to restore the position of the Project Manager window when loading a project from file
	private WindowInfo seedProjectWindow = null;
	private IProjection projection;


	/**
	 * Creates a new Project object.
	 */
	public Project() {
		change = new PropertyChangeSupport(this);

		//        change.addPropertyChangeListener(this);
		creationDate = DateFormat.getDateInstance().format(new Date());
		modificationDate = creationDate;
//		signatureAtStartup = computeSignature();

		/*                LayerFactory.setDriversPath(PluginServices.getPluginServices(this)
		   .getPluginDirectory()
		   .getAbsolutePath() +
		   File.separator + "drivers");
		 */
	}

	/**
	 * Obtiene la fecha de creacin del proyecto
	 *
	 * @return
	 */
	public String getCreationDate() {
		return creationDate;
	}

	/**
	 * Obtiene el nombre del proyecto
	 *
	 * @return
	 */
	public String getName() {
		return name;
	}

//	/**
//	 * Obtiene la ruta completa del fichero donde se guardo por ltima vez el
//	 * proyecto
//	 *
//	 * @return
//	 */
//	public String getPath() {
//		return path;
//	}

	/**
	 * Asigna la fecha de creacin del proyecto. Este mtodo tiene sentido slo
	 * por que al recuperar la fecha del XML hay que asignarla al objeto
	 * proyecto de alguna manera. La fecha se asigna en el constructor y no se
	 * debera de modificar nunca
	 *
	 * @param string
	 */
	public void setCreationDate(String string) {
		creationDate = string;
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Aade un mapa al proyecto
	 *
	 * @param m
	 */
	public void addMap(ProjectMap m) {
		maps.add(m);
		m.addPropertyChangeListener(this);
		modified = true;
		change.firePropertyChange("", null, null);
		m.setProject(this, maps.size() - 1);
	}

	/**
	 * Elimina un mapa del proyecto
	 *
	 * @param i indice del mapa
	 */
	public void delMap(int i) {
		maps.remove(i);
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Establece el nombre del proyecto
	 *
	 * @param string
	 */
	public void setName(String string) {
		name = string;
		modified = true;
		change.firePropertyChange("", null, null);
	}

//	/**
//	 * establece la ruta completa de donde se encuentra guardado el proyecto
//	 *
//	 * @param string
//	 */
//	public void setPath(String string) {
//		path = string;
//		modified = true;
//		change.firePropertyChange("", null, null);
//	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param co DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public ProjectTable getTable(AlphanumericData co) {
		/**
		 * Como las tablas se pueden aadir cuando se pincha en "ver tabla" de
		 * una capa, se puede intentar aadir dos veces la misma tabla
		 */
		for (int i = 0; i < tables.size(); i++) {
			if (((ProjectTable) tables.get(i)).getAssociatedTable() == co) {
				return (ProjectTable) tables.get(i);
			}
		}

		return null;
	}

	public ProjectTable getTable(String name) {
		/**
		 * Como las tablas se pueden aadir cuando se pincha en "ver tabla" de
		 * una capa, se puede intentar aadir dos veces la misma tabla
		 */
		for (int i = 0; i < tables.size(); i++) {
			if (((ProjectTable) tables.get(i)).getName().equals(name)) {
				return (ProjectTable) tables.get(i);
			}
		}

		return null;
	}

	/**
	 * Aade una tabla al proyecto
	 *
	 * @param t
	 */
	public void addTable(ProjectTable t) {
		tables.add(t);
		t.addPropertyChangeListener(this);
		modified = true;
		change.firePropertyChange("", null, null);
		t.setProject(this, tables.size() - 1);
	}

	/**
	 * Elimina una tabla del proyecto
	 *
	 * @param i indice de la tabla
	 */
	public void delTable(int i) {
		tables.remove(i);
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Opens a window for provided ProjectTable, which has been previously created,
	 * and it's present in the project manager. 
	 *
	 * @return The window created to open the table
	 */
	public static IWindow openTable(ProjectTable pt) {
		com.iver.cit.gvsig.gui.Table tableWindow = new com.iver.cit.gvsig.gui.Table();
		tableWindow.setModel(pt);
		pt.setAndamiWindow(tableWindow);
		PluginServices.getMDIManager().addWindow(tableWindow);
		return tableWindow;
	}
	
	/**
	 * Opens the provided ProjectView, which has been previously created, and
	 * it's present in the project manager. 
	 *	 
	 * @return The window created to open the view
	 */
	public static IWindow openView(ProjectView pv) {
		com.iver.cit.gvsig.gui.View viewWindow = new com.iver.cit.gvsig.gui.View();
		viewWindow.setModel(pv);
		PluginServices.getMDIManager().addWindow(viewWindow);
		return viewWindow;
	}
	
	/**
	 * Opens a window for provided ProjectMap, which has been previously created,
	 * and it's present in the project manager. 
	 * 
	 * @return The window created to open the map
	 */
	public static IWindow openMap(ProjectMap pm) {
		pm.getModel().setProjectMap(pm);
		PluginServices.getMDIManager().addWindow(pm.getModel());

		return pm.getModel();
	}

	/**
	 * Aade una vista al proyecto
	 *
	 * @param v
	 */
	public void addView(ProjectView v) {
		views.add(v);
		v.addPropertyChangeListener(this);
		modified = true;
		change.firePropertyChange("", null, null);
		v.setProject(this, views.size() - 1);
	}

	/**
	 * Elimina una tabla del proyecto
	 *
	 * @param i indice del proyecto
	 */
	public void delView(int k) {
		ProjectView pv=(ProjectView)views.get(k);
		FLayers layers=pv.getMapContext().getLayers();

		for (int i = layers.getLayersCount()-1; i>=0; i--){
        	try {
        		if (layers.getLayer(i) instanceof AlphanumericData){
                    Project project = ((ProjectExtension)PluginServices.getExtension(ProjectExtension.class)).getProject();
                    ProjectTable pt = project.getTable((AlphanumericData) layers.getLayer(i));

                    ArrayList tables = project.getTables();
                    for (int j = 0; j < tables.size(); j++) {
                        if (tables.get(j) == pt){
                            project.delTable(j);
                            break;
                        }
                    }

                    PluginServices.getMDIManager().closeSingletonWindow(pt);
                }
                layers.getLayer(i).getParentLayer().removeLayer(layers.getLayer(i));
			} catch (CancelationException e1) {
    			e1.printStackTrace();
    		}
    	}
		views.remove(k);
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Devuelve true si el proyecto (o alguna tabla, vista o mapa que contiene)
	 * fue modificado
	 *
	 * @return
	 */
	public boolean isModified() {
		if (this.getMaps().size()==0 && this.getViews().size()==0 && this.getTables().size() == 0){
			return false;
		}
		return true;
		///return modified;    TODO El atributo modified solo detecta cuando se elimina o aade una vista,
		///mapa o tabla pero no cuando se modifican.
	}

	/**
	 * Obtiene los comentarios
	 *
	 * @return
	 */
	public String getComments() {
		return comments;
	}

	/**
	 * Obtiene la fecha de la ltima modificacin
	 *
	 * @return
	 */
	public String getModificationDate() {
		return modificationDate;
	}

	/**
	 * Obtiene el propietario del proyecto
	 *
	 * @return
	 */
	public String getOwner() {
		return owner;
	}

	/**
	 * Establece una cadena como comentarios al proyecto
	 *
	 * @param string
	 */
	public void setComments(String string) {
		comments = string;
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Establece la fecha de la ltima modificacin
	 *
	 * @param string
	 */
	public void setModificationDate(String string) {
		modificationDate = string;
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Establece el propietario del proyecto
	 *
	 * @param string
	 */
	public void setOwner(String string) {
		owner = string;
		modified = true;
		change.firePropertyChange("", null, null);
	}

	/**
	 * Establece el flag de modificado del proyecto
	 *
	 * @param b
	 */
	public void setModified(boolean b) {
		modified = b;
	}

	/**
	 * Obtiene el color de seleccin que se usar en el proyecto
	 *
	 * @return
	 */
	public Color getSelectionColor() {
		return selectionColor;
	}

	/**
	 * Establece el color de seleccin
	 *
	 * @param color
	 */
	public void setSelectionColor(Color color) {
		selectionColor = color;
		FSymbol.setSelectionColor(color);
		modified = true;
		change.firePropertyChange("selectionColor", null, color);
	}

	/**
	 * Obtiene el color como un entero para su serializacin a XML
	 *
	 * @return
	 */
	public String getColor() {
		return StringUtilities.color2String(selectionColor);
	}

	/**
	 * Mtodo invocado al recuperar de XML para establecer el color de
	 * seleccion del proyecto
	 *
	 * @param color Entero que representa un color
	 */
	public void setColor(String color) {
		modified = true;
		selectionColor = StringUtilities.string2Color(color);
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		this.modified = true;
		change.firePropertyChange(evt);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg1
	 */
	public void addExtent(ProjectExtent arg1) {
		extents.add(arg1);
		modified = true;
		change.firePropertyChange("addExtent", null, null);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public Object removeExtent(int arg0) {
		modified = true;
		change.firePropertyChange("delExtent", null, null);

		return extents.remove(arg0);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public ProjectExtent[] getExtents() {
		return (ProjectExtent[]) extents.toArray(new ProjectExtent[0]);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 */
	public synchronized void addPropertyChangeListener(
		PropertyChangeListener arg0) {
		change.addPropertyChangeListener(arg0);
	}

	/**
	 * Get the list of maps in the project.
	 *
	 * @return
	 */
	public ArrayList getMaps() {
		return maps;
	}

	/**
	 * Get the list of tables in the project.
	 *
	 * @return
	 */
	public ArrayList getTables() {
		return tables;
	}

	/**
	 * Get the list of ProjectViews in the project.
	 *
	 * @return
	 */
	public ArrayList getViews() {
		return views;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws DriverException
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws DriverException, XMLException {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className", this.getClass().getName());
		xml.putProperty("VERSION", VERSION);
		xml.putProperty("comments", comments);
		xml.putProperty("creationDate", creationDate);

		int numExtents = extents.size();

		/*double[] xs = new double[size];
		   double[] ys = new double[size];
		   double[] ws = new double[size];
		   double[] hs = new double[size];
		 */
		xml.putProperty("numExtents", numExtents);

		for (int i = 0; i < numExtents; i++) {
			/*        Rectangle2D rect = (Rectangle2D) extents.get(i);
			   xs[i] = rect.getX();
			   ys[i] = rect.getY();
			   ws[i] = rect.getWidth();
			   hs[i] = rect.getHeight();
			 */
			xml.addChild(((ProjectExtent) extents.get(i)).getXMLEntity());
		}

        // NUEVO: ESTO ESTA EN PRUEBAS. SIRVE PARA
        // BORRAR LAS REFERENCIAS A DATASOURCES QUE HEMOS
        // BORRADO. Hay que probar a borrarlos cuando se
        // borra una tabla y cuando se borra una capa.
        cleanBadReferences();
		SourceInfo[] infos = LayerFactory.getDataSourceFactory().getDriverInfos();
		xml.putProperty("data-source-count", infos.length);

		for (int i = 0; i < infos.length; i++) {
			SourceInfo di = infos[i];
			XMLEntity child = this.getSourceInfoXMLEntity(di);
			xml.addChild(child);
		}

		/*
		   xml.putProperty("extentsX", xs);
		   xml.putProperty("extentsY", ys);
		   xml.putProperty("extentsW", ws);
		   xml.putProperty("extentsH", hs);
		 */

		int numViews=0;
		for (int i = 0; i < views.size(); i++) {
			try {
				XMLEntity xmlchild=((ProjectView) views.get(i)).getXMLEntity();
				xml.addChild(xmlchild);
				numViews++;
			} catch (SaveException e) {
				e.showError();
			}
		}
		xml.putProperty("numViews", numViews);

		int numMaps=0;
		for (int i = 0; i < maps.size(); i++) {
			try {
				XMLEntity xmlchild=((ProjectMap) maps.get(i)).getXMLEntity();
				xml.addChild(xmlchild);
				numMaps++;
			} catch (SaveException e) {
				e.showError();
			}
		}
		xml.putProperty("numMaps", numMaps);
		xml.putProperty("modificationDate", modificationDate);
		xml.putProperty("modified", modified);
		xml.putProperty("name", name);
		xml.putProperty("owner", owner);
		//xml.putProperty("path", path);
		xml.putProperty("selectionColor",
			StringUtilities.color2String(selectionColor));

		int numTables=0;
		for (int i = 0; i < tables.size(); i++) {
			try {
				XMLEntity xmlchild=((ProjectTable) tables.get(i)).getXMLEntity();
				xml.addChild(xmlchild);
				numTables++;
			} catch (SaveException e) {

				e.showError();
			}
		}
		xml.putProperty("numTables", numTables);
		xml.putProperty("projection", defaultProjection.getAbrev());

		saveWindowProperties(xml);
		
		return xml;
	}
	
	private void saveWindowProperties(XMLEntity xml) {
		XMLEntity propertyList = new XMLEntity();
		
		propertyList.setName("AndamiPersistence");
		propertyList.putProperty("className", Project.class.getName());
		
		boolean projectWindowSaved = false;
		
		IWindow[] windowList = PluginServices.getMDIManager().getOrderedWindows();
		WindowInfo wi;
		XMLEntity windowProperties;
		for (int winIndex=windowList.length-1; winIndex>=0; winIndex--) {
			wi = PluginServices.getMDIManager().getWindowInfo(windowList[winIndex]);
			if (wi!=null && wi.checkPersistence()) {
				if (windowList[winIndex] instanceof View) {
					View viewWindow = (View) windowList[winIndex];
					windowProperties = wi.getXMLEntity();
					windowProperties.putProperty("documentType", "ProjectView");
					windowProperties.putProperty("documentName", viewWindow.getModel().getName());
					windowProperties.putProperty("zPosition", winIndex);
					propertyList.addChild(windowProperties);
				}
				else if (windowList[winIndex] instanceof Table) {
					Table tableWindow = (Table) windowList[winIndex];
					windowProperties = wi.getXMLEntity();
					windowProperties.putProperty("documentType", "ProjectTable");
					windowProperties.putProperty("documentName", tableWindow.getModel().getName());
					windowProperties.putProperty("zPosition", winIndex);
					propertyList.addChild(windowProperties);
				}
				else if (windowList[winIndex] instanceof Layout) {
					Layout layoutWindow = (Layout) windowList[winIndex];
					windowProperties = wi.getXMLEntity();
					windowProperties.putProperty("documentType", "ProjectMap");
					windowProperties.putProperty("documentName", layoutWindow.getName());
					windowProperties.putProperty("zPosition", winIndex);
					propertyList.addChild(windowProperties);
				}
				else if (windowList[winIndex] instanceof ProjectWindow) {
					projectWindowSaved = true;
					windowProperties = wi.getXMLEntity();	
					windowProperties.putProperty("className", "com.iver.cit.gvsig.project.document.gui.ProjectWindow");
					windowProperties.putProperty("zPosition", winIndex);
					propertyList.addChild(windowProperties);
				}
			}
		}
		
		if (projectWindowSaved == false) {
			// If the Project Manager was closed, it was not in the
			// previous window list. Save it now
			ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
			
			if (pe!=null) {
				IWindow projectWindow = pe.getProjectWindow();
				wi = PluginServices.getMDIManager().getWindowInfo(projectWindow);
				if (wi!=null && wi.checkPersistence()) {
					windowProperties = wi.getXMLEntity();
					if (windowProperties!=null) {
						windowProperties.putProperty("className", "com.iver.cit.gvsig.project.document.gui.ProjectWindow");
						propertyList.addChild(windowProperties);
					}
				}
			}
		}
		
		xml.addChild(propertyList);
	}
	
	/**
	 * Restores the position ans size of windows from projects
	 * (gvSIG > 1.0rc1). It also restores the order of the windows
	 * (which windows are on the foreground).
	 *  
	 * @param xml An XMLEntity object containing the properties of the
	 * windows
	 */
	private void restoreWindowProperties(XMLEntity xml) {
		XMLEntity child;
		int childNumb;
		
		boolean projectWindowRestored = false;
		
		if (xml.contains("className") // the current window persistence method (>1.0rc2)
				&& xml.getStringProperty("className").equals(Project.class.getName())
				&& xml.contains("name")
				&& xml.getStringProperty("name").equals("AndamiPersistence")) {
			
			// order the window properties before restoring them, so that we also
			// restore the zPosition
			TreeMap orderedProperties = new TreeMap();
			int maximum = 1;
			for (childNumb=xml.getChildrenCount()-1; childNumb>=0; childNumb--) {
				child = xml.getChild(childNumb);
				if (child.contains("zPosition")) {
					orderedProperties.put(new Integer(-child.getIntProperty("zPosition")), child); // reverse the order, so that we add the background windows first
				}
				else {
					orderedProperties.put(new Integer(maximum++), child); // the windows without zPosition will be on the foreground
				}
			}
			
			Iterator propertiesIterator = orderedProperties.values().iterator();
			while (propertiesIterator.hasNext()) {
				child = (XMLEntity) propertiesIterator.next();
				 if (child.contains("documentType") // restore the position of the layout windows
							&& child.getStringProperty("documentType").equals("ProjectView")
							&& child.contains("name")
							&& child.getStringProperty("name").equals("ViewInfoProperties")) {
						boolean isClosed = true;
						if (child.contains("isClosed"))
							isClosed = child.getBooleanProperty("isClosed");
						if (isClosed==false) {
							WindowInfo windowProps = WindowInfo.createFromXMLEntity(child);
							String documentName = child.getStringProperty("documentName");
							ProjectView pv = this.getViewByName(documentName);
							IWindow win = Project.openView(pv);
							PluginServices.getMDIManager().changeWindowInfo(win, windowProps);
						}
				}
				else if (child.contains("className") // restore the position of the project manager window
						&& child.getStringProperty("className").equals("com.iver.cit.gvsig.project.document.gui.ProjectWindow")
						&& child.contains("name")
						&& child.getStringProperty("name").equals("ViewInfoProperties")) {
					this.seedProjectWindow = WindowInfo.createFromXMLEntity(child);

					ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
					
					if (pe!=null) {
						pe.setProject(this);
						pe.showProjectWindow();
						projectWindowRestored = true;
					}
				}
				else if (child.contains("documentType") // restore the position of the layout windows
						&& child.getStringProperty("documentType").equals("ProjectMap")
						&& child.contains("name")
						&& child.getStringProperty("name").equals("ViewInfoProperties")) {
					boolean isClosed = true;
					if (child.contains("isClosed"))
						isClosed = child.getBooleanProperty("isClosed");
					if (isClosed==false) {
						WindowInfo windowProps = WindowInfo.createFromXMLEntity(child);
						String documentName = child.getStringProperty("documentName");
						ProjectMap pm = this.getLayoutByName(documentName);
						IWindow win = Project.openMap(pm);
						PluginServices.getMDIManager().changeWindowInfo(win, windowProps);
					}
				}
				else if (child.contains("documentType") // restore the position of the table windows
						&& child.getStringProperty("documentType").equals("ProjectTable")
						&& child.contains("name")
						&& child.getStringProperty("name").equals("ViewInfoProperties")) {
					boolean isClosed = true;
					if (child.contains("isClosed"))
						isClosed = child.getBooleanProperty("isClosed");
					if (isClosed==false) {
						WindowInfo windowProps = WindowInfo.createFromXMLEntity(child);
						String documentName = child.getStringProperty("documentName");
						ProjectTable pt = this.getTableByName(documentName);
						IWindow win = Project.openTable(pt);
						PluginServices.getMDIManager().changeWindowInfo(win, windowProps);
					}
				}

			}
		}
		
		if (!projectWindowRestored) { // if the project manager window was not stored in the project, just open it
			ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
			
			if (pe!=null) {
				pe.setProject(this);
				pe.showProjectWindow();
			}
		}
	}

	/**
	 * Restores the position ans size of windows from old projects
	 * (gvSIG 0.6.1, 1.0alpha, 1.0rc1). Previous versions did not
	 * store any information about the window properties. Later
	 * versions use a new system which also stores the window order.
	 * 
	 * It just restores positions of the Project Manager window, the
	 * Views and the Maps (not the tables, they were not stored).
	 * The window order (which windows are on the foreground)
	 * is not restored (it was not stored).
	 *  
	 * @param xml An XMLEntity object containing the whole project.
	 */
	private void restoreWindowProperties061(XMLEntity xml) {
		XMLEntity child;
		int childNumb;
		
		child = xml.getChild(xml.getChildrenCount()-1);
		if (child.contains("className")
				&& child.getStringProperty("className").equals("com.iver.cit.gvsig.project.Project")
				&& child.contains("name")
				&& child.getStringProperty("name").equals("ViewInfoProperties")) {
			this.seedProjectWindow = WindowInfo.createFromXMLEntity(child);
			
			ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
			
			if (pe!=null) {
				pe.setProject(this);
				pe.showProjectWindow();
			}
			
		}
		else  { // if the project manager window was not stored in the project, just open it
			ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);
			
			if (pe!=null) {
				pe.setProject(this);
				pe.showProjectWindow();
			}
		}
		
		// try to open the views
		childNumb = xml.getIntProperty("numExtents") + xml.getIntProperty("data-source-count");
		int limit = (xml.getIntProperty("numViews") + childNumb);
		WindowInfo winProps;
		XMLEntity view;
		for (int i = childNumb; i < limit; i++) {
			view = xml.getChild(i);
			child = view.getChild(view.getChildrenCount()-1);
			if (child.contains("className") && child.getStringProperty("className").equals("com.iver.cit.gvsig.project.ProjectView") && child.contains("name") && child.getStringProperty("name").equals("ViewInfoProperties")) {
				winProps = WindowInfo.createFromXMLEntity(child);
				String documentName = view.getStringProperty("name");
				ProjectView pv = this.getViewByName(documentName);
				IWindow win = Project.openView(pv);
				PluginServices.getMDIManager().changeWindowInfo(win, winProps);
			}
		}
		
		childNumb += xml.getIntProperty("numViews");
		limit = childNumb + xml.getIntProperty("numMaps"); 

		// try to open the maps
		XMLEntity map;
		for (int i = childNumb; i < limit; i++) {
			map = xml.getChild(i);
			for (int j=0; j<map.getChildrenCount(); j++)
			{
				child = map.getChild(j);
				if (child.contains("className") && child.getStringProperty("className").equals("com.iver.cit.gvsig.project.ProjectMap") && child.contains("name") && child.getStringProperty("name").equals("ViewInfoProperties")) {
					winProps = WindowInfo.createFromXMLEntity(child);
					String documentName = map.getStringProperty("name");
					ProjectMap pm = this.getLayoutByName(documentName);
					IWindow win = Project.openMap(pm);
					PluginServices.getMDIManager().changeWindowInfo(win, winProps);
				}
			}
		}
	}

	
	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException
	 */
	public static Project createFromXML03(XMLEntity xml)
		throws OpenException {
		Project p = new Project();
		try{
		p.comments = xml.getStringProperty("comments");
		p.creationDate = xml.getStringProperty("creationDate");
/*
		double[] xs = xml.getDoubleArrayProperty("extentsX");
		double[] ys = xml.getDoubleArrayProperty("extentsY");
		double[] ws = xml.getDoubleArrayProperty("extentsW");
		double[] hs = xml.getDoubleArrayProperty("extentsH");
*/
		int numExtents=xml.getIntProperty("numExtents");
		for (int i = 0; i < numExtents; i++) {
			ProjectExtent pe=ProjectExtent.createFromXML03(xml.getChild(i));
			p.extents.add(pe);
		}

		int numViews = xml.getIntProperty("numViews");

		for (int i = numExtents; i < numViews+numExtents; i++) {
			p.views.add(ProjectView.createFromXML03(xml.getChild(i), p));
		}

		int numMaps = xml.getIntProperty("numMaps");

		for (int i = numViews+numExtents; i < (numMaps + numViews+numExtents); i++) {
			p.maps.add(ProjectMap.createFromXML03(xml.getChild(i), p));
		}

		p.modificationDate = xml.getStringProperty("modificationDate");
		p.modified = xml.getBooleanProperty("modified");
		p.name = xml.getStringProperty("name");
		p.owner = xml.getStringProperty("owner");
//		p.path = xml.getStringProperty("path");
		p.selectionColor = StringUtilities.string2Color(xml.getStringProperty(
					"selectionColor"));

		int numTables = xml.getIntProperty("numTables");

		for (int i = numExtents+numMaps + numViews; i < (numExtents+numTables + numMaps + numViews);
				i++) {
			p.tables.add(ProjectTable.createFromXML03(xml.getChild(i), p));
		}
		String strProj = xml.getStringProperty("projection");
		if (strProj != null)
			p.setProjection(CRSFactory.getCRS(strProj));
		}catch (Exception e) {
			throw new OpenException(e,p.getClass().getName());
		}
		return p;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException
	 * @throws VersionException
	 */
	public static Project createFromXML(XMLEntity xml)
		throws OpenException {

		int childNumber = 0;
		Project p = new Project();
		try{
		p.comments = xml.getStringProperty("comments");
		p.creationDate = xml.getStringProperty("creationDate");

		/*
		   double[] xs = xml.getDoubleArrayProperty("extentsX");
		   double[] ys = xml.getDoubleArrayProperty("extentsY");
		   double[] ws = xml.getDoubleArrayProperty("extentsW");
		   double[] hs = xml.getDoubleArrayProperty("extentsH");
		 */
		int numExtents = xml.getIntProperty("numExtents");

		for (int i = 0; i < numExtents; i++) {
			ProjectExtent pe = ProjectExtent.createFromXML(xml.getChild(i));
			p.extents.add(pe);
		}

		childNumber = numExtents;

		int numDataSources = xml.getIntProperty("data-source-count");

		for (int i = childNumber; i < (childNumber + numDataSources); i++) {
			XMLEntity child = xml.getChild(i);

			if (child.getStringProperty("type").equals("otherDriverFile")) {
				LayerFactory.getDataSourceFactory().addFileDataSource(child.getStringProperty(
						"driverName"), child.getStringProperty("gdbmsname"),
					child.getStringProperty("file"));
			} else if (child.getStringProperty("type").equals("sameDriverFile")) {
				/*                                String layerName = child.getStringProperty("layerName");
				   ProjectView vista = project.getViewByName(child.getStringProperty(
				   "viewName"));
				   FLayer layer = vista.getMapContext().getLayers().getLayer(layerName);

				   modelo = ((AlphanumericData) layer).getRecordset();
				   associatedTable = (AlphanumericData) layer;
				 */
			} else if (child.getStringProperty("type").equals("db")) {
				LayerFactory.getDataSourceFactory().addDBDataSourceByTable(child.getStringProperty(
						"gdbmsname"), child.getStringProperty("host"),
					child.getIntProperty("port"),
					child.getStringProperty("user"),
					child.getStringProperty("password"),
					child.getStringProperty("dbName"),
					child.getStringProperty("tableName"),
					child.getStringProperty("driverInfo"));
			}
		}

		childNumber += numDataSources;

		int numViews = xml.getIntProperty("numViews");

		for (int i = childNumber; i < (numViews + childNumber); i++) {
			try{
				ProjectView pv = (ProjectView) ProjectView.createFromXML(xml.getChild(i), p);
				p.addView(pv);
			}catch(OpenException e){
				e.showError();
			}
		}

		childNumber += numViews;

		int numMaps = xml.getIntProperty("numMaps");

		for (int i = childNumber; i < (numMaps + childNumber); i++) {
			try{
				ProjectMap map = (ProjectMap) ProjectMap.createFromXML(xml.getChild(i), p);
				p.maps.add(map);
			}catch(OpenException e){
				e.showError();
			}
		}

		childNumber += numMaps;
		p.modificationDate = xml.getStringProperty("modificationDate");
		p.modified = xml.getBooleanProperty("modified");
		p.name = xml.getStringProperty("name");
		p.owner = xml.getStringProperty("owner");
//		p.path = xml.getStringProperty("path");
		p.selectionColor = StringUtilities.string2Color(xml.getStringProperty(
					"selectionColor"));

		int numTables = xml.getIntProperty("numTables");

		for (int i = childNumber; i < (childNumber + numTables); i++) {
			try{
				ProjectTable ptable = (ProjectTable) ProjectTable.createFromXML(xml.getChild(i), p);
				p.tables.add(ptable);
			}catch(OpenException e){
				e.showError();
			}
		}

		childNumber += numTables;
		
		if (childNumber < xml.getChildrenCount()) { // restore the position of the windows
			XMLEntity child = xml.getChild(childNumber);
			if (child.contains("className")
					&& child.getStringProperty("className").equals(Project.class.getName())
					&& child.contains("name")
					&& child.getStringProperty("name").equals("AndamiPersistence")) {
				p.restoreWindowProperties(child);
			}
			else if (child.contains("className")
					&& child.getStringProperty("className").equals("com.iver.cit.gvsig.project.Project")
					&& child.contains("name")
					&& child.getStringProperty("name").equals("ViewInfoProperties")) {
				p.restoreWindowProperties061(xml); 
			}
		}
		else { // if there was no info about the windows, just open the project manager window
			ProjectExtension pe =(ProjectExtension) PluginServices.getExtension(com.iver.cit.gvsig.ProjectExtension.class);			
			if (pe!=null) {
				pe.setProject(p);
				pe.showProjectWindow();
			}
		}

		p.setLinkTable();

		String strProj = xml.getStringProperty("projection");

		if (strProj != null) {
			p.setProjection(CRSFactory.getCRS(strProj));
		}
        PostProcessSupport.executeCalls();
		}catch (Exception e) {
			throw new OpenException(e,p.getClass().getName());
		}
//		p.setSignature( p.hashCode());
		return p;

	}
	
	
//	private void setSignature(int signature) {
//		this.signatureAtStartup = computeSignature();
//	}

	/**
	 * Reestablece los link que tena cada tabla con las dems.
	 */
	public void setLinkTable() {
		ArrayList tables = getTables();

		for (int i = 0; i < tables.size(); i++) {
			for (int j = 0; j < tables.size(); j++) {
				/*System.out.println("name = " +
					((ProjectTable) tables.get(j)).getModelo().getName());
				System.out.println("linktable = " +
					((ProjectTable) tables.get(i)).getLinkTable());
*/
				try {
				if ((((ProjectTable) tables.get(i)).getLinkTable() != null) &&
						((ProjectTable) tables.get(i)).getLinkTable().equals(((ProjectTable) tables
																				  .get(
								j)).getModelo().getRecordset().getName())) {
					LinkSelectionListener lsl;

						lsl = new LinkSelectionListener((SelectableDataSource)((ProjectTable) tables.get(
									i)).getModelo().getRecordset(),
									(SelectableDataSource)((ProjectTable) tables.get(j)).getModelo().getRecordset(),
								((ProjectTable) tables.get(i)).getField1(),
								((ProjectTable) tables.get(i)).getField2());

						((SelectableDataSource)((ProjectTable) tables.get(i)).getModelo().getRecordset())
					 .addSelectionListener(lsl);

				}
				} catch (DriverLoadException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * Obtiene la vista que contiene a la capa que se pasa como parmetro
	 *
	 * @param layer Capa cuya vista se quiere obtener
	 *
	 * @return
	 *
	 * @throws RuntimeException Si la capa que se pasa como parmetro no se
	 * 		   encuentra en ninguna vista
	 */
	public String getView(FLayer layer) {
		for (int v = 0; v < views.size(); v++) {
			ProjectView pView = (ProjectView) views.get(v);
			FLayers layers = pView.getMapContext().getLayers();
			if (isView(layers,layer))return pView.getName();
		}

		throw new RuntimeException("The layer is not in a view");
	}

	public boolean isView(FLayers layers, FLayer layer) {
		for (int i = 0; i < layers.getLayersCount(); i++) {
			if (layers.getLayer(i) instanceof FLayers){
				return isView((FLayers)layers.getLayer(i),layer);
			}else{
				if (layers.getLayer(i) == layer) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Devuelve la vista cuyo nombre coincide (sensible a mayusculas) con el
	 * que se pasa como parmetro. Devuelve null si no hay ninguna vista con
	 * ese nombre
	 *
	 * @param viewName Nombre de la vista que se quiere obtener
	 *
	 * @return DOCUMENT ME!
	 */
	public ProjectView getViewByName(String viewName) {
		Object o = getProjectElementByName(viewName, views);

		if (o == null) {
			return null;
		}

		return (ProjectView) o;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getProjection() {
		if (projection == null)
			projection = defaultProjection;
		return defaultProjection;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param defaultProjection DOCUMENT ME!
	 */
	public void setProjection(IProjection defaultProjection) {
		Project.defaultProjection = defaultProjection;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param name DOCUMENT ME!
	 * @param l DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private Object getProjectElementByName(String name, ArrayList l) {
		for (Iterator iter = l.iterator(); iter.hasNext();) {
			ProjectElement elem = (ProjectElement) iter.next();

			if (elem.getName().equals(name)) {
				return elem;
			}
		}

		return null;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param name
	 *
	 * @return
	 */
	public ProjectTable getTableByName(String name) {
		Object o = getProjectElementByName(name, tables);

		if (o == null) {
			return null;
		}

		return (ProjectTable) o;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param name
	 *
	 * @return
	 */
	public ProjectMap getLayoutByName(String name) {
		Object o = getProjectElementByName(name, maps);

		if (o == null) {
			return null;
		}

		return (ProjectMap) o;
	}

    public SelectableDataSource getDataSourceByLayer(FLayer layer) throws DriverException {
        ArrayList tables = getTables();
        SelectableDataSource dataSource = null;
        try {
        for (int i = 0; i < tables.size(); i++) {
            ProjectTable pt = (ProjectTable) tables.get(i);
            if (pt.getOriginal() == ((AlphanumericData) layer).getRecordset()){
                dataSource = (SelectableDataSource)pt.getModelo().getRecordset();
                break;
            }else if (pt.getModelo() == ((AlphanumericData) layer).getRecordset()){

					dataSource = (SelectableDataSource)pt.getModelo().getRecordset();

                break;
            }
        }
        } catch (DriverLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

        if (dataSource == null){
            //No se ha creado la tabla asociada al tema
            dataSource = ((AlphanumericData) layer).getRecordset();
        }

        return dataSource;

    }

    /**
     * Recorremos las capas y las tablas del proyecto, y creamos una lista
     * con todos los datasources de GDBMS que estamos usando.
     * Luego recorremos los que estn registrados, y borramos
     * aquellos que no estn siendo usados, es decir, aquellos
     * que no estn en nuestra lista (un Hash con clave el nombre
     * del GDBMS)
     *
     */
    private void cleanBadReferences()
    {
        Hashtable usedDataSources = new Hashtable();
        // Primero las tablas
        int i, j;
        try {
        for (i=0; i<tables.size(); i++)
        {
            ProjectTable t = (ProjectTable) tables.get(i);
            SelectableDataSource ds;

				ds = (SelectableDataSource)t.getModelo().getRecordset();

            if (t.getOriginal() != null)
                usedDataSources.put(t.getOriginal().getRecordset().getName(), t.getOriginal());
            usedDataSources.put(ds.getName(), ds);
        }
        } catch (DriverLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        // Ahora las vistas
        try {
            for (i=0; i<views.size(); i++)
            {
                ProjectView pv = (ProjectView) views.get(i);

                FLayers lyrs = pv.getMapContext().getLayers();

                for (j=0; j<lyrs.getLayersCount(); j++)
                {
                    FLayer lyr = lyrs.getLayer(j);
                    if (lyr instanceof FLyrVect)
                    {
                        FLyrVect lyrVect = (FLyrVect) lyr;
                        if (lyrVect.isAvailable())
                        	usedDataSources.put(lyrVect.getRecordset().getName(), lyrVect.getSource().getRecordset());
                    }
                }
                MapContext aux = pv.getMapOverViewContext();
                if (aux != null)
                {
                    FLayers lyrsOverview = aux.getLayers();
                    for (j=0; j<lyrsOverview.getLayersCount(); j++)
                    {
                        FLayer lyr = lyrsOverview.getLayer(j);
                        if (lyr instanceof FLyrVect)
                        {
                            FLyrVect lyrVect = (FLyrVect) lyr;
                            usedDataSources.put(lyrVect.getSource().getRecordset().getName(), lyrVect.getSource().getRecordset());
                        }
                    }
                }

            } // for i
        } catch (DriverLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        // Recorremos los dataSources y los borramos si no
        // los estamos usando.
        SourceInfo[] infos = LayerFactory.getDataSourceFactory().getDriverInfos();
        try {
            for (i=0; i < infos.length; i++)
            {
                if (!usedDataSources.containsKey(infos[i].name))
                {
                    DataSource ds;
                    ds = LayerFactory.getDataSourceFactory().createRandomDataSource(infos[i].name);
                    ds.remove();
                }
            }
        } catch (DriverLoadException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (NoSuchTableException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

	public WindowInfo getSeedProjectWindow() {
		return seedProjectWindow;
	}
	
	public void setSeedProjectWindow(WindowInfo projectWindow) {
		seedProjectWindow = projectWindow;
	}

	public XMLEntity getSourceInfoXMLEntity(SourceInfo di) {
		XMLEntity child = new XMLEntity();


		if (di instanceof ObjectSourceInfo) {
			ObjectSourceInfo driver = (ObjectSourceInfo) di;
			child.putProperty("type", "sameDriverFile");
			child.putProperty("gdbmsname", driver.name);
		} else if (di instanceof FileSourceInfo) {
			FileSourceInfo vfdi = (FileSourceInfo) di;
			child.putProperty("type", "otherDriverFile");
			child.putProperty("gdbmsname", vfdi.name);
			child.putProperty("file", vfdi.file);
			child.putProperty("driverName", vfdi.driverName);
		} else if (di instanceof DBSourceInfo) {
			DBTableSourceInfo dbdi = (DBTableSourceInfo) di;
			child.putProperty("type", "db");
			child.putProperty("gdbmsname", dbdi.name);
			child.putProperty("dbms", dbdi.dbms);
			child.putProperty("host", dbdi.host);
			child.putProperty("port", dbdi.port);
			child.putProperty("user", dbdi.user);
			child.putProperty("password", dbdi.password);
			child.putProperty("dbName", dbdi.dbName);
			child.putProperty("tableName", dbdi.tableName);
			child.putProperty("driverInfo", dbdi.driverName);
		}

		return child;
	}


	/**
	 * Sets the projection used when no projection is defined
	 * @param defaultProjection DOCUMENT ME!
	 */
	public static void setDefaultProjection(IProjection defaultProjection) {
		Project.defaultProjection = defaultProjection;
	}

	public static IProjection getDefaultProjection() {
		return Project.defaultProjection;
	}

//	public boolean hasChanged() {
//		return signatureAtStartup != computeSignature();
//	}

//	public int computeSignature() {
//
////		/* TODO method also present in ProjectMap, ProjectTable,
////		 *  ProjectView and ProjectExtent
////		 */
//		int result = 17;
////
////		Class clazz = getClass();
////		Field[] fields = clazz.getDeclaredFields();
////		for (int i = 0; i < fields.length; i++) {
////			try {
////				// don't compute following fields
////				if (fields[i].getName().equals("modificationDate")
////						|| fields[i].getName().equals("creationDate")
////						|| fields[i].getName().equals("signatureAtStartup")
////						|| fields[i].getName().equals("change")
////						|| Modifier.isStatic(fields[i].getModifiers()))
////					continue;
////
////
////				String type = fields[i].getType().getName();
////				if (type.equals("boolean")) {
////					result += 37 + ((fields[i].getBoolean(this)) ? 1 : 0);
////				} else if (type.equals("java.lang.String")) {
////					Object v = fields[i].get(this);
////					if (v == null) {
////						result += 37;
////						continue;
////					}
////					char[] chars = ((String) v).toCharArray();
////					for (int j = 0; j < chars.length; j++) {
////						result += 37 + (int) chars[i];
////					}
////				} else if (type.equals("byte")) {
////					result += 37 + (int) fields[i].getByte(this);
////				} else if (type.equals("char")) {
////					result += 37 + (int) fields[i].getChar(this);
////				} else if (type.equals("short")) {
////					result += 37 + (int) fields[i].getShort(this);
////				} else if (type.equals("int")) {
////					result += 37 + fields[i].getInt(this);
////				} else if (type.equals("long")) {
////					long f = fields[i].getLong(this) ;
////					result += 37 + (f ^ (f >>> 32));
////				} else if (type.equals("float")) {
////					result += 37 + Float.floatToIntBits(fields[i].getFloat(this));
////				} else if (type.equals("double")) {
////					long f = Double.doubleToLongBits(fields[i].getDouble(this));
////					result += 37 + (f ^ (f >>> 32));
////				} else {
//////					Object obj = fields[i].get(this);
//////					result += 37 + ((obj != null)? obj.hashCode() : 0);
////				}
////			} catch (Exception e) { e.printStackTrace(); }
////
////		}
////
////		for (int i = 0; i < views.size(); i++) {
////			ProjectView e = (ProjectView) views.get(i);
////			result += 37 + e.computeSignature();
////		}
////
////		for (int i = 0; i < tables.size(); i++) {
////			ProjectTable e = (ProjectTable) tables.get(i);
////			result += 37 + e.computeSignature();
////		}
////
////		for (int i = 0; i < maps.size(); i++) {
////			ProjectMap e = (ProjectMap) maps.get(i);
////			result += 37 + e.computeSignature();
////		}
////
////		for (int i = 0; i < extents.size(); i++) {
////			ProjectExtent e = (ProjectExtent) extents.get(i);
////			result += 37 + e.computeSignature();
////		}
//
//
//		return result;
//	}

}
