/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project;

import com.hardcode.driverManager.DriverLoadException;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.WindowInfo;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.SaveException;

import com.iver.utiles.XMLEntity;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import java.io.Serializable;

import java.text.DateFormat;

import java.util.Date;


/**
 * Clase base de los elementos del proyecto (mapas, tablas y vistas)
 *
 * @author Fernando Gonzlez Corts
 */
public abstract class ProjectElement implements Serializable {
	protected PropertyChangeSupport change;
	protected Project project;
	protected int index;
	protected String name;
	protected String creationDate;
	protected String owner;
	protected String comment;
	private boolean locked = false;
	// the window associated to this project element (if any)
	protected com.iver.andami.ui.mdiManager.IWindow window = null;

	/**
	 * Creates a new ProjectElement object.
	 */
	public ProjectElement() {
		creationDate = DateFormat.getDateInstance().format(new Date());
		change = new PropertyChangeSupport(this);
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return name;
	}

	/**
	 * Obtiene el nombre del elemento
	 *
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 * Establece el nombre del elemento
	 *
	 * @param string
	 */
	public void setName(String string) {
		String oldValue = name;
		name = string;
		change.firePropertyChange("name", oldValue, name);
	}

	/**
	 * Obtiene la fecha de creacin del elemento
	 *
	 * @return
	 */
	public String getCreationDate() {
		return creationDate;
	}

	/**
	 * Obtiene el propietario del elemento
	 *
	 * @return
	 */
	public String getOwner() {
		return owner;
	}

	/**
	 * Establece la fecha de creacin del elemento.
	 *
	 * @param string
	 */
	public void setCreationDate(String string) {
		creationDate = string;
		change.firePropertyChange("creationDate", creationDate, creationDate);
	}

	/**
	 * Establece el propietario del elemento
	 *
	 * @param string
	 */
	public void setOwner(String string) {
		owner = string;
		change.firePropertyChange("owner", owner, owner);
	}

	/**
	 * Obtiene los comentarios del proyecto
	 *
	 * @return
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * Establece los comentarios del proyecto
	 *
	 * @param string
	 */
	public void setComment(String string) {
		comment = string;
		change.firePropertyChange("comment", comment, comment);
	}

	/**
	 * Aade un listener para los cambios en las bounded properties
	 *
	 * @param listener
	 */
	public synchronized void addPropertyChangeListener(
		PropertyChangeListener listener) {
		change.addPropertyChangeListener(listener);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws SaveException 
	 */
	public XMLEntity getXMLEntity() throws SaveException {
		XMLEntity xml = new XMLEntity();
		try{
		xml.putProperty("className", this.getClass().getName());
		xml.putProperty("comment", comment);
		xml.putProperty("creationDate", creationDate);
		xml.putProperty("name", name);
		xml.putProperty("owner", owner);
		}catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
		return xml;
	}

    /**
     * DOCUMENT ME!
     *
     * @param xml DOCUMENT ME!
     * @param p DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     * @throws XMLException
     * @throws DriverException
     * @throws DriverIOException
     * @throws CancelationException
     * @throws ClassNotFoundException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws DriverIOException
     * @throws DriverLoadException
     */
    public static ProjectElement createFromXML03(XMLEntity xml, Project p) throws XMLException, DriverException, DriverIOException{
        ProjectElement pe = null;

            Class clase;
			try {
				clase = Class.forName(xml.getStringProperty("className"));
			pe = (ProjectElement) clase.newInstance();
			} catch (ClassNotFoundException e) {
	            NotificationManager.addError("Clase de ProjectElement no reconocida",
	                    e);
			} catch (InstantiationException e) {
	            NotificationManager.addError("Clase de ProjectElement no reconocida",
	                    e);
			} catch (IllegalAccessException e) {
	            NotificationManager.addError("Clase de ProjectElement no reconocida",
	                    e);
			}

        pe.setComment(xml.getStringProperty("comment"));
        pe.setCreationDate(xml.getStringProperty("creationDate"));
        pe.setName(xml.getStringProperty("name"));
        pe.setOwner(xml.getStringProperty("owner"));
        pe.project = p;

		pe.setXMLEntity03(xml,p);

        return pe;
    }

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException 
	 */
	public static ProjectElement createFromXML(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException, OpenException {
		ProjectElement pe = null;
		try{
		Class clase;

		try {
			clase = Class.forName(xml.getStringProperty("className"));
			pe = (ProjectElement) clase.newInstance();
		} catch (ClassNotFoundException e) {
			NotificationManager.addError("Clase de ProjectElement no reconocida",
				e);
		} catch (InstantiationException e) {
			NotificationManager.addError("Clase de ProjectElement no reconocida",
				e);
		} catch (IllegalAccessException e) {
			NotificationManager.addError("Clase de ProjectElement no reconocida",
				e);
		}

		pe.setComment(xml.getStringProperty("comment"));
		pe.setCreationDate(xml.getStringProperty("creationDate"));
		pe.setName(xml.getStringProperty("name"));
		pe.setOwner(xml.getStringProperty("owner"));
		pe.project = p;
		}catch (Exception e) {
			throw new OpenException(e,pe.getClass().getName());
		}
		pe.setXMLEntity(xml, p);

		return pe;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException 
	 */
	public abstract void setXMLEntity(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException, OpenException;

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public abstract void setXMLEntity03(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException;

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Project getProject() {
		return project;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param project DOCUMENT ME!
	 */
	public void setProject(Project project, int index) {
		this.project = project;
		this.index = index;
	}
	public int getIndex() {
		return index;
	}
	
	/**
	 * Gets the window associated to this project element.
	 */
	public com.iver.andami.ui.mdiManager.IWindow getAndamiWindow() {
		return window;
	}
	
	/**
	 * Sets the window associated to this project element.
	 * @param view
	 */
	public void setAndamiWindow(com.iver.andami.ui.mdiManager.IWindow window) {
		this.window = window;
	}
	
	/**
	 * Locks this project element protecting it from deleting from the project.
	 */
	public void lock() {
		locked = true;
	}
	
	/**
	 * Unlocks this element. So, from now on, it can be removed from the project.
	 */
	public void unlock() {
		locked = false;
	}

	/**
	 * Tells whether if this project's element is locked/protected or not. A protected
	 * element cannot be removed from the current project.
	 * 
	 * @see <b>lock()</b> and <b>unlock()</b> methods.
	 *      
	 * @return true if it is locked, false otherwise
	 */
	public boolean isLocked() {
		return locked;
	}
}
