package es.prodevelop.cit.gvsig.arcims.extension;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.About;
import com.iver.cit.gvsig.AddLayer;
import com.iver.cit.gvsig.gui.toc.FPopupMenu;
import com.iver.utiles.extensionPoints.ExtensionPoints;
import com.iver.utiles.extensionPoints.ExtensionPointsSingleton;

import es.prodevelop.cit.gvsig.arcims.fmap.layers.FFeatureLyrArcIMS;
import es.prodevelop.cit.gvsig.arcims.fmap.layers.FFeatureLyrArcIMSCollection;
import es.prodevelop.cit.gvsig.arcims.fmap.layers.FRasterLyrArcIMS;
import es.prodevelop.cit.gvsig.arcims.gui.toc.ArcImsLayerScaleTocMenuEntry;
import es.prodevelop.cit.gvsig.arcims.gui.toc.ArcImsPropsTocMenuEntry;
import es.prodevelop.cit.gvsig.arcims.gui.toc.ArcImsRasterPropsTocMenuEntry;
import es.prodevelop.cit.gvsig.arcims.gui.wizards.ArcImsWizard;

/**
 * This class implements the extension to access ArcIMS servers.
 * 
 * @author jldominguez
 *
 */
public class ArcimsClientModule extends Extension {
	
	/**
     * This method initializes the extension. Adds the ArcIMS wizard
     * and the right-click popup menus to the gvSIG resources.
     */
    public void initialize() {
    	// must add menus and a new tab to the wizard
    	// to create an arcims layer
    	FPopupMenu.addEntry(new ArcImsPropsTocMenuEntry());
    	FPopupMenu.addEntry(new ArcImsRasterPropsTocMenuEntry());
    	FPopupMenu.addEntry(new ArcImsLayerScaleTocMenuEntry());
    	AddLayer.addWizard(ArcImsWizard.class);
    	
    	// about
        java.net.URL newurl = createResourceUrl("about/extarcims-about.html");
    	About claseAbout = (About) PluginServices.getExtension(com.iver.cit.gvsig.About.class);
        claseAbout.getAboutPanel().addAboutUrl("ArcIMS", newurl);
        
        // catalog
    	ExtensionPoints extensionPoints = ExtensionPointsSingleton.getInstance();
    	extensionPoints.add("CatalogLayers", "arcims_raster", FRasterLyrArcIMS.class);
    	extensionPoints.add("CatalogLayers", "arcims_vectorial", FFeatureLyrArcIMSCollection.class);
    }

    /**
     * This method is called when the extension's controls are used.
     * So far, the ArcIMS plugin does <i>not</i> add any controls.
     * 
     * @param actionCommand the control's string action commmad.
     * 
     */
    public void execute(String actionCommand) {
    }

    /**
     * This method is called to find out if the plugin's controls are
     * enabled or not. So far, the ArcIMS plugin does <i>not</i>
     * add any controls.
     *
     * @return <b>true</b> if controls must be enabled, <b>false</b> if not.
     */
    public boolean isEnabled() {
    	return false;
    }

    /**
     * This method is called to find out if the plugin's controls are
     * visible or not. So far, the ArcIMS plugin does <i>not</i>
     * add any controls.
     *
     * @return <b>true</b> if controls must be visible, <b>false</b> if not.
     */
    public boolean isVisible() {
    	return false;
    }
    
    private java.net.URL createResourceUrl(String path) {
		return getClass().getClassLoader().getResource(path);
	}

    
}
