/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gvsig.catalogClient.gui;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.geom.Rectangle2D;
import java.util.Collection;

import javax.swing.JDialog;

import org.gvsig.i18n.Messages;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.ColorEvent;
import com.iver.cit.gvsig.fmap.ExtentEvent;
import com.iver.cit.gvsig.fmap.ProjectionEvent;
import com.iver.cit.gvsig.fmap.ViewPortListener;
import com.iver.cit.gvsig.gui.View;

import es.gva.cit.catalogClient.CatalogClient;
import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.catalogClient.ui.search.SearchDialogPanel;
import es.gva.cit.catalogClient.ui.showResults.ShowResultsPanel;
import es.gva.cit.catalogClient.utils.Frames;

/**
 * This class is used to search a record using the catalog client
 * @author luisw
 * @modified by Jorge Piera
 */
public class SearchDialog extends SearchDialogPanel implements
		IWindow,ViewPortListener {
    private JDialog frame = null;
    public WindowInfo m_viewinfo = null;
   
	public SearchDialog(JDialog frame,Object serverConnectFrame,CatalogClient client) {
		super(null,serverConnectFrame,client);
		this.frame = frame;
		setViewChangeListener();		
		loadViewPortCoordinates();
	}

	/**
	 * This method opens the show results frame.
	 * @param nodes
	 * XML node returned by the query
	 */
	protected void showResults(Collection nodes) {
        JDialog panel = new JDialog((Frame) PluginServices.getMainFrame(), false);
        Frames.centerFrame(panel,610,418);
        panel.setTitle(Messages.getText("search_results")); 
        panel.setResizable(false);
        
        ShowResultsPanel.mustShowThumbnails = true;
		ShowResultsDialog dialog =
			new ShowResultsDialog(panel,client,nodes,1);
		
        panel.getContentPane().add(dialog);
        panel.show();      
	}
		
	/**
	 * It Closes the dialog
	 */
	public void closeJDialog() {
	    frame.setVisible(false);
		
	}
	
	/* (non-Javadoc)
	 * @see com.iver.andami.ui.mdiManager.View#getViewInfo()
	 */
	public WindowInfo getWindowInfo() {
	    if (m_viewinfo == null){
	        m_viewinfo = new WindowInfo(WindowInfo.MODALDIALOG);
	        m_viewinfo.setTitle(getName());
	    }
		return m_viewinfo;
	}

	
	/**
	 * Close button Action performed
	 */	
	 public void closeButtonActionPerformed() {
	     closeJDialog();
	 }
	 
	 /**
	  * Size button action performed
	  */	 
	 public void sizeButtonActionPerformed(){
	        if (isMinimized){
	            frame.setSize(525,457);
	            ppalPanel.setPreferredSize(new Dimension(514,423));
	            getLowerPanel().setPreferredSize(new Dimension(514,440));
	            getLowerPanel().setVisible(true);
	            frame.setVisible(true);
	            getUpperPanel().setUpIcon();
	        }else{
	            frame.setSize(525,118);
	            ppalPanel.setPreferredSize(new Dimension(514,91));
	            getLowerPanel().setPreferredSize(new Dimension(514,0));
	            getLowerPanel().setVisible(false);
	            frame.setVisible(true);
	            getUpperPanel().setDownIcon();
	        }
	        isMinimized = !isMinimized;
	 }
	
	 /**
	  * Return button action
	  */
	public void returnButtonActionPerformed() {  
		closeJDialog();
		ConnectDialog serverConnect = (ConnectDialog)serverConnectFrame;
		serverConnect.setVisible(true);
		serverConnect.getSearchButton().setEnabled(false);
		PluginServices.getMDIManager().addWindow(serverConnect);
		
		
	} 

	 
	 /**
	  * This method loads the view coordinates to the catalog search dialog
	  *
	  */
	 private void loadViewPortCoordinates(){
		 View activeView = 
				(View) PluginServices.getMDIManager().getActiveWindow();
	       
	        Rectangle2D r2d= activeView.getMapControl().getViewPort().getAdjustedExtent();
	        
	        try{
	        	getLowerPanel().setCoordinates(new Coordinates(r2d.getMinX(),
	                r2d.getMaxY(),
	                r2d.getMaxX(),
	                r2d.getMinY()));
	        }catch(NullPointerException E){
	            //We cant retrieve the coordinates if it doesn't 
	            //exist a loaded layer
	        }
	 }  
    
    /**
     * This methos joins the viewPort event to the listener
     */
    private void setViewChangeListener(){
    	View activeView = 
			(View) PluginServices.getMDIManager().getActiveWindow();
               
        activeView.getMapControl().getViewPort().addViewPortListener(this);
    }

	public void extentChanged(ExtentEvent e) {
		loadViewPortCoordinates();   
		
	}

	public void backColorChanged(ColorEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void projectionChanged(ProjectionEvent e) {
		loadViewPortCoordinates();
	}
    
  
	 
	 
	    
}
