/*
 * Created on 11-abr-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: UnionGeoprocessController.java,v 1.4 2006/08/11 16:30:38 azabala Exp $
* $Log: UnionGeoprocessController.java,v $
* Revision 1.4  2006/08/11 16:30:38  azabala
* *** empty log message ***
*
* Revision 1.3  2006/07/21 09:10:34  azabala
* fixed bug 608: user doesnt enter any result file to the geoprocess panel
*
* Revision 1.2  2006/06/29 07:33:57  fjp
* Cambios ISchemaManager y IFieldManager por terminar
*
* Revision 1.1  2006/06/20 18:20:45  azabala
* first version in cvs
*
* Revision 1.5  2006/06/12 19:15:38  azabala
* cambios para poder trabajar en geoprocessing con capas MULTI (dxf, jdbc, etc)
*
* Revision 1.4  2006/06/08 18:25:34  azabala
* modificaciones para admitir capas de shapeType MULTI
*
* Revision 1.3  2006/06/02 18:21:28  azabala
* *** empty log message ***
*
* Revision 1.2  2006/05/25 08:21:48  jmvivo
* Aadida peticion de confirmacion para sobreescribir el fichero de salida, si este ya existiera
*
* Revision 1.1  2006/05/24 21:09:11  azabala
* primera version en cvs despues de refactoring orientado a crear un framework extensible de geoprocessing
*
* Revision 1.2  2006/05/08 15:35:32  azabala
* *** empty log message ***
*
* Revision 1.1  2006/04/11 17:55:51  azabala
* primera version en cvs
*
*
*/
package com.iver.cit.gvsig.geoprocess.impl.union;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.SHPLayerDefinition;
import com.iver.cit.gvsig.fmap.edition.IWriter;
import com.iver.cit.gvsig.fmap.edition.ShpSchemaManager;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.geoprocess.core.fmap.AbstractGeoprocessController;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.IGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.gui.AddResultLayerTask;
import com.iver.cit.gvsig.geoprocess.core.gui.IGeoprocessPanel;
import com.iver.cit.gvsig.geoprocess.core.gui.OverlayPanelIF;
import com.iver.cit.gvsig.geoprocess.impl.union.fmap.UnionGeoprocess;
import com.iver.utiles.swing.threads.IMonitorableTask;
import com.iver.utiles.swing.threads.MonitorableDecoratorMainFirst;

public class UnionGeoprocessController extends AbstractGeoprocessController {

	private OverlayPanelIF geoProcessingUnionPanel;
	private UnionGeoprocess union;
	
	public void setView(IGeoprocessPanel viewPanel) {
		this.geoProcessingUnionPanel = (OverlayPanelIF) viewPanel;
	}

	public IGeoprocess getGeoprocess() {
		return union;
	}

	public boolean launchGeoprocess() {
		FLyrVect inputLayer = geoProcessingUnionPanel.getInputLayer();
		FLyrVect overlayLayer = geoProcessingUnionPanel.getSecondLayer();
		FLayers layers = geoProcessingUnionPanel.getFLayers();
		File outputFile = null;
		try {
			outputFile = geoProcessingUnionPanel.getOutputFile();
		} catch (FileNotFoundException e3) {
			String error = PluginServices.getText(this, "Error_entrada_datos");
			String errorDescription = PluginServices.getText(this, "Error_seleccionar_resultado");
			geoProcessingUnionPanel.error(errorDescription, error);
			return false;
		}
		if (outputFile == null || (outputFile.getAbsolutePath().length() == 0)) {
			String error = PluginServices.getText(this, "Error_entrada_datos");
			String errorDescription = PluginServices.getText(this, "Error_seleccionar_resultado");
			geoProcessingUnionPanel.error(errorDescription, error);
			return false;
		}
		if (outputFile.exists()) {
			if (!geoProcessingUnionPanel.askForOverwriteOutputFile(outputFile)) {
				return false;
			}
		}
		UnionGeoprocess union = new UnionGeoprocess(inputLayer);
		union.setSecondOperand(overlayLayer);
		SHPLayerDefinition definition = (SHPLayerDefinition) union
				.createLayerDefinition();
		definition.setFile(outputFile);
		ShpSchemaManager schemaManager = new ShpSchemaManager(outputFile.getAbsolutePath());
		IWriter writer = null;
		try {
			writer = getShpWriter(definition);
		} catch (Exception e1) {
			String error = PluginServices.getText(this, "Error_escritura_resultados");
			String errorDescription = PluginServices.getText(this, "Error_preparar_escritura_resultados");
			geoProcessingUnionPanel.error(errorDescription, error);
			return false;
		} 
		union.setResultLayerProperties(writer, schemaManager);
		HashMap params = new HashMap();
		boolean onlySelectedFirst = geoProcessingUnionPanel
				.onlyFirstLayerSelected();
		boolean onlySelectedSecond = geoProcessingUnionPanel
				.onlySecondLayerSelected();
		Boolean first = new Boolean(onlySelectedFirst);
		params.put("firstlayerselection", first);

		Boolean second = new Boolean(onlySelectedSecond);
		params.put("secondlayerselection", second);

		try {
			union.setParameters(params);
			union.checkPreconditions();
			IMonitorableTask task1 = union.createTask();
			if(task1 == null){
				return false;
			}
			AddResultLayerTask task2 = new AddResultLayerTask(union);
			task2.setLayers(layers);
			MonitorableDecoratorMainFirst globalTask = new MonitorableDecoratorMainFirst(task1,
					task2);
			if(!overlayLayer.isSpatiallyIndexed()){
				final IMonitorableTask sptIdxTask = 
					geoProcessingUnionPanel.askForSpatialIndexCreation(overlayLayer);
				if(sptIdxTask != null){
					PluginServices.backgroundExecution(
							new Runnable(){
						public void run() {
							PluginServices.
							cancelableBackgroundExecution(sptIdxTask);	
						}}
					);
				}	
			}//if
			if (globalTask.preprocess())
				PluginServices.cancelableBackgroundExecution(globalTask);
		} catch (GeoprocessException e) {
			String error = PluginServices.getText(this, "Error_ejecucion");
			String errorDescription = PluginServices.getText(this, "Error_fallo_geoproceso");
			geoProcessingUnionPanel.error(errorDescription, error);
			return false;
		}
		return true;
	}

}

