/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.cresques.px.Extent;
import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.GeoPoint;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Listeners.PanListener;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;


/**
 * Behaviour que espera un listener de tipo GeoMoveListener.
 * Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class GeoMoveBehavior extends Behavior {
	
	private final Image defaultCursor = new ImageIcon(MapControl.class.getResource(
	"images/CruxCursor.png")).getImage();
	private final Image handCursor = new ImageIcon(MapControl.class.getResource(
	"images/Hand.gif")).getImage();

	private Cursor 							cur = null;	
	private FLyrGeoRaster 					lyrGeoRaster = null;
	private boolean 						isMoveable = false;
	
	/**
	 * Variable booleana que est a true si el cursor por defecto est 
	 * activo y a false si hay otro.
	 */
	private boolean 						defaultCursorActive = true;
	
	/**
	 * Puntos de inicio y final para el arrastre de la imagen.
	 */
	private Point2D 						ptoIni = null;
	private Point2D 						ptoFin = null;
	private com.iver.cit.gvsig.gui.View 	theView = null;
	private PanListener 					listener;
		
	/**
	 * Crea un nuevo RectangleBehavior.
	 *
	 * @param zili listener.
	 */
	public GeoMoveBehavior(PanListener zili) {
		listener = zili;
		cur = zili.getCursor();
	}
	
	/**
	 * Funcin que carga la capa si todava no lo est.
	 */
	private void loadLayer() throws InstantiationException, ClassCastException{
		//Cargamos la capa
		
		theView = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveWindow();
		for(int i=0;i<theView.getMapControl().getMapContext().getLayers().getLayersCount();i++){
			FLayer lyr = theView.getMapControl().getMapContext().getLayers().getLayer(i);
			if(	lyr instanceof FLyrGeoRaster && 
				lyr.getName().startsWith("*") &&
				lyr.isActive())
				this.lyrGeoRaster = (FLyrGeoRaster)lyr;
		}
		if(lyrGeoRaster == null)
			throw new InstantiationException("No se ha podido cargar la capa de georreferenciacin");
	}
	
	/**
	 * Coloca el cursor del ratn con el icono adecuado cuando entra dentro de la 
	 * imagen.
	 */
	public void mouseMoved(MouseEvent e) throws BehaviorException {
		
		ViewPort vp = getMapControl().getMapContext().getViewPort();
		
		try{
			loadLayer();
			
			Point2D pto = vp.toMapPoint(e.getX(), e.getY());
				
			if(	pto.getX() > lyrGeoRaster.getAssignExtent().getMin().getX() &&
				pto.getX() < lyrGeoRaster.getAssignExtent().getMax().getX() &&
				pto.getY() > lyrGeoRaster.getAssignExtent().getMin().getY() &&
				pto.getY() < lyrGeoRaster.getAssignExtent().getMax().getY()){
				if(defaultCursorActive){
					getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(handCursor,
						new Point(16, 16), ""));
					defaultCursorActive = false;
				}
			}else{
				if(!defaultCursorActive){
					getMapControl().setCursor(Toolkit.getDefaultToolkit().createCustomCursor(defaultCursor,
						new Point(16, 16), ""));
					defaultCursorActive = true;
				}
			}
		}catch(InstantiationException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}catch(ClassCastException exc){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
		
	}
	
	/**
	 * Obtiene el cursor actual
	 */
	public Cursor getCursor() {
		return this.cur;
	}
	
	/**
	 * Si no est activo el cursor por defecto capturamos el punto 
	 * seleccionado en coordenadas del mundo real.
	 */
	public void mousePressed(MouseEvent e) throws BehaviorException {
		if(e.getButton() == MouseEvent.BUTTON1 && !defaultCursorActive){
			
			try{
				loadLayer();
			}catch(InstantiationException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
				return;
			}catch(ClassCastException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
				return;
			}
			
			ViewPort vp = getMapControl().getMapContext().getViewPort();
			ptoIni = vp.toMapPoint(e.getPoint());
			
			//Cargamos la imagen de arrastre
/*			Point2D ul = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMin().getX(), 
										lyrGeoRaster.getAssignExtent().getMin().getY());
			Point2D lr = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMax().getX(), 
										lyrGeoRaster.getAssignExtent().getMax().getY());
						
			double ulx = 0D, uly = 0D, lrx = 0D, lry = 0D;
			if(ul.getX() > vp.getExtent().getMinX())
				ulx = ul.getX();
			else
				ulx = vp.getExtent().getMinX();
			
			if(ul.getY() > vp.getExtent().getMinY())
				uly = ul.getY();
			else
				uly = vp.getExtent().getMinY();
			
			if(lr.getX() < vp.getExtent().getMaxX())
				lrx = lr.getX();
			else
				lrx = vp.getExtent().getMaxX();
			
			if(lr.getY() < vp.getExtent().getMaxY())
				lry = lr.getY();
			else
				lry = vp.getExtent().getMaxY();
			
			BufferedImage img = getMapControl().getImage();
			System.out.println("-->"+img.getMinX()+" "+img.getMinY()+" "+img.getWidth()+" "+img.getHeight());
			Rectangle r = new Rectangle((int)ulx, (int)uly, (int)Math.abs(lrx-ulx), (int)Math.abs(lry-uly));
			System.out.println("--->"+ulx+" "+uly+" "+lrx+" "+lry);
			Raster raster = img.getData(r);
			miniGeoRaster.setData(raster);*/
			/*miniGeoRaster = new BufferedImage((int)Math.abs(lrx-ulx), (int)Math.abs(lry-uly), BufferedImage.TYPE_INT_ARGB);
			int[] rasterData = new int[r.width * r.height * 100];
			rasterData = img.getRGB(r.x, r.y, r.width, r.height, rasterData, 0, r.width);
			miniGeoRaster.setRGB(0, 0, r.width, r.height, rasterData, 0, r.width);*/

			
			isMoveable = true;
		}		
	}
	
	/**
	 *  Cuando soltamos el botn del ratn desplazamos la imagen a la posicin
	 * de destino calculando el extent nuevamente.
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		if(e.getButton() == MouseEvent.BUTTON1 && isMoveable){
			
			ViewPort vp = getMapControl().getMapContext().getViewPort();
			ptoFin = vp.toMapPoint(e.getPoint());
			double distX = ptoFin.getX() - ptoIni.getX();
			double distY = ptoFin.getY() - ptoIni.getY();
			if(this.lyrGeoRaster != null){
				Extent ext = new Extent(lyrGeoRaster.getAssignExtent().getMin().getX() + distX, 
						lyrGeoRaster.getAssignExtent().getMin().getY() + distY,
						lyrGeoRaster.getAssignExtent().getMax().getX() + distX, 
						lyrGeoRaster.getAssignExtent().getMax().getY() + distY);
			
				//Asignamos el nuevo extent a la capa
				((FLyrGeoRaster)getMapControl().getMapContext().getLayers().getLayer(lyrGeoRaster.getName())).setAssignExtent(ext);
				
				//Si la capa tiene puntos de control actualizamos el punto que tiene coordenadas
				//en pixeles de la imagen.
				for(int i=0; i<lyrGeoRaster.getFLyrPoints().getCountPoints(); i++){
					Point2D center = lyrGeoRaster.img2World(lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint);
					lyrGeoRaster.getFLyrPoints().setMiniExtent(	i,
																center,
																lyrGeoRaster.getGeoDialog().getLeftInitViewport(vp, center, lyrGeoRaster.getFLyrPoints().getPoint(i).leftViewPort, 1),
																false);
					if(i == lyrGeoRaster.getGeoDialog().getSelectedPoint())
						lyrGeoRaster.updateData(i + 1, lyrGeoRaster.getFLyrPoints().getPoint(i).pixelPoint, null, theView);
				}
				theView.getMapControl().getMapContext().invalidate();
							
			}
			isMoveable = false;
		}
		this.lyrGeoRaster = null;
	}
	
	/**
	 * Funcin de pintado del canvas.Pintamos un marco a la imagen para saber 
	 * donde la movemos. 
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if(isMoveable){
			BufferedImage img = getMapControl().getImage();
			
			ViewPort vp = getMapControl().getMapContext().getViewPort();
	
			Rectangle r = new Rectangle();
			
			Point2D ul = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMin().getX(), 
										 lyrGeoRaster.getAssignExtent().getMin().getY());
			Point2D lr = vp.fromMapPoint(lyrGeoRaster.getAssignExtent().getMax().getX(), 
										 lyrGeoRaster.getAssignExtent().getMax().getY());
			r.setFrameFromDiagonal(ul, lr);
			
			Point2D ini = vp.fromMapPoint(ptoIni.getX() , ptoIni.getY());
			Point2D fin = vp.fromMapPoint(ptoFin.getX() , ptoFin.getY());
			
			double distX = fin.getX() - ini.getX();
			double distY = fin.getY() - ini.getY();
			
			g.drawImage(img, 0, 0, null);
			g.setColor(Color.red);
	
			//g.drawImage(miniGeoRaster, r.x + (int)distX, r.y + (int)distY, null);
			g.drawRect(r.x + (int)distX, r.y + (int)distY, r.width, r.height);
		}
	}
	
	/**
	 * Esta funcin repinta el canvas si se est arrasrtando la imagen para
	 * poder ver el marco de arrastre.
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		if(isMoveable){
			ViewPort vp = getMapControl().getMapContext().getViewPort();
			ptoFin = vp.toMapPoint(e.getPoint());
			getMapControl().repaint();
		}
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getMapControl()
	 */
	public MapControl getMapControl() {
		// TODO Auto-generated method stub
		return super.getMapControl();
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) throws BehaviorException {
		// TODO Auto-generated method stub
		super.mouseClicked(e);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) throws BehaviorException {
		// TODO Auto-generated method stub
		super.mouseEntered(e);
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) throws BehaviorException {
		// TODO Auto-generated method stub
		super.mouseExited(e);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#mouseWheelMoved(java.awt.event.MouseWheelEvent)
	 */
	public void mouseWheelMoved(MouseWheelEvent e) throws BehaviorException {
		// TODO Auto-generated method stub
		super.mouseWheelMoved(e);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setMapControl(com.iver.cit.gvsig.fmap.MapControl)
	 */
	public void setMapControl(MapControl mc) {
		// TODO Auto-generated method stub
		super.setMapControl(mc);
	}

}
