/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.panels;

import java.awt.Component;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;
import org.gvsig.georeferencing.gui.options.GeorefTableOptionsPanel;
import org.gvsig.georeferencing.gui.pointsTable.TablePointsPanel;
import org.gvsig.georeferencing.gui.selectPoints.DataPointPanel;
import org.gvsig.georeferencing.gui.selectPoints.SelectPointsPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.View;

public class DataPointsTabPanel extends JPanel implements ChangeListener{

	private GeoreferencingDialog 	parent = null;
	private SelectPointsPanel 		selectPointsPanel = null;
	private TablePointsPanel 		tablePointsPanel = null;
	private GeorefOptionsPanel 		georefOptionsPanel = null;
	private GeorefTableOptionsPanel georefTableOptionsPanel = null;
	private JTabbedPane 			tbPoints = null;
	private int 					widthTab = 390, heightTab = 205;
	
	/**
	 * This is the default constructor
	 */
	public DataPointsTabPanel(GeoreferencingDialog p) {
		super();
		this.parent = p;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setPreferredSize(new java.awt.Dimension(395,210));
        this.setSize(new java.awt.Dimension(395,210));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getTbPoints(), null);
	}

	/**
	 * @return Returns the selectPointsPanel.
	 */
	public TablePointsPanel getTablePointsPanel() {
		if(tablePointsPanel == null){
			tablePointsPanel = new TablePointsPanel(parent);
		}
		return tablePointsPanel;
	}
	
	/**
	 * @return Returns the selectPointsPanel.
	 */
	public SelectPointsPanel getSelectPointsPanel() {
		if(selectPointsPanel == null){
			selectPointsPanel = new SelectPointsPanel(parent);
		}
		return selectPointsPanel;
	}
	
	/**
	 * @return Returns the georefOptionsPanel.
	 */
	public GeorefOptionsPanel getGeorefOptionsPanel() {
		if(georefOptionsPanel == null){
			georefOptionsPanel = new GeorefOptionsPanel(parent);
		}
		return georefOptionsPanel;
	}

	/**
	 * @return Returns the georefTableOptionsPanel.
	 */
	public GeorefTableOptionsPanel getGeorefTableOptionsPanel() {
		if(georefTableOptionsPanel == null){
			georefTableOptionsPanel = new GeorefTableOptionsPanel(parent);
		}
		return georefTableOptionsPanel;
	}
	
	/**
	 * This method initializes jTabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	public JTabbedPane getTbPoints() {
		if (tbPoints == null) {
			tbPoints = new JTabbedPane();
			tbPoints.setSize(widthTab, heightTab);
			tbPoints.setPreferredSize(new java.awt.Dimension(390,205));
			tbPoints.addTab(PluginServices.getText(this, "punto"), this.getSelectPointsPanel());
			tbPoints.addTab(PluginServices.getText(this,"table"), this.getTablePointsPanel());
			tbPoints.addTab(PluginServices.getText(this,"options"), this.getGeorefOptionsPanel());
			//tbPoints.addTab(PluginServices.getText(this,"options"), this.getGeorefTableOptionsPanel());
			tbPoints.addChangeListener(this);
		}
		return tbPoints;
	}

	/**
	 * Cuando cambiamos de tab controlamos lo siguiente:
	 * <UL>
	 * <LI>Comprobamos que los TextField del selector de puntos no esten en 
	 * blanco</LI>
	 * <LI>Controlamos que la ventana sea redimensionable solo cuando 
	 * est en el tab de tabla</LI>
	 * <LI>Actualizamos los controles de la ventana a la que vamos con lo
	 * que se seleccion en la ventana de la que venimos</LI>
	 * </UL>
	 */
	public void stateChanged(ChangeEvent ev) {
				
		if (((JTabbedPane)ev.getSource()).getSelectedIndex() == 0 || ((JTabbedPane)ev.getSource()).getSelectedIndex() == 2){
			parent.getFrame().setResizable(false);
			if(getTablePointsPanel().getTableControlerPanel().getSelectedIndex() != -1)
				getSelectPointsPanel().updateComboPoint(
						getTablePointsPanel().getTableControlerPanel().getSelectedIndex()
						);
			//Ponemos los paneles en su tamao original
			parent.resetSize();
		}
		
		if (((JTabbedPane)ev.getSource()).getSelectedIndex() == 1){			
			//Si la ventana ha sido redimensionada ponemos la dimensin temporal
		
			if(parent.getTmpWidth() > parent.getNormalWidth()){
				
				if(parent.isEnlarge()){
					parent.getFrame().setSize(parent.getTmpWidth(), parent.getEnlargeHeight() + 10);
					parent.newFrameSize(parent.getTmpWidth(), parent.getEnlargeHeight());
				}else{
					parent.getFrame().setSize(parent.getTmpWidth(), parent.getNormalHeight() + 10);
					parent.newFrameSize(parent.getTmpWidth(), parent.getNormalHeight());
				}
			}
			
			try{
				getSelectPointsPanel().checkInfoPointText();
			}catch(NoSuchFieldException exc){
				//Si no se han salvado los valores de un punto cuando se activa
				//el cambio de tab las coordenadas a las que no se les ha dado 
				//valor se pondrn a 0 y se guardarn.
				double tx = 0D, ty = 0D, lon = 0D, lat = 0D; 
				try{
					DataPointPanel dp = getSelectPointsPanel().getDataPointPanel();
					if(dp.getTX() != null && !dp.getTX().equals(""))
						tx = Double.valueOf(getSelectPointsPanel().getDataPointPanel().getTX().getText()).doubleValue();
					if(dp.getTY() != null && !dp.getTY().equals(""))
						ty = Double.valueOf(getSelectPointsPanel().getDataPointPanel().getTY().getText()).doubleValue();
					if(dp.getLongitud() != null && !dp.getLongitud().equals(""))
						lon = Double.valueOf(getSelectPointsPanel().getDataPointPanel().getLongitud().getText()).doubleValue();
					if(dp.getLatitud() != null && !dp.getLatitud().equals(""))
						lat = Double.valueOf(getSelectPointsPanel().getDataPointPanel().getLatitud().getText()).doubleValue();
				}catch(NumberFormatException ex){
					
				}
				Point2D pixel = new Point2D.Double();
				pixel.setLocation(tx, ty);
				Point2D map = new Point2D.Double();
				map.setLocation(lon, lat);
				
				View  theView = null;
    			try{
    				theView = (View)PluginServices.getMDIManager().getActiveWindow();
    			}catch(ClassCastException exce){
    				return;
    			}
				
				if(parent.getLyrGeoRaster() != null){
					try{
						parent.getPointManager().updateData(getSelectPointsPanel().getTableControlerPanel().getCPoint().getSelectedIndex() + 1,
												pixel,
												map,
												parent,
												theView);
					}catch(ArrayIndexOutOfBoundsException ex){
						//Si da una excepcin de este tipo es que no hay cargado ninn punto
						//por lo que no hacemos nada.
					}catch(ClassCastException ex){
						//Si da una excepcin de este tipo lo ms probable es que no haya una
						//vista activa y no se pueda crear una capa de puntos. Tampoco se hace nada. 
					}
				}else{ 
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
							PluginServices.getText(this, "error_capa_puntos"));
				}

			}
			
			if(getSelectPointsPanel().getTableControlerPanel().getSelectedIndex() != -1)
				getTablePointsPanel().updateComboPoint(
					getSelectPointsPanel().getTableControlerPanel().getSelectedIndex()
					);
			
			parent.getFrame().setResizable(true);
		}
			
	}
	
	/**
	 * Calculo del nuevo tamao a partir de un frame redimensionado
	 * @param w Ancho del frame
	 * @param h Alto del frame
	 */
	public void newFrameSize(int w, int h){
		int newWidth = w;
				
        this.setSize(new java.awt.Dimension(newWidth, heightTab));
        this.setPreferredSize(new java.awt.Dimension(newWidth, heightTab));
        this.getTbPoints().setSize(new java.awt.Dimension(newWidth, heightTab));
        this.getTbPoints().setPreferredSize(new java.awt.Dimension(newWidth, heightTab));
        this.getTablePointsPanel().newFrameSize(newWidth, h);
	}
	
}
