/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.pointsTable;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;

/**
 * Panel que contiene la tabla de puntos con sus controles. 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class TablePointsPanel extends JPanel implements ActionListener{
	
	//**********************Vars**********************************
	private GeoreferencingDialog 			parent = null;
	private JPanel 							pTable = null;
	private JPanel 							pPointSelection = null;
	private TableControlerPanel 			tableControlerPanel = null;
	private int 							widthGeneralPanel = 390, heightGeneralPanel = 175;
	private int 							widthSupPanel = 375, heightSupPanel = 137;
	private int 							widthInfPanel = 375, heightInfPanel = 27;
	private int 							widthTable = 375, heightTable = 110;
	private FLyrPoints 						lyrPoints = null;
	private int 							lastIndexSelection = -1;
	private boolean 						disableEvent = false;
	final static String[] 					columnNames = {"", "N", "X", "Y", 
														"X'", "Y'", "Error X",  
														"Error Y",  "RMS"};
	private PointTable 						table = null;
	private TablePanelButtons	 			pTablePanelButtons = null;
	/**
	 * Estado de los botones cuando se ejecuta disableAllControls
	 */
	private boolean[] 						buttonsState = new boolean[3];
	/**
	 * DisableAllControls ha sido ejecutada si est a true esta variabled
	 */
	private boolean 						disableAllControls = false;
	//**********************End Vars******************************
	
	//**********************Classes*******************************
	/**
	 * Clase que representa la tabla de puntos
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
	public class PointTable extends JPanel {
	   
		private JTable table = null;
		private JScrollPane scrollPane = null;
		
		private TableModelPoint tabModel = null;
		
		public TableModelPoint getTableModel(){
			return tabModel;
		}
		
	    public PointTable() {
	        super(new GridLayout(1,0));
	       
	        tabModel = new TableModelPoint();
	        table = new JTable(tabModel);
	        table.setPreferredScrollableViewportSize(new Dimension(500, 70));

	       
	        table.addMouseListener(new MouseAdapter() {
	           public void mouseClicked(MouseEvent e) {
	        	   //Al seleccionar una fila de la tabla se cambia al punto correspondiente. 
	        	   tableControlerPanel.setSelectedIndex(table.getSelectedRow());
		           parent.getPointManager().selectPoint(table.getSelectedRow(), parent);
		           tableControlerPanel.checkArrows();
	        	 
		           //Si hemos pinchado el checkBox cambiamos el valor de este y ponemos activo/desactivo el punto
	        	   if(table.getSelectedColumn() == 0){
	        		   boolean value = (((Boolean)tabModel.getValueAt(table.getSelectedRow(), 0)).booleanValue());
	        		   //tabModel.setValueAt(new Boolean(!value), table.getSelectedRow(), 0);
	        		   try{
	        			   parent.deactivatePanel((com.iver.cit.gvsig.gui.View)PluginServices.getMDIManager().getActiveWindow(), new JCheckBox("", !value));
	        		   }catch(ClassCastException exc){
	        			   //Si no podemos obtener la vista dejamos el control como estaba
	        			   tabModel.setValueAt(new Boolean(value), table.getSelectedRow(), 0);
	        			   return;
	        		   }catch(Exception exc){
	        			   //Si no podemos obtener la vista dejamos el control como estaba
	        			   tabModel.setValueAt(new Boolean(value), table.getSelectedRow(), 0);
	        			   return;
	        		   }
	        	   }
	        	   
	           }
	        });

	        scrollPane = new JScrollPane(table);
	        
	        TableColumn column = null;
            for (int i = 0; i < table.getModel().getColumnCount(); i++) {
            	column = table.getColumnModel().getColumn(i);
            	if(i == 0){
	                column.setCellRenderer(new CheckColumnRenderer());
	                column.setCellEditor(new CheckColumnEditor());
	                column.setMaxWidth(19);
	                column.setMinWidth(19);
            	}
            	if(i == 1){
	                column.setMaxWidth(25);
	                column.setMinWidth(25);
            	}
            	if((i == 2) || (i == 3)){
            		column.setMinWidth(50);
	                column.setMaxWidth(140);
            	}
            	if((i == 4) || (i == 5)){
            		column.setMinWidth(80);
	                column.setMaxWidth(190);
            	}
            }
            
	        add(scrollPane);
	    }

	    /**
	     * TableModel del JTable que contiene los puntos con sus errores
	     * @author Nacho Brodin (brodin_ign@gva.es)
	     */
	    public class TableModelPoint extends DefaultTableModel {
	    	 
	        public TableModelPoint(){
	        	super(new Object[0][9], columnNames);
	        }
	        
	        public int getColumnCount() {
	            return columnNames.length;
	        }

	        public String getColumnName(int col) {
	            return columnNames[col];
	        }

	        public Class getColumnClass(int c) {
	        	if (c == 0)
	        		return Boolean.class;
	        	return String.class;
	        }

	        public void addNew() {
	            super.addRow(new Object[] {new Boolean(true), "", "", "", "", "", "", "", ""});
	        }
	        
	        public void setValueAt(Object value, int row, int col) { 
	        	super.setValueAt(value, row, col);
	        }
	        	        
	        /**
	         * Devuelve true si la celda es editable. Solo son editables
	         * las celdas de los puntos.
	         */
	        public boolean isCellEditable(int row, int col){
	        	return false;
	        }
	    }
	    
	    class CheckColumnRenderer extends JCheckBox implements TableCellRenderer {
	        final private static long serialVersionUID = -3370601314380922368L;

	        public Component getTableCellRendererComponent(JTable table,
	                                                       Object value,
	                                                       boolean isSelected,
	                                                       boolean hasFocus,
	                                                       int row, int column) {
	            if (value == null) {
	                this.setSelected(false);
	            }

	            Boolean ValueAsBoolean = (Boolean) value;
	            this.setSelected(ValueAsBoolean.booleanValue());
	            this.setHorizontalAlignment(SwingConstants.CENTER);

	            return this;
	        }
	    }
	    
	    class CheckColumnEditor extends AbstractCellEditor implements TableCellEditor {
	    	final private static long serialVersionUID = -3370601314380922368L;
	    	public JCheckBox checkBox;

	        public CheckColumnEditor() {
	            super();
	            checkBox = new JCheckBox();
	            checkBox.addActionListener(new ActionListener() {
	                    public void actionPerformed(ActionEvent event) {
	                    	 //checkBox.setSelected(!checkBox.isSelected());
	                        fireEditingStopped();
	                    }
	                });
	        }

	        public Component getTableCellEditorComponent(JTable table, Object obj,
	                                                     boolean isSelected,
	                                                     int row, int col) {
	        	checkBox.setHorizontalAlignment(SwingUtilities.CENTER);
	
	            Boolean lValueAsBoolean = (Boolean) obj;
	            checkBox.setSelected(lValueAsBoolean.booleanValue());
	
	            return checkBox;
	        }

	        public Object getCellEditorValue() {
	            return new Boolean(checkBox.isSelected());
	        }
	    }
	    
		/**
		 * @return Returns the table.
		 */
		public JTable getTable() {
			return table;
		}
	}
	//**********************End Classes*****************************
	
	//**********************Methods*********************************
	/**
	 * This is the default constructor
	 */
	public TablePointsPanel(GeoreferencingDialog p) {
		super();
		this.parent = p;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
		private void initialize() {
        GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        this.setPreferredSize(new java.awt.Dimension(390,175));
        this.setSize(new java.awt.Dimension(390,175));
        this.setLocation(new java.awt.Point(0,0));
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 1;
        gridBagConstraints5.insets = new java.awt.Insets(2,0,0,0);
			this.add(getPTable(), gridBagConstraints1);
			this.add(getPPointSelection(), gridBagConstraints5);
	}

	/**
	 * Actualiza el nmero de puntos del combo a partir de los contenidos en
	 * la capa de puntos. Esta funcin es usada desde otros paneles que cambien
	 * el punto seleccionado para comunicar a la tabla la nueva seleccin.
	 */
	public void updateComboPoint(int pointSelected){
		try{
			loadLyrPoint();
		}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    		return;
    	}
		
		disableEvent = true;
		tableControlerPanel.setNItems(lyrPoints.getCountPoints());
		try{
			tableControlerPanel.getCPoint().setSelectedIndex(pointSelected);
		}catch(IllegalArgumentException ex){
			
		}
		getJTable().getTable().setRowSelectionInterval(pointSelected, pointSelected);
		tableControlerPanel.checkArrows();
		disableEvent = false;
	}
	
	/**
	 * Obtiene una capa de puntos. Si no existe crea una.
	 * @return
	 */
	private void loadLyrPoint() throws InstantiationException{
			
		if(lyrPoints == null){
			if(lyrPoints == null){
				lyrPoints = this.parent.getLyrPoints();
				if(lyrPoints == null)
					throw new InstantiationException();
			}
		}

	}
	
	/**
	 * Comprueba si los valores de un punto en los campos de la tabla estn
	 * vacios y lanza una excepcin si hay alguno sin informacin.
	 * @throws NoSuchFieldException
	 */
	private void checkInfoPointText(int nPunto)throws NoSuchFieldException{
		try{
			if(	getJTable().getTable().getValueAt(nPunto, 1).toString().equals("") ||
				getJTable().getTable().getValueAt(nPunto, 2).toString().equals("") ||
				getJTable().getTable().getValueAt(nPunto, 3).toString().equals("") ||
				getJTable().getTable().getValueAt(nPunto, 4).toString().equals("") ||
				getJTable().getTable().getValueAt(nPunto, 5).toString().equals("") ){
					disableEvent = true;
					tableControlerPanel.setSelectedIndex(lastIndexSelection);
					disableEvent = false;
					throw new NoSuchFieldException();
			}
		}catch(ArrayIndexOutOfBoundsException ex){
			//Si el acceso al elemento de la tabla da error no hacemos nada
		}
	}
	/**
	 * Cuando pasamos de un punto a otro
	 * <UL>
	 * <LI>Cargamos la capa de puntos si no est cargada todavia</LI>
	 * <LI>Comprobamos que el punto desde el que partimos no tenga campos inicio o fin vacios</LI>
	 * <LI>Seleccionamos en el combo en nuevo punto</LI>
	 * <LI>Cargamos la informacin del nuevo punto en los JTextField, mini imagen y seleccionamos la fila de la tabla</LI>
	 * <LI>Chequeamos la flechas para activar o desactivar las que toquen.</LI>
	 * </UL>
	 * @param changeSelection	Cambia la posicin seleccionada del combo si es true
	 * @param pos	Posicin a la que cambia el combo si el parametro chageSelection es true
	 */
	private void changePoint(boolean changeSelection, int pos){
		disableEvent = true;
    	try{
    		loadLyrPoint();
    		checkInfoPointText(pos);			
        	if(changeSelection){
        		lastIndexSelection = pos;
        		tableControlerPanel.setSelectedIndex(pos);
        	}
        	lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), parent);
        	tableControlerPanel.checkArrows();
    	}catch(NoSuchFieldException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "coordenadas_vacias"));
    	}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    	}
    	disableEvent = false;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		//Botn de ir al punto inicial
		if(e.getSource() == tableControlerPanel.getBFirst()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, 0);
		}
		
		//Botn de ir al punto anterior
		if(e.getSource() == tableControlerPanel.getBPrev()){
			if(tableControlerPanel.getSelectedIndex() != 0)		        	 
		       	changePoint(true, tableControlerPanel.getSelectedIndex() - 1);
		}
		
		//Botn de ir al punto siguiente
		if(e.getSource() == tableControlerPanel.getBNext()){
			if(tableControlerPanel.getSelectedIndex() != (tableControlerPanel.getItemCount() - 1))
	        	changePoint(true, tableControlerPanel.getSelectedIndex() + 1);
		
		}
		
		//Botn de ir al punto final
		if(e.getSource() == tableControlerPanel.getBLast()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, tableControlerPanel.getItemCount() - 1);
		}
		
		//Combo de seleccin de punto
		if(e.getSource() == tableControlerPanel.getCPoint()){
			if(tableControlerPanel.getSelectedIndex() != -1){
				if(!disableEvent){
					changePoint(true,tableControlerPanel.getSelectedIndex());	
				}
			}
		}
	
	}
	
	/**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;
			
			//Salvamos los estados
			buttonsState[0] = getPTablePanelButtons().getBSave().isEnabled();
			buttonsState[1] = getPTablePanelButtons().getBLoad().isEnabled();
			buttonsState[2] = getPTablePanelButtons().getBExportToAscii().isEnabled();
			
			//Desactivamos controles
			getPTablePanelButtons().getBSave().setEnabled(false);
			getPTablePanelButtons().getBLoad().setEnabled(false);
			getPTablePanelButtons().getBExportToAscii().setEnabled(false);
		}
	}
	
	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin 
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			getPTablePanelButtons().getBSave().setEnabled(buttonsState[0]);
			getPTablePanelButtons().getBLoad().setEnabled(buttonsState[1]);
			getPTablePanelButtons().getBExportToAscii().setEnabled(buttonsState[2]);
		}
	}
	//**********************End Methods*****************************
	
	//**********************Setters & Getters***********************
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public JPanel getPTable() {
		if (pTable == null) {
			BorderLayout borderLayout4 = new BorderLayout();
			pTable = new JPanel();
			pTable.setLayout(borderLayout4);
			pTable.setSize(new java.awt.Dimension(widthSupPanel,heightSupPanel));
			pTable.setPreferredSize(new java.awt.Dimension(375,137));
			pTable.add(getJTable(), BorderLayout.CENTER);
		}
		return pTable;
	}
	
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public JPanel getPPointSelection() {
		if (pPointSelection == null) {
			pPointSelection = new JPanel();
			pPointSelection.setPreferredSize(new java.awt.Dimension(375,27));
			if(this.tableControlerPanel == null){
				FlowLayout flowLayout6 = new FlowLayout();
				pPointSelection.setLayout(flowLayout6);
				tableControlerPanel = new TableControlerPanel();
				flowLayout6.setAlignment(java.awt.FlowLayout.LEFT);
				flowLayout6.setHgap(0);
				flowLayout6.setVgap(1);
				pPointSelection.add(tableControlerPanel, null);
				pPointSelection.add(getPTablePanelButtons(), null);
				pPointSelection.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED));
				tableControlerPanel.getBFirst().addActionListener(this);
				tableControlerPanel.getBLast().addActionListener(this);
				tableControlerPanel.getBNext().addActionListener(this);
				tableControlerPanel.getBPrev().addActionListener(this);
				tableControlerPanel.getCPoint().addActionListener(this);
			}
		}
		return pPointSelection;
	}
	
	/**
	 * This method initializes jTable	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	public PointTable getJTable() {
		if (table == null) {
			table = new PointTable();
			table.setSize(new java.awt.Dimension(widthTable, heightTable));
			table.setPreferredSize(new java.awt.Dimension(widthTable, heightTable));
		}
		return table;
	}
	
	/**
	 * Elimina todas las filas de la tabla.
	 */
	public void clearTable(){
		int l = table.getTable().getRowCount();
		this.setSaveToDisk(false);
		for(int i=0;i<l;i++)
			((DefaultTableModel)table.getTable().getModel()).removeRow(0);
	}

	/**
	 * @return Returns the tableControlerPanel.
	 */
	public TableControlerPanel getTableControlerPanel() {
		return tableControlerPanel;
	}
	
	/**
	 * Asigna una tabla
	 * @param table
	 */
	public void setTable(PointTable table){
		this.table = table;
		table.setSize(new java.awt.Dimension(widthTable, heightTable));
		table.setPreferredSize(new java.awt.Dimension(widthTable, heightTable));
	}
	
	/**
	 * Calculo del nuevo tamao a partir de un frame redimensionado
	 * @param w Ancho del frame
	 * @param h Alto del frame
	 */
	public void newFrameSize(int w, int h){
		int newWidth = w;
		double coef = 0.95;
		
        this.setSize(new java.awt.Dimension(newWidth, heightGeneralPanel));
        this.setPreferredSize(new java.awt.Dimension(newWidth, heightGeneralPanel));
        
        getPTable().setSize(new java.awt.Dimension((int)(coef * newWidth),heightSupPanel));
        getPTable().setPreferredSize(new java.awt.Dimension((int)(coef * newWidth),heightSupPanel));
                
        getJTable().setSize(new java.awt.Dimension((int)(coef * newWidth), heightTable));
		getJTable().setPreferredSize(new java.awt.Dimension((int)(coef * newWidth), heightTable));
		
		getPPointSelection().setSize(new java.awt.Dimension((int)(coef * newWidth), heightInfPanel));
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public TablePanelButtons getPTablePanelButtons() {
		if (pTablePanelButtons == null) {
			pTablePanelButtons = new TablePanelButtons();
		}
		return pTablePanelButtons;
	}
	
	/**
	 * Activa o desactiva los botones de exportar capa de puntos y salvar
	 * la tabla a Ascii.
	 *@param isPossible Si vale true activa los controles de salvar y exportacin
	 *y si vale false los desactiva
	 */
	public void setSaveToDisk(boolean isPossible){
		this.getPTablePanelButtons().setSaveToDisk(isPossible);
	}
	//**********************End Setters & Getters*******************
}
