/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.selectPoints;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;
import java.beans.PropertyVetoException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import org.gvsig.georeferencing.GeoreferencingToolsModule;
import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;
import org.gvsig.georeferencing.gui.listeners.GeorefPointSelectorListener;
import org.gvsig.georeferencing.gui.pointsTable.TableControlerPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.gui.View;

public class SelectPointsPanel extends JPanel implements ActionListener, KeyListener, FocusListener{

	//**********************Vars**********************************
	private String					pathToImages = "images/"; //"/com/iver/cit/gvsig/gui/panels/images/";
	private JPanel 					pGeneral = null;
	private JPanel 					pPointSelection = null;
	private JPanel 					pControlPoint = null;
	private JToggleButton 			bInsertPoint = null;
	private FLyrPoints 				lyrPoints = null;
	//private boolean 				disableEvent = false;
	private int						lastIndexSelection = -1;
	private JButton 				bExtended = null;
	private JPanel 					pExtend = null;
	private TableControlerPanel 	tableControlerPanel = null;
	private DataPointPanel 			dataPointPanel = null;
	private ErrorPointPanel 		errorPointPanel = null;
	private GeoreferencingDialog	dialog = null;
	private boolean 				update = false;
	/**
	 * Valores ltimos que se han introducido en los JTextField para poder reponerlos.
	 */
	public String 					lastTx = "", lastTy = "", lastLong = "", lastLat = "";
	/**
	 * Valores ltimos que se han introducido parcialmente en los JTextField para poder reponerlos si
	 * al pulsar una tecla se introduce un valor erroneo.
	 */
	public String 					lastBlockTx = "", lastBlockTy = "", lastBlockLong = "", lastBlockLat = "";
	//**********************End Vars********************************
	
	//**********************Methods*********************************
	/**
	 * Contructor. Inicializa el panel.
	 */
	public SelectPointsPanel() {
		super();
		initialize();
	}
	
	/**
	 * This is the default constructor
	 */
	public SelectPointsPanel(GeoreferencingDialog grd) {
		super();
		initialize();
		this.dialog = grd;
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        FlowLayout flowLayout2 = new FlowLayout();
        flowLayout2.setHgap(5);
        this.setLayout(flowLayout2);
        this.setPreferredSize(new java.awt.Dimension(390,175));
        this.setSize(new java.awt.Dimension(390,175));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPGeneral(), null);
	}
		
	/**
	 * Actualiza el nmero de puntos del combo a partir de los contenidos en
	 * la capa de puntos. Esta funcin es usada desde otros paneles que cambien
	 * el punto seleccionado para comunicar al control la nueva seleccin.
	 */
	public void updateComboPoint(int pointSelected){
		//disableEvent = true;
		try{
			loadLyrPoint();
		}catch(InstantiationException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
    		return;
    	}
		
		tableControlerPanel.setNItems(lyrPoints.getCountPoints());
		try{
			tableControlerPanel.getCPoint().setSelectedIndex(pointSelected);
		}catch(IllegalArgumentException ex){
			
		}
		tableControlerPanel.checkArrows();
		lyrPoints.getPointManager().selectPoint(pointSelected, this.dialog);
		//disableEvent = false;
	}
	
	/**
	 * Comprueba si los valores de un punto en los campos de texto estn
	 * vacios y lanza una excepcin si hay alguno sin informacin.
	 * @throws NoSuchFieldException
	 */
	public void checkInfoPointText()throws NoSuchFieldException{
		if(	dataPointPanel.getTX().getText().equals("") ||
				dataPointPanel.getTY().getText().equals("") ||
				dataPointPanel.getLatitud().getText().equals("") ||
				dataPointPanel.getLongitud().getText().equals("")){
				//disableEvent = true;
				try{
					tableControlerPanel.getCPoint().setSelectedIndex(lastIndexSelection);
					//disableEvent = false;
				}catch(IllegalArgumentException ex){
					//Si el lastIndex tiene un valor erroneo no hacemos ninguna seleccin
				}
				throw new NoSuchFieldException();
		}
	}
		
	/**
	 * Esta funcin resetea los controles del panel de info de un punto.
	 */
	public void resetControls(boolean active){
		dataPointPanel.resetControls(null);
		errorPointPanel.resetControls(null);
		getBInsertPoint().setEnabled(active);
		getTableControlerPanel().getBClear().setEnabled(active);
		getTableControlerPanel().getBDelPoint().setEnabled(active);
		if(!active)
			getTableControlerPanel().resetControls();
	}
	
	/**
	 * Cuando pasamos de un punto a otro o se va a crear uno nuevo
	 * <UL>
	 * <LI>Cargamos la capa FLyrPoint si no est ya cargada.</LI>
	 * <LI>Comprobamos que los campos no esten vacios y lanzamos una excepcin si hay algn campo vacio si lo estn.</LI>
	 * <LI>Insertamos el nuevo punto seleccionado en los campos de informacin.</LI>
	 * <LI>Checkeamos las flechas para ver si tenemos que activarlas o desactivarlas.</LI>
	 * </UL>
	 * @param changeSelection	Cambia la posicin seleccionada del combo si es true
	 * @param pos	Posicin a la que cambia el combo si el parametro chageSelection es true
	 */
	public void changePoint(boolean changeSelection, int pos){
    	try{
    		loadLyrPoint();
    		checkInfoPointText();			
        	if(changeSelection){
        		lastIndexSelection = pos;
        		tableControlerPanel.setSelectedIndex(pos);
        	}
        	lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), dialog);
        	tableControlerPanel.checkArrows();
    	}catch(NoSuchFieldException exc){
    		JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "coordenadas_vacias"));
    	}catch(InstantiationException e){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_capa_puntos"));
		}
	}
	
	/**
	 * Obtiene una capa de puntos. Si no existe crea una.
	 * @return
	 */
	private void loadLyrPoint() throws InstantiationException{
		if(lyrPoints == null){
			lyrPoints = dialog.getLyrPoints();
			if(lyrPoints == null)
				throw new InstantiationException();
		}
	}
	
	private void dialogWithoutImageError(){
		
	}
	/**
	 * Este mtodo inicializa el botn seleccionar desde la vista. Cuando lo 
	 * activamos se carga la herramienta de seleccin de puntos desde la vista. 
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JToggleButton getBInsertPoint() {
		if (bInsertPoint == null) {
			bInsertPoint = new JToggleButton();
			bInsertPoint.setText("");
			bInsertPoint.setPreferredSize(new java.awt.Dimension(22,22));
			try{
				bInsertPoint.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "selectfromview.png")));
			}catch(NullPointerException e){
				//Sin icono
			}
			bInsertPoint.setToolTipText(PluginServices.getText(this, "selectfromview"));
			bInsertPoint.setEnabled(false);
			bInsertPoint.setSelected(true);
			bInsertPoint.addActionListener(this);
		}
		return bInsertPoint;
	}
			
	/**
	 * Control de eventos de TableControlerPanel
	 */
	public void actionPerformed(ActionEvent e) {
		
		//Botn de ir al punto inicial
		if(e.getSource() == tableControlerPanel.getBFirst()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, 0);
		}
		
		//Botn de ir al punto anterior
		if(e.getSource() == tableControlerPanel.getBPrev()){
			if(tableControlerPanel.getSelectedIndex() != 0)		        	 
		       	changePoint(true, tableControlerPanel.getSelectedIndex() - 1);
		}
		
		//Botn de ir al punto siguiente
		if(e.getSource() == tableControlerPanel.getBNext()){
			if(tableControlerPanel.getSelectedIndex() != (tableControlerPanel.getItemCount() - 1))
	        	changePoint(true, tableControlerPanel.getSelectedIndex() + 1);
		
		}
		
		//Botn de ir al punto final
		if(e.getSource() == tableControlerPanel.getBLast()){
			if(tableControlerPanel.getItemCount() >= 1)
				changePoint(true, tableControlerPanel.getItemCount() - 1);
		}
		
		//Combo de seleccin de punto
		if(e.getSource() == tableControlerPanel.getCPoint()){
			if(tableControlerPanel.getSelectedIndex() != -1){
				try{ 
					loadLyrPoint();
				}catch(InstantiationException exc){
					return;
				}
				lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), dialog);
				tableControlerPanel.checkArrows();
			}
		}
		
		//Botn de activar/desactivar la insercin de un punto
		if((e.getSource() == getBInsertPoint()) || (e.getSource() == dialog.getBInsertPointOfTablePoints())){
			View theView = (View) PluginServices.getMDIManager().getActiveWindow();
			if(!bInsertPoint.isSelected()){
				String toolSelected = theView.getMapControl().getCurrentTool();
				if(toolSelected.equals("pointLyrSelection"))
					theView.getMapControl().setTool(lyrPoints.getLastTool());
				else
					theView.getMapControl().setTool("pointLyrSelection");
			}else
				theView.getMapControl().setTool("pointLyrSelection");
		}
		
		//Botn de nuevo punto
		if((e.getSource() == tableControlerPanel.getBNew()) || (e.getSource() == dialog.getBNewPointOfTablePoints())){
			//Comprobamos si est activa la insercin de punto o no. Si no est activa no hacemos nada
			if(bInsertPoint.isSelected()){
				dialog.getPointManager().newEmptyPoint();
				
				//Desactivamos todos los controles para centrar la visin del usuario
				//sobre la introduccin de puntos
				dialog.getPointManager().disableAllControls();
				
				View theView = null;
				try{
					theView = (View) PluginServices.getMDIManager().getActiveWindow();
				}catch(ClassCastException ex){
					//Controlamos que cuado se produzca esta excepcin cerremos el cuadro con un aviso. 
					System.err.println("**Vista asociada al dialogo no encontrada");
					try{
						String string1 = PluginServices.getText(this, "yes");
						Object[] options = {string1};
						JOptionPane.showOptionDialog((Component)PluginServices.getMainFrame(),
								PluginServices.getText(this, "cuadro_no_asociado"),
								PluginServices.getText(this, "confirmacion"),
								JOptionPane.YES_OPTION,
								JOptionPane.QUESTION_MESSAGE,
								null,     
								options,  
								string1); 
						dialog.getFrame().setClosed(true);
						return;
					}catch(PropertyVetoException exc){}
				}
			
				//Salvamos la tool actualmente seleccionada, asignamos el punto y cargamos la herramienta de seleccin de pto.
				try{
					loadLyrPoint();
					lyrPoints.setLastTool(theView.getMapControl().getCurrentTool());
					GeorefPointSelectorListener.posPoint = tableControlerPanel.getSelectedIndex();
					GeorefPointSelectorListener.firstPoint = true;
					theView.getMapControl().setTool("pointLyrSelection");
				}catch(InstantiationException exc){
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
							PluginServices.getText(this, "error_capa_puntos"));
				}
			}
		}
				
		TableControlerPanel ctrlPanel = dialog.getConectorPanel().getDataPointsTabPanel().getTablePointsPanel().getTableControlerPanel();
		
		//Botn de eliminado de todos los puntos
		if(	e.getSource() == getTableControlerPanel().getBClear() ||
			e.getSource() == ctrlPanel.getBClear()){
			dialog.getPointManager().deleteAllPoints();
		}
		
		//Botn de eliminado de un punto
		if(e.getSource() == getTableControlerPanel().getBDelPoint() ||
				e.getSource() == ctrlPanel.getBDelPoint()){
			dialog.getPointManager().deletePoint();
		}
		
		//Botn de extensin del cuadro de dialogo
		if(e.getSource() == this.getBExtended())
			this.setEnlarge(dialog.isEnlarge());
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		// TODO Auto-generated method stu
	}
	
	/**
	 * Checkea un textField para que contenga caracteres validos. Consideramos validos
	 * los caracteres del 0 al 9, el punto, DEL, Supr, flechas y Enter teniendo en cuenta que el 
	 * cdigo ascii del teclado numerico es distinto.
	 */
	public boolean checkTextField(int code){
		
		if(	(code >= 0 && code <= 7) || 
			code == 9 || 
			(code >= 11 && code <= 36) ||
			(code >= 41 && code <= 45) ||
			code == 47 ||
			(code >= 58 && code <= 85) ||
			(code >= 87 && code <= 95) ||
			(code >= 106 && code <= 109) ||
			(code >= 111 && code <= 126) ||
			code >= 128 ){
			return false;
		}
		
		return true;
	}
	
	/**
	 * Control de eventos de las entradas de texto de los puntos
	 */
	public void keyReleased(KeyEvent e) {
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		//Campo de texto de la coordenada X en pixeles
		Point2D pixel = null;
		Point2D map = null;
		
		//Si el evento es sobre TX
		if(e.getSource() == dataPointPanel.getTX()){
			if(e.getKeyCode() == 10){
				txEvent();
				return;
			}
			
			if(checkTextField(e.getKeyCode()))
				lastBlockTx = tx;
			else
				dataPointPanel.getTX().setText(lastBlockTx);
		}
		
		//Si el evento es sobre TY			
		if(e.getSource() == dataPointPanel.getTY()){
			if(e.getKeyCode() == 10){
				tyEvent();
				return;
			}
			
			if(checkTextField(e.getKeyCode()))
				lastBlockTy = ty;
			else
				dataPointPanel.getTY().setText(lastBlockTy);
		}
		
		//Si el evento es sobre Longitud
		if(e.getSource() == dataPointPanel.getLongitud()){
			if(e.getKeyCode() == 10){
				lonEvent();
				return;
			}
			
			if(checkTextField(e.getKeyCode()))
				lastBlockLong = lon;
			else
				dataPointPanel.getLongitud().setText(lastBlockLong);
		}
		
		//Si el evento es sobre Latitud
		if(e.getSource() == dataPointPanel.getLatitud()){
			if(e.getKeyCode() == 10){
				latEvent();
				return;
			}
			
			if(checkTextField(e.getKeyCode()))
				lastBlockLat = lat;
			else
				dataPointPanel.getLatitud().setText(lastBlockLat);
		}
				
	}

	/* (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent arg0) {
		// TODO Auto-generated method stub

	}
	
	/**
	 * Cancela la seleccin de puntos sobre la vista de forma grfica. Esta accin
	 * conllevar seleccionar la ltima herramienta del mapCtrl
	 */
	private void abortSelectPoints(){
		//Si todos los puntos estn rellenos deseleccionamos la herramienta de seleccin de puntos
		GeorefPointSelectorListener.firstPoint = true;
		
		//Activamos los controles de esta ya que habr sido desactivados al llamar a newEmptyPoint
		GeoreferencingToolsModule.setEnabled(true);
		
		com.iver.cit.gvsig.gui.View theView = null;
		try{
			theView = (com.iver.cit.gvsig.gui.View)PluginServices.getMDIManager().getActiveWindow();
			theView.getMapControl().setTool(dialog.getLyrPoints().getLastTool());
		}catch(ClassCastException exc){
			//No se selecciona la herramienta
		}
	}
		
	/**
	 * Validacin de que el parmetro tenga formato numerico y de si 
	 * este valor est dentro de los lmites de la imagen
	 * @param tx Campo TX
	 * @return true si es valido y false si no lo es
	 */
	private boolean validateTX(String tx){
		double x = 0D;
		try{
			x = Double.valueOf(tx).doubleValue();
			if(x > dialog.getLyrGeoRaster().getImageWidth() || x < 0)
				throw new NumberFormatException();
		}catch(NumberFormatException exc){
			//Si no es valido reponemos lastValue
			dataPointPanel.getTX().setText(lastTx);
			return false;
		}
		
		lastTx = tx;
		return true;
	}
	
	
	/**
	 * Validacin de que el parmetro tenga formato numerico y de si 
	 * este valor est dentro de los lmites de la imagen
	 * @param ty Campo TY
	 * @return true si es valido y false si no lo es
	 */
	private boolean validateTY(String ty){
		double y = 0D;
		try{
			y = Double.valueOf(ty).doubleValue();
			if(y > dialog.getLyrGeoRaster().getImageHeight() || y < 0)
				throw new NumberFormatException();
		}catch(NumberFormatException exc){
			//Si no es valido reponemos lastValue
			dataPointPanel.getTY().setText(lastTy);
			return false;
		}
		
		lastTy = ty;
		return true;
	}
	
	/**
	 * Validacin de que el parmetro tenga formato numerico
	 * @param t Valor del campo
	 * @return true si es valido y false si no lo es
	 */
	private boolean validateNumericFormat(String field){
		double d = 0D;
		try{
			d = Double.valueOf(field).doubleValue();
		}catch(NumberFormatException exc){
			return false;
		}
		return true;
	}
	
	/**
	 * Funcin que se ejecuta cuando TX pierde el foco o se pulsa enter sobre
	 * el JTextField
	 */
	private void txEvent(){
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		
		if(!validateTX(tx)) return;
		
		if(!ty.equals("")){
			if(!validateTY(ty)) return;
			else{
				lastBlockTx = "";
				//Llamamos a la funcin que gestiona la entrada del primer punto
				dialog.getPointManager().selectFirstPoint(tx, ty);
			}
		}
	}
	
	/**
	 * Funcin que se ejecuta cuando TY pierde el foco o se pulsa enter sobre
	 * el JTextField
	 */
	private void tyEvent(){
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		
		if(!validateTY(ty)) return;
		
		if(!tx.equals("")){
			if(!validateTX(tx)) return;
			else{
				lastBlockTy = "";
				//Llamamos a la funcin que gestiona la entrada del primer punto
				dialog.getPointManager().selectFirstPoint(tx, ty);
			}
		}
	}
	
	/**
	 * Funcin que se ejecuta cuando el campo de texto Longitud pierde el foco o se 
	 * pulsa enter sobre el JTextField
	 */
	private void lonEvent(){
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		if(!validateNumericFormat(lon))
			return;
		
		lastLong = lon;
		if(!lat.equals("")){
			lastLat = lat;
			lastBlockLat = "";
			//Llamamos a la funcin que gestiona la entrada del segundo punto
			dialog.getPointManager().selectSecondPoint(lon, lat);
		}
	}
	
	/**
	 * Funcin que se ejecuta cuando el campo de texto Latitud pierde el foco o se 
	 * pulsa enter sobre el JTextField
	 */
	private void latEvent(){
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		if(!validateNumericFormat(lon))
			return;
		
		lastLat = lat;
		if(!lon.equals("")){
			lastLong = lon;
			lastBlockLong = "";
			//Llamamos a la funcin que gestiona la entrada del segundo punto
			dialog.getPointManager().selectSecondPoint(lon, lat);
		}
	}
	
	/**
	 * Los campos de texto de las coordenadas cuando pierden el foco actualizan el valor del pixel
	 * en la cap, vista, mini imagen y tabla
	 */
	public void focusLost(FocusEvent e) {
		String tx = dataPointPanel.getTX().getText();
		String ty = dataPointPanel.getTY().getText();
		String lon = dataPointPanel.getLongitud().getText();
		String lat = dataPointPanel.getLatitud().getText();
		
		//Si el elemento que pierde el foco es TX
		if(e.getSource() == dataPointPanel.getTX())
			txEvent();
		
		//Si el elemento que pierde el foco es TY
		if(e.getSource() == dataPointPanel.getTY())
			tyEvent();
		
		//Si el elemento que pierde el foco es Longitud
		if(e.getSource() == dataPointPanel.getLongitud())
			lonEvent();
		
		//Si el elemento que pierde el foco es Latitud
		if(e.getSource() == dataPointPanel.getLatitud())
			latEvent();
			
		//Si estn rellenos todos los campos restauramos los controles
		if(	!dataPointPanel.getTX().getText().equals("") && 
			!dataPointPanel.getTY().getText().equals("") &&
			!dataPointPanel.getLongitud().getText().equals("") && 
			!dataPointPanel.getLatitud().getText().equals("")){
				dialog.getPointManager().restoreControlsValue();
				this.abortSelectPoints();	
		}
				
	}

	//**********************End Methods*******************************
	
	//**********************Setters & Getters*************************
	/**
	 * Extiende o pliega el panel inferior controlando el icono del botn. Si se
	 * asigna true quiere decir que el panel est desplegado, en este caso lo plegar.
	 * Si se asigna false quiere decir que el panel est plegado, en este caso lo 
	 * desplegar.
	 * @param enlarge True para recogerlo y false para desplegarlo
	 */
	public void setEnlarge(boolean enlarge){
		if(enlarge){
			dialog.setEnlarge(!enlarge);
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "down.png")));
		}else{
			dialog.setEnlarge(!enlarge);
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "up.png")));
			if(lyrPoints != null){
				//lyrPoints.getPointManager().selectPoint(tableControlerPanel.getSelectedIndex(), this.grd);
				if(lyrPoints.isNewPoint(dialog.getSelectedPoint())){
					dialog.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().setEnabled(false);
					dialog.getConectorPanel().getAdjustGeorefPanel().getZoomRight().setEnabled(false);
				}
				dialog.getZoomControlRight().draw();
				dialog.getZoomControlLeft().draw();
			}
		}
	}
	
	/**
	 * @return Returns the errorPointPanel.
	 */
	public ErrorPointPanel getErrorPointPanel() {
		if(this.errorPointPanel == null)
			errorPointPanel = new ErrorPointPanel();
			errorPointPanel.setPreferredSize(new java.awt.Dimension(175,130));
		return errorPointPanel;
	}
	/**
	 * @return Returns the lyrPoints.
	 */
	public FLyrPoints getLyrPoints() {
		if(lyrPoints == null){
			try{
				loadLyrPoint();
					}catch(InstantiationException exc){
				JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						PluginServices.getText(this, "error_capa_puntos"));
			}
		}
		return lyrPoints;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBExtended() {
		if (bExtended == null) {
			bExtended = new JButton();
			bExtended.setPreferredSize(new java.awt.Dimension(22,22));
			bExtended.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "down.png")));
			bExtended.setToolTipText(PluginServices.getText(this, "plegar_despl"));
			bExtended.addActionListener(this);
		}
		return bExtended;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPExtend() {
		if (pExtend == null) {
			FlowLayout flowLayout21 = new FlowLayout();
			pExtend = new JPanel();
			pExtend.setLayout(flowLayout21);
			flowLayout21.setHgap(0);
			flowLayout21.setVgap(1);
			flowLayout21.setAlignment(java.awt.FlowLayout.LEFT);
			pExtend.setPreferredSize(new java.awt.Dimension(50,22));
			pExtend.add(getBInsertPoint(), null);
			pExtend.add(getBExtended(), null);
		}
		return pExtend;
	}

	/**
	 * @return Returns the dataPointPanel.
	 */
	public DataPointPanel getDataPointPanel() {
		return dataPointPanel;
	}

	/**
	 * @param dataPointPanel The dataPointPanel to set.
	 */
	public void setDataPointPanel(DataPointPanel dataPointPanel) {
		this.dataPointPanel = dataPointPanel;
	}
	
	/**
	 * @return Returns the tableControlerPanel.
	 */
	public TableControlerPanel getTableControlerPanel() {
		return tableControlerPanel;
	}

	/**
	 * @param tableControlerPanel The tableControlerPanel to set.
	 */
	public void setTableControlerPanel(TableControlerPanel tableControlerPanel) {
		this.tableControlerPanel = tableControlerPanel;
	}
	
	/**
	 * This method initializes pGeneral	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new java.awt.Insets(5,0,5,0);
			gridBagConstraints.gridy = 1;
			gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
			gridBagConstraints.gridheight = 1;
			gridBagConstraints.gridx = 0;
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(390,175));
			gridBagConstraints.gridwidth = 1;
			pGeneral.add(getPPointSelection(), gridBagConstraints);
			pGeneral.add(getPControlPoint(), gridBagConstraints1);
		}
		return pGeneral;
	}

	/**
	 * This method initializes pFileSelection	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPPointSelection() {
		if (pPointSelection == null) {
			//jLabel = new JLabel();
			FlowLayout flowLayout1 = new FlowLayout();
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(0);
			flowLayout.setVgap(1);
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			pPointSelection = new JPanel();
			pPointSelection.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.LOWERED));
			pPointSelection.setName("");
			pPointSelection.setLayout(flowLayout);
			pPointSelection.setPreferredSize(new java.awt.Dimension(375,27));
			if(this.tableControlerPanel == null){
				tableControlerPanel = new TableControlerPanel();
				tableControlerPanel.setLayout(flowLayout1);
				flowLayout1.setVgap(0);
				flowLayout1.setHgap(0);
				
				tableControlerPanel.getBFirst().addActionListener(this);
				tableControlerPanel.getBLast().addActionListener(this);
				tableControlerPanel.getBNext().addActionListener(this);
				tableControlerPanel.getBPrev().addActionListener(this);
				tableControlerPanel.getBNew().addActionListener(this);
				tableControlerPanel.getCPoint().addActionListener(this);
				tableControlerPanel.getBClear().addActionListener(this);
				tableControlerPanel.getBDelPoint().addActionListener(this);
			}

			pPointSelection.add(tableControlerPanel, null);
			pPointSelection.add(getPExtend(), null);
		}
		return pPointSelection;
	}

	/**
	 * This method initializes pControls	
	 * 		
	 */
	private JPanel getPControlPoint() {
		if (pControlPoint == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridy = 0;
			pControlPoint = new JPanel();
			pControlPoint.setLayout(new GridBagLayout());
			pControlPoint.setPreferredSize(new java.awt.Dimension(380,130));
			pControlPoint.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
			gridBagConstraints11.gridy = 0;
			gridBagConstraints11.gridx = 1;
			if(this.dataPointPanel == null){
				dataPointPanel = new DataPointPanel();
				dataPointPanel.setPreferredSize(new java.awt.Dimension(200,130));
				dataPointPanel.getTX().addKeyListener(this);
				dataPointPanel.getTY().addKeyListener(this);
				dataPointPanel.getLatitud().addKeyListener(this);
				dataPointPanel.getLongitud().addKeyListener(this);
				dataPointPanel.getTX().addFocusListener(this);
				dataPointPanel.getTY().addFocusListener(this);
				dataPointPanel.getLatitud().addFocusListener(this);
				dataPointPanel.getLongitud().addFocusListener(this);
			}
			pControlPoint.add(dataPointPanel, gridBagConstraints2);
		
			getErrorPointPanel();
			pControlPoint.add(errorPointPanel, gridBagConstraints11);
		}
		return pControlPoint;
	}
	
	/**
	 * Asigna la ruta donde estn las imagenes
	 * @param pathToImages
	 */
	public void setPathToImages(String pathToImages) {
		this.pathToImages = pathToImages;
	}
	
	/**
	 * Obtiene el ltimo indice seleccionado de la lista de puntos
	 * @return entero con el ltimo indice seleccionado de la lista de puntos
	 */
	public int getLastIndexSelection() {
		return lastIndexSelection;
	}

	/**
	 * Asigna el ltimo indice seleccionado de la lista de puntos
	 * @param entero con el ltimo indice seleccionado de la lista de puntos
	 */
	public void setLastIndexSelection(int lastIndexSelection) {
		this.lastIndexSelection = lastIndexSelection;
	}
	
	/**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	this.dataPointPanel.setEnabled(enabled);
    	this.errorPointPanel.setEnabled(enabled);
    	this.tableControlerPanel.setEnabled(enabled);
    	this.getBInsertPoint().setEnabled(enabled);
    }
	//**********************End Setters & Getters*********************
 }
