/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.wizards;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterFile;
import org.cresques.px.Extent;
import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;
import org.gvsig.georeferencing.utils.GeoLayerFactory;
import org.gvsig.georeferencing.utils.GeoUtils;
import org.gvsig.georeferencing.utils.PointManager;

import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.gui.FOpenDialog;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.panels.CRSSelectPanel;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.gui.wizards.WizardListenerSupport;
 
/**
 * Clase que implementa el Wizard para la georreferenciacin
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoRasterWizard extends WizardPanel{
	
	//**********************Vars**********************************
	private JPanel 					pGeneral = null;
	private JPanel 					pFileSelection = null;
	private JPanel 					pControls = null;
	private JTextField 				tFile = null;
	private JButton 				bSelectFile = null;
	private CRSSelectPanel			pProyection = null;
	private WizardListenerSupport 	listenerSupport = new WizardListenerSupport();
	/**
	 * Lista de formatos soportados
	 */
	private String[] 				fileFilters = {"tif", "tiff", "jpg", "jpeg", "png", "gif", "bmp"};
	/**
	 * Nombre del fichero seleccionado
	 */
	private String 					fName = "";
	
	/**
	 * Nombre de la capa
	 */
	private String 					lyrName = "";
	/**
	 * Recuerda la ltima ruta seleccionada por el usuario
	 */
	//private String 					lastPath = "./";
	private static String 			lastPath = null;
	private FLyrGeoRaster 			lyrGeoRaster = null;
	private JPanel 					pCheckUseGeoref = null;
	private JCheckBox 				cbUseGeoref = null;
	private JLabel 					lUseGeoref = null;
	private double 					widthPxImg, heightPxImg;
	/**
	 * Extent calculado a partir de la vista.
	 */
	private Extent 					ext = null;
	//**********************Vars**********************************

	//**********************Classes*******************************
	/**
	 * Filtro para los ficheros de georreferenciacin.
	 * @author Nacho Brodin (brodin_ign@gva.es)
	 */
    public class DriverFileFilter extends FileFilter{

		public DriverFileFilter(){}

		/**
		 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File f) {
			if (f.isDirectory()) return true;
			for(int i=0; i<fileFilters.length;i++){
				if(f.getName().endsWith(fileFilters[i]))
					return true;
			}
			return false;
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#getDescription()
		 */
		public String getDescription() {
			return PluginServices.getText(this, "georef_file") + " ( tif, jpg, gif, png)";
		}
	}
	//**********************End Classes***************************
	
	//**********************Methods*******************************
	/**
	 * This is the default constructor
	 */
	public GeoRasterWizard() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setPreferredSize(new java.awt.Dimension(750,320));
        this.setSize(new java.awt.Dimension(510,311));
        this.setLocation(new java.awt.Point(0,0));
        this.add(getPGeneral(), null);
        this.getBSelectFile().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	acceptButtonActionPerformed(evt);
            }
        });
	 }
	
	/**
	 * Calcula un extent posible para la imagen a partir del extent de la vista.
	 * En este caso centra la imagen en la vista para que cuando se cargue la imagen
	 * a georreferenciar nos quede centrada si elegimos la opcin de no utilizar el
	 * extent de la imagen.
	 * @param w	Ancho de la imagen
	 * @param h Alto de la imagen
	 * @return	Extent para la imagen
	 */
	protected Extent calcTempExtent(String file, ViewPort vp, IProjection proj){
		
		//Obtenemos el ancho y alto de la imagen y obtenemos un extent a partir
		//del viewport.
		Extent tempExtent = null;
		GeoRasterFile grf = GeoRasterFile.openFile(proj, file);
		try{
			widthPxImg = grf.getWidth();
			heightPxImg = grf.getHeight();
		}catch(NullPointerException exc){
			System.err.println("FileName: "+file);
			exc.printStackTrace();
			return null;
		}
		if(vp == null || vp.getAdjustedExtent() == null){
			vp = new ViewPort(proj);
			Rectangle2D r2d = new Rectangle2D.Double(0, 0, widthPxImg, heightPxImg);
			vp.setExtent(r2d);
			tempExtent = new Extent(0, 0, widthPxImg, heightPxImg);
		}else
			tempExtent = new Extent(vp.getAdjustedExtent());
		//grf.close();		
		
		double ulX = 0D, ulY = 0D, lrX = 0D, lrY = 0D;
		if(widthPxImg > heightPxImg){
			double widthView = tempExtent.maxX() - tempExtent.minX();
			ulX = tempExtent.minX() + (widthView / 4);
			lrX = ulX + (widthView / 2);
			double newAlto = ((heightPxImg * (widthView / 2)) / widthPxImg);
			double centroY = tempExtent.minY()+((tempExtent.maxY() - tempExtent.minY())/2);
			ulY = centroY - (newAlto / 2);
			lrY = centroY + (newAlto / 2);
		}else{
			double heightView = tempExtent.maxY() - tempExtent.minY();
			ulY = tempExtent.minY() + (heightView / 4);
			lrY = ulY + (heightView / 2);
			double newAncho = ((widthPxImg * (heightView / 2)) / heightPxImg);
			double centroX = tempExtent.minX()+((tempExtent.maxX() - tempExtent.minX())/2);
			ulX = centroX - (newAncho / 2);
			lrX = centroX + (newAncho / 2);
		}
		return new Extent(ulX, ulY, lrX, lrY);
	}
	
	/**
	 * Evento de pulsado del botn de seleccionar fichero
	 * @param e
	 */
	private void acceptButtonActionPerformed(ActionEvent e) {
		//Selector de Fichero que se quiere georeferenciar
				
		if(	e.getSource().equals(this.getBSelectFile())){
			JFileChooser chooser = new JFileChooser(lastPath);
			chooser.setDialogTitle(PluginServices.getText(this, "seleccionar_fichero"));
			
			DriverFileFilter f = new DriverFileFilter();
			chooser.addChoosableFileFilter(f);
			
			int returnVal = chooser.showOpenDialog(this);
			if(returnVal == JFileChooser.APPROVE_OPTION){			
			 	this.fName = chooser.getSelectedFile().toString();
			 	FileFilter filter = chooser.getFileFilter();
			 	this.getTFile().setText(fName);
			 	lastPath = chooser.getCurrentDirectory().getAbsolutePath();
			 	
			 	if (PluginServices.getMainFrame() == null) {
                    ((JDialog) (getParent().getParent().getParent()
                            .getParent())).dispose();
                } else {                	
                	//Creamos la capa y la cargamos
                	IProjection proj = this.getPProyection().getCurProj();
                	try{
                		View  theView = null;
            			try{
            				theView = (View)PluginServices.getMDIManager().getActiveWindow();
            			}catch(ClassCastException exc){
            				return;
            			}
                		                		
                		com.hardcode.driverManager.Driver driver = LayerFactory.getDM().getDriver("gvSIG Image Driver");
                		
                		File fich = new File(fName);
                		
						ViewPort viewPort = theView.getMapControl().getMapContext().getViewPort();
						
						//Calcula el extent para que la imagen que se carga quede centrada.
						ext = calcTempExtent( fName, viewPort, proj);
						
						if(ext == null){
							JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
									PluginServices.getText(this, "imagen_no_accesible"));
							return;
						}
						
						//Asigna el nombre de la capa con el marcador *
                		this.setLyrName("*"+fName.substring(fName.lastIndexOf(File.separator)+1));
                		
                		//Antes de crear la capa si tiene georreferenciacin por .rmf la reseteamos
                		if(!this.getCbUseGeoref().isSelected())
                			removeGeoref(fName);
                			
                		//Crea la capa
                		lyrGeoRaster = GeoLayerFactory.createLayer(getLyrName(), 
                												(RasterDriver) driver, 
                												fich, 
                												proj,
                												widthPxImg,
                												heightPxImg);
                		//Pone activa la capa      			
                		if(lyrGeoRaster != null)
                    		lyrGeoRaster.setActive(true);
                    	
                		
                		if(	ext != null && fich != null && 
                			this.lyrGeoRaster != null && !getTFile().getText().equals(""))
                			listenerSupport.callStateChanged(true);
                		
                	}catch(DriverLoadException exc){
                		NotificationManager.addError("No se pudo acceder a los drivers", exc);
                	}catch(DriverException exc){
                		NotificationManager.addError("El driver ha producido un error", exc);
                	}
                   
                }
			 }
	    }	
		
	}
	
	/**
	 * Elimina la georrefenciacin externa de la imagen antes de comenzar a georreferenciar
	 * @param fName Nombre del fichero a georreferenciar.
	 */
	private void removeGeoref(String fName){
		String rmf = fName.substring(0, fName.lastIndexOf(".")) + ".rmf";
		File file = new File(rmf);
		String ext = fName.substring(fName.lastIndexOf(".") + 1, fName.length());
		
		//Hacemos un backup del fichero RMF 
		String backupRmf = GeoUtils.fileBackup(rmf);
    	
		//Eliminamos el fichero rmf y lo volvemos a crear sin la informacin de georreferenciacin
		if(file.exists()){
			file.delete();
			try{
				file.createNewFile();
				String encoding = GeoUtils.readFileEncoding(backupRmf);
				
				FileInputStream fis = new FileInputStream(backupRmf);
				BufferedReader inReader = null;
				BufferedWriter outWriter = null;
								
				if(encoding != null)
					inReader = new BufferedReader(new InputStreamReader(fis, encoding));
				else
					inReader = new BufferedReader(new InputStreamReader(fis, "ISO-8859-15"));
				outWriter = new BufferedWriter(new FileWriter(file, false));
												
			    String line = null;
			    boolean write = true;
				while((line = inReader.readLine()) != null) {
					if(line.startsWith("<FLyrGeoRaster "))
						write = false;
					if(write)
						outWriter.write(line+"\n");
					if(line.startsWith("</FLyrGeoRaster>"))
						write = true;
				}
				inReader.close();
				outWriter.close();
				    
	    	}catch(FileNotFoundException exc){
	    		System.err.print("RMF Backup: File not found, "+fName);
	    		return;
	    	}catch(IOException exc){
	    		System.err.print("RMF Backup: IOException, "+fName);
	    		return;
	    	} 
	    	
		}
		
		//Si tiene worldfile lo renombramos
		String georef = fName.substring(0, fName.lastIndexOf(".")) + GeoUtils.getWorldFileExtensionFromFileName(fName);
		File worldFile = new File(georef);
		if(worldFile.exists()){
			GeoUtils.fileBackup(georef);
			worldFile.delete();
		}
		
	}
	
	/**
	 * Mtodo que se ejecuta al pulsar el botn de aceptar. En este caso asignamos el
	 * extent calculado desde la vista si el checkbox no est marcado y lanzaremos el
	 * dialogo de georeferenciacin. El resto del trabajo lo realiza la clase AddLayer.
	 */
	public void execute() {
		if(!this.getCbUseGeoref().isSelected())
			lyrGeoRaster.setAssignExtent(ext);  
		
		PointManager.initWindow(lyrGeoRaster, true, this);
	}
	   		
	public void initWizard() {
	
	}

	/**
	 * Adds the gvSIG's wizard listener 
	 * 
	 * @param listener
	 */
	public void addWizardListener(WizardListener listener) { 
		listenerSupport.addWizardListener(listener);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#callError(java.lang.Exception)
	 */
	public void callError(Exception descripcion) {
		// TODO Auto-generated method stub
		super.callError(descripcion);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#callStateChanged(boolean)
	 */
	public void callStateChanged(boolean finishable) {
		// TODO Auto-generated method stub
		super.callStateChanged(finishable);
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#removeWizardListener(com.iver.cit.gvsig.gui.wizards.WizardListener)
	 */
	public void removeWizardListener(WizardListener listener) {
		// TODO Auto-generated method stub
		super.removeWizardListener(listener);
	}	
	//**********************End Methods***************************
	
	//**********************Getters & Setters*********************
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#setTabName(java.lang.String)
	 */
	protected void setTabName(String name) {
		// TODO Auto-generated method stub
		super.setTabName(name);
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#getLayer()
	 */
	public FLayer getLayer() {
		if(lyrGeoRaster != null)
			return lyrGeoRaster;
		return null;
	}

	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#getTabName()
	 */
	public String getTabName() {
		return PluginServices.getText(this, "georef");
	}
	
	/**
	 * @return Returns the fName.
	 */
	public String getLyrName() {
		return lyrName;
	}
	
	/**
	 * @param name The fName to set.
	 */
	public void setLyrName(String name) {
		lyrName = name;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPCheckUseGeoref() {
		if (pCheckUseGeoref == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.LEFT);
			lUseGeoref = new JLabel();
			pCheckUseGeoref = new JPanel();
			pCheckUseGeoref.setLayout(flowLayout1);
			pCheckUseGeoref.setPreferredSize(new java.awt.Dimension(370,31));
			lUseGeoref.setText("Usar georeferenciacin de la imagen");
			pCheckUseGeoref.add(getCbUseGeoref(), null);
			pCheckUseGeoref.add(lUseGeoref, null);
		}
		return pCheckUseGeoref;
	}
	
	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getCbUseGeoref() {
		if (cbUseGeoref == null) {
			cbUseGeoref = new JCheckBox();
			cbUseGeoref.setSelected(false);
		}
		return cbUseGeoref;
	}
	
	/**
	 * @return Returns the lyrRaster.
	 */
	public FLyrGeoRaster getFLyrGeoRaster() {
		return lyrGeoRaster;
	}

	/**
	 * @param lyrRaster The lyrRaster to set.
	 */
	public void setFLyrGeoRaster(FLyrGeoRaster lyrGeoRaster) {
		this.lyrGeoRaster = lyrGeoRaster;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(5,0,0,0);
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new java.awt.Insets(0,0,2,0);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
			gridBagConstraints.gridx = 0;
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(750,320));
			gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
			pGeneral.add(getPFileSelection(), gridBagConstraints);
			pGeneral.add(getPControls(), gridBagConstraints1);
		}
		return pGeneral;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPFileSelection() {
		if (pFileSelection == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setVgap(10);
			pFileSelection = new JPanel();
			pFileSelection.setBorder(javax.swing.BorderFactory.createTitledBorder(null, PluginServices.getText(this,"cargar"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			pFileSelection.setLayout(flowLayout);
			pFileSelection.setPreferredSize(new java.awt.Dimension(475,70));
			pFileSelection.add(getTFile(), null);
			pFileSelection.add(getBSelectFile(), null);
		}
		return pFileSelection;
	}

	private JPanel getPControls() {
		if (pControls == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			pControls = new JPanel();
			pControls.setLayout(new GridBagLayout());
			pControls.setPreferredSize(new java.awt.Dimension(475,200));
			pControls.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.gridy = 0;
			pControls.add(getPCheckUseGeoref(), gridBagConstraints11);
			pControls.add(getPProyection(), gridBagConstraints2);
		}
		return pControls;
	}
	
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTFile() {
		if (tFile == null) {
			tFile = new JTextField();
			tFile.setPreferredSize(new java.awt.Dimension(350,25));
		}
		return tFile;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBSelectFile() {
		if (bSelectFile == null) {
			bSelectFile = new JButton();
			bSelectFile.setText(PluginServices.getText(this, "cargar"));
		}
		return bSelectFile;
	}
	
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private CRSSelectPanel getPProyection() {
		if (pProyection == null) {
			pProyection = CRSSelectPanel.getPanel(GeoreferencingDialog.getLastProjection());
			pProyection.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
			        if (pProyection.isOkPressed()) {
			        	FOpenDialog.setLastProjection(pProyection.getCurProj());
			        }
				}
			});
		}
		return pProyection;
	}
	//**********************End Getters & Setters*****************
}

