/*
 * Created on 04-mar-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.jdbc.postgis;

import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.postgis.PGbox2d;
import org.postgis.PGbox3d;

import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.ICanReproject;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.DefaultDBDriver;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.WKBParser2;
import com.iver.cit.gvsig.fmap.drivers.XTypes;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IWriteable;
import com.iver.cit.gvsig.fmap.edition.IWriter;

/**
 * @author FJP
 *
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class PostGisDriver extends DefaultDBDriver implements ICanReproject, IWriteable {
	private static Logger logger = Logger.getLogger(PostGisDriver.class
			.getName());

	private static int FETCH_SIZE = 5000;

	private PostGISWriter writer = new PostGISWriter();

	private WKBParser2 parser = new WKBParser2();

	private int fetch_min = -1;

	private int fetch_max = -1;

	private String sqlOrig;

	/**
	 * Used by setAbsolutePosition
	 */
	private String sqlTotal;

	private String strEPSG = null;

	private String originalEPSG = null;

	private Rectangle2D fullExtent = null;

	private String strAux;

	private String completeWhere;

	private String provCursorName = null;

	int numProvCursors = 0;
	
	boolean bShapeTypeRevised = false;

	static {
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 *
	 */
	public PostGisDriver() {

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDriver#getDriverAttributes()
	 */
	public DriverAttributes getDriverAttributes() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "PostGIS JDBC Driver";
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getShape(int)
	 */
	public IGeometry getShape(int index) {
		IGeometry geom = null;
		boolean resul;
		try {
			setAbsolutePosition(index);
			// strAux = rs.getString(1);
			// geom = parser.read(strAux);
			if (rs != null) {
				byte[] data = rs.getBytes(1);
				if (data == null) // null geometry. 
					return null;
				geom = parser.parse(data);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return geom;
	}

	/**
	 * First, the geometry field. After, the rest of fields
	 *
	 * @return
	 */
	/*
	 * public String getTotalFields() { String strAux = "AsBinary(" +
	 * getLyrDef().getFieldGeometry() + ")"; String[] fieldNames =
	 * getLyrDef().getFieldNames(); for (int i=0; i< fieldNames.length; i++) {
	 * strAux = strAux + ", " + fieldNames[i]; } return strAux; }
	 */

	/**
	 * Antes de llamar a esta funcin hay que haber fijado el workingArea si se
	 * quiere usar.
	 *
	 * @param conn
	 */
	public void setData(Connection conn, DBLayerDefinition lyrDef) {
		this.conn = conn;
		// TODO: Deberamos poder quitar Conneciton de la llamada y meterlo
		// en lyrDef desde el principio.

		lyrDef.setConnection(conn);
		setLyrDef(lyrDef);

		getTableEPSG_and_shapeType();
		
		getLyrDef().setSRID_EPSG(originalEPSG);

		try {
			conn.setAutoCommit(false);
			sqlOrig = "SELECT " + getTotalFields() + " FROM "
					+ getLyrDef().getTableName() + " ";
					// + getLyrDef().getWhereClause();
			if (canReproject(strEPSG)) {
				completeWhere = getCompoundWhere(sqlOrig, workingArea, strEPSG);
			} else {
				completeWhere = getCompoundWhere(sqlOrig, workingArea,
						originalEPSG);
			}
			// completeWhere = getLyrDef().getWhereClause() + completeWhere;

			String sqlAux = sqlOrig + completeWhere + " ORDER BY "
			+ getLyrDef().getFieldID();

			sqlTotal = sqlAux;
			logger.info("Cadena SQL:" + sqlAux);
			st = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_READ_ONLY);
			// st.setFetchSize(FETCH_SIZE);
			st.execute("declare wkb_cursor binary cursor for " + sqlAux);
			rs = st.executeQuery("fetch forward " + FETCH_SIZE
					+ " in wkb_cursor");
			// st.execute("begin");
			// bCursorActivo = true;
			// rs = st.executeQuery(sqlOrig);
			fetch_min = 0;
			fetch_max = FETCH_SIZE - 1;
			metaData = rs.getMetaData();
			doRelateID_FID();
			
			writer.setCreateTable(false);
			writer.setWriteAll(false);
			writer.initialize(lyrDef);


		} catch (SQLException e) {
			NotificationManager.addError(
					"Error al conectar a la base de datos.", e);
		} catch (EditionException e) {
			e.printStackTrace();
			NotificationManager.addError(
					"Error inicializando PosGIS Writer.", e);
			
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.layers.ReadableVectorial#getFullExtent()
	 */
	public Rectangle2D getFullExtent() {
		if (fullExtent == null) {
			try {
				Statement s = conn.createStatement();
				ResultSet r = s.executeQuery("SELECT extent("
						+ getLyrDef().getFieldGeometry()
						+ ") AS FullExtent FROM " + getLyrDef().getTableName()
						+ " " + getCompleteWhere());
				r.next();
				String strAux = r.getString(1);
				System.out.println("fullExtent = " + strAux);
				if (strAux == null) 
				{
					logger.debug("La capa " + getLyrDef().getName() + " no tiene FULLEXTENT");
					return null;
				}
				if (strAux.startsWith("BOX3D")) {
					PGbox3d regeom = new PGbox3d(strAux);
					double x = regeom.getLLB().x;
					double y = regeom.getLLB().y;
					double w = regeom.getURT().x - x;
					double h = regeom.getURT().y - y;
					fullExtent = new Rectangle2D.Double(x, y, w, h);
				} else {
					PGbox2d regeom = new PGbox2d(strAux);
					double x = regeom.getLLB().x;
					double y = regeom.getLLB().y;
					double w = regeom.getURT().x - x;
					double h = regeom.getURT().y - y;
					fullExtent = new Rectangle2D.Double(x, y, w, h);
				}
			} catch (SQLException e) {
				System.err.println(e.getMessage());
			}

		}
		return fullExtent;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getGeometryIterator(java.lang.String)
	 */
	public IFeatureIterator getFeatureIterator(String sql)
			throws com.iver.cit.gvsig.fmap.DriverException {
		PostGisFeatureIterator geomIterator = null;
		try {
			// st = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
			// ResultSet.CONCUR_READ_ONLY);


			if (provCursorName != null) {
				/* st.execute("BEGIN");
				st.execute("CLOSE " + provCursorName);
				bCursorActivo = false;
				st.execute("COMMIT"); */
				numProvCursors++;
			}
			// st.execute("BEGIN");
			provCursorName = "wkb_cursor_prov_" + System.currentTimeMillis() + "" + numProvCursors;

			// st.execute("BEGIN");
			bCursorActivo = true;
			// ResultSet rs = st.executeQuery(sql);
			geomIterator = new PostGisFeatureIterator(conn, provCursorName, sql);
			geomIterator.setLyrDef(getLyrDef());
		} catch (SQLException e) {
			e.printStackTrace();

			throw new DriverException(e);
			// return null;
		}

		return geomIterator;
	}

	public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG)
			throws DriverException {
		if (workingArea != null)
			r = r.createIntersection(workingArea);

		String sqlAux;
		if (canReproject(strEPSG)) {
			sqlAux = sqlOrig + getCompoundWhere(sqlOrig, r, strEPSG);
		} else {
			sqlAux = sqlOrig + getCompoundWhere(sqlOrig, r, originalEPSG);
		}

		System.out.println("SqlAux getFeatureIterator = " + sqlAux);

		return getFeatureIterator(sqlAux);
	}

	/**
	 * Le pasas el rectngulo que quieres pedir. La primera vez es el
	 * workingArea, y las siguientes una interseccion de este rectangulo con el
	 * workingArea
	 *
	 * @param r
	 * @param strEPSG
	 * @return
	 */
	private String getCompoundWhere(String sql, Rectangle2D r, String strEPSG) {
		if (r == null)
			return getWhereClause();

		double xMin = r.getMinX();
		double yMin = r.getMinY();
		double xMax = r.getMaxX();
		double yMax = r.getMaxY();
		String wktBox = "GeometryFromText('LINESTRING(" + xMin + " " + yMin
				+ ", " + xMax + " " + yMin + ", " + xMax + " " + yMax + ", "
				+ xMin + " " + yMax + ")', " + strEPSG + ")";
		String sqlAux;
		if (getWhereClause().indexOf("WHERE") != -1)
			sqlAux = getWhereClause() + " AND " + getLyrDef().getFieldGeometry() + " && " + wktBox;
		else
			sqlAux = "WHERE " + getLyrDef().getFieldGeometry() + " && "
					+ wktBox;
		return sqlAux;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getConnectionStringBeginning()
	 */
	public String getConnectionStringBeginning() {
		return "jdbc:postgresql:";
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.DefaultDBDriver#getFieldValue(long,
	 *      int)
	 */
	public Value getFieldValue(long rowIndex, int idField)
			throws com.hardcode.gdbms.engine.data.driver.DriverException {
		boolean resul;
		// EL ABSOLUTE NO HACE QUE SE VUELVAN A LEER LAS
		// FILAS, ASI QUE MONTAMOS ESTA HISTORIA PARA QUE
		// LO HAGA
		// System.out.println("getShape " + index);
		int index = (int) (rowIndex);
		try {
			setAbsolutePosition(index);
			int fieldId = idField + 2;
			byte[] byteBuf = rs.getBytes(fieldId);
			if (byteBuf == null)
				return ValueFactory.createNullValue();
			else {
				ByteBuffer buf = ByteBuffer.wrap(byteBuf);
				if (metaData.getColumnType(fieldId) == Types.VARCHAR)
					return ValueFactory.createValue(rs.getString(fieldId));
				if (metaData.getColumnType(fieldId) == Types.FLOAT)
					return ValueFactory.createValue(buf.getFloat());
				if (metaData.getColumnType(fieldId) == Types.DOUBLE)
					return ValueFactory.createValue(buf.getDouble());
				if (metaData.getColumnType(fieldId) == Types.INTEGER)
					return ValueFactory.createValue(buf.getInt());
				if (metaData.getColumnType(fieldId) == Types.BIGINT)
					return ValueFactory.createValue(buf.getLong());
				if (metaData.getColumnType(fieldId) == Types.BIT)
					// TODO
					return ValueFactory.createValue(rs.getBoolean(fieldId));
				if (metaData.getColumnType(fieldId) == Types.DATE)
				{
	                long daysAfter2000 = buf.getInt() + 1;
	                long msecs = daysAfter2000*24*60*60*1000;
	                long real_msecs_date1 = (long) (XTypes.NUM_msSecs2000 + msecs);
	                Date realDate1 = new Date(real_msecs_date1);
					return ValueFactory.createValue(realDate1);
				}
				if (metaData.getColumnType(fieldId) == Types.TIME)
				{
					// TODO:
					// throw new RuntimeException("TIME type not implemented yet");
					return ValueFactory.createValue("NOT IMPLEMENTED YET");
				}				
				if (metaData.getColumnType(fieldId) == Types.TIMESTAMP)				
				{
					double segsReferredTo2000 = buf.getDouble();
					long real_msecs = (long) (XTypes.NUM_msSecs2000 + segsReferredTo2000*1000);
					Timestamp valTimeStamp = new Timestamp(real_msecs);
					return ValueFactory.createValue(valTimeStamp);
				}

				if (metaData.getColumnType(fieldId) == Types.NUMERIC) {
					// System.out.println(metaData.getColumnName(fieldId) + " "
					// + metaData.getColumnClassName(fieldId));
					short ndigits = buf.getShort();
					short weight = buf.getShort();
					short sign = buf.getShort();
					short dscale = buf.getShort();
					String strAux;
					if (sign == 0)
						strAux = "+";
					else
						strAux = "-";

					for (int iDigit = 0; iDigit < ndigits; iDigit++) {
						short digit = buf.getShort();
						strAux = strAux + digit;
						if (iDigit == weight)
							strAux = strAux + ".";

					}
					strAux = strAux + "0";
					String str2;
					BigDecimal dec;
					dec = new BigDecimal(strAux);
					// System.out.println(ndigits + "_" + weight + "_" + dscale
					// + "_" + strAux);
					// System.out.println(strAux + " Big= " + dec);
					return ValueFactory.createValue(dec.doubleValue());
				}

			}
		} catch (SQLException e) {
			throw new com.hardcode.gdbms.engine.data.driver.DriverException(e
					.getMessage());
		}
		return ValueFactory.createNullValue();

	}

	public void open() throws com.iver.cit.gvsig.fmap.DriverException {
		/*
		 * try { st = conn.createStatement(); st.setFetchSize(2000); if
		 * (bCursorActivo) close(); st.execute("declare wkb_cursor binary cursor
		 * for " + sqlOrig); rs = st.executeQuery("fetch forward all in
		 * wkb_cursor"); // st.execute("BEGIN"); bCursorActivo = true; } catch
		 * (SQLException e) { e.printStackTrace(); throw new
		 * com.iver.cit.gvsig.fmap.DriverException(e); }
		 */

	}

	private void setAbsolutePosition(int index) throws SQLException {
		// TODO: USAR LIMIT Y ORDER BY, Y HACERLO TAMBIN PARA
		// MYSQL

		// EL ABSOLUTE NO HACE QUE SE VUELVAN A LEER LAS
		// FILAS, ASI QUE MONTAMOS ESTA HISTORIA PARA QUE
		// LO HAGA
		// System.out.println("getShape " + index + " fetchMin=" + fetch_min + "
		// fetchMax=" + fetch_max);
		if (index < fetch_min) {
			// rs.close();
			st.execute("CLOSE wkb_cursor");
			st.execute("declare wkb_cursor binary cursor for " + sqlTotal);
			rs = st.executeQuery("fetch forward " + FETCH_SIZE
					+ " in wkb_cursor");

			// rs.beforeFirst();

			// rs = st.executeQuery(sqlOrig);
			fetch_min = 0;
			fetch_max = FETCH_SIZE - 1;
		}
		while (index > fetch_max && index != 0) {
			rs = st.executeQuery("fetch forward " + FETCH_SIZE
					+ " in wkb_cursor");
			// rs.next();
			/*
			 * rs.afterLast(); // forzamos una carga rs.next();
			 */
			fetch_min = fetch_max + 1;
			fetch_max = fetch_min + FETCH_SIZE - 1;
			// System.out.println("fetchSize = " + rs.getFetchSize() + " " +
			// fetch_min + "-" + fetch_max);
		}
		if (rs != null)
			rs.absolute(index - fetch_min + 1);

		// rs.absolute(index+1);

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getGeometryField(java.lang.String)
	 */
	public String getGeometryField(String fieldName) {
		return "ASBINARY(" + fieldName + ", 'XDR')";
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#getPrimaryKeys()
	 */
	public int[] getPrimaryKeys()
			throws com.hardcode.gdbms.engine.data.driver.DriverException {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver#getDefaultPort()
	 */
	public int getDefaultPort() {
		return 5432;
	}

	/**
	 * @see com.hardcode.gdbms.engine.data.driver.ObjectDriver#write(com.hardcode.gdbms.engine.data.edition.DataWare)
	 */
	public void write(DataWare arg0)
			throws com.hardcode.gdbms.engine.data.driver.DriverException {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.core.ICanReproject#getSourceProjection()
	 */
	public String getSourceProjection() {
		if (originalEPSG == null)
			getTableEPSG_and_shapeType();
		return originalEPSG;
	}

	/**
	 * Las tablas con geometras estn en la tabla GEOMETRY_COLUMNS y de
	 * ah sacamos en qu proyeccin estn.
	 * El problema es que si el usuario hace una vista de esa
	 * tabla, no estar dada de alta aqu y entonces gvSIG
	 * no se entera en qu proyeccin est trabajando (y le
	 * ponemos un -1 como mal menor). El -1 implica que luego
	 * no podremos reproyectar al vuelo desde la base de datos.
	 */
	private void getTableEPSG_and_shapeType() {
		try {
			Statement stAux = conn.createStatement();

			String sql = "SELECT * FROM GEOMETRY_COLUMNS WHERE F_TABLE_NAME = '"
					+ getTableName() + "' AND F_GEOMETRY_COLUMN = '" + getLyrDef().getFieldGeometry() + "'";
			ResultSet rs = stAux.executeQuery(sql);
			rs.next();
			originalEPSG = "" + rs.getInt("SRID");
			String geometryType = rs.getString("TYPE");
			int shapeType = FShape.MULTI;
			if (geometryType.compareToIgnoreCase("POINT") == 0)
				shapeType = FShape.POINT;
			if (geometryType.compareToIgnoreCase("LINESTRING") == 0)
				shapeType = FShape.LINE;
			if (geometryType.compareToIgnoreCase("POLYGON") == 0)
				shapeType = FShape.POLYGON;
			if (geometryType.compareToIgnoreCase("MULTIPOINT") == 0)
				shapeType = FShape.POINT;
			if (geometryType.compareToIgnoreCase("MULTILINESTRING") == 0)
				shapeType = FShape.LINE;
			if (geometryType.compareToIgnoreCase("MULTIPOLYGON") == 0)
				shapeType = FShape.POLYGON;			
			
			getLyrDef().setShapeType(shapeType);
			rs.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			originalEPSG = "-1";
			logger.error(e);
			e.printStackTrace();
		}

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.core.ICanReproject#getDestProjection()
	 */
	public String getDestProjection() {
		return strEPSG;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.core.ICanReproject#setDestProjection(java.lang.String)
	 */
	public void setDestProjection(String toEPSG) {
		this.strEPSG = toEPSG;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.core.ICanReproject#canReproject(java.lang.String)
	 */
	public boolean canReproject(String toEPSGdestinyProjection) {
		// TODO POR AHORA, REPROYECTA SIEMPRE gvSIG.
		return false;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.DefaultDBDriver#doRelateID_FID()
	 */
	protected void doRelateID_FID() {
		hashRelate = new Hashtable();
		try {
			String strSQL = "SELECT " + getLyrDef().getFieldID() + " FROM "
					+ getLyrDef().getTableName() + " ";
					// + getLyrDef().getWhereClause();
			if (canReproject(strEPSG)) {
				strSQL = strSQL
						+ getCompoundWhere(strSQL, workingArea, strEPSG);
			} else {
				strSQL = strSQL
						+ getCompoundWhere(strSQL, workingArea, originalEPSG);
			}
			strSQL = strSQL + " ORDER BY " + getLyrDef().getFieldID();
			Statement s = getConnection().createStatement(
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_READ_ONLY);
			int fetchSize = 5000;
			ResultSet r = s.executeQuery(strSQL);
			int id = 0;
			String gid;
			while (r.next()) {
				gid = r.getString(1);
				Value aux = ValueFactory.createValue(gid);
				hashRelate.put(aux, new Integer(id));
				// System.out.println("ASOCIANDO CLAVE " + aux + " CON VALOR " + id);
				id++;
				// System.out.println("Row " + id + ":" + strAux);
			}
			s.close();
			numReg = id;

			/*
			 * for (int index = 0; index < getShapeCount(); index++) { Value aux =
			 * getFieldValue(index, idFID_FieldName-2); hashRelate.put(aux, new
			 * Integer(index)); // System.out.println("Row " + index + " clave=" +
			 * aux); }
			 */
			/*
			 * int index = 0;
			 *
			 * while (rs.next()) { Value aux = getFieldValue(index,
			 * idFID_FieldName-2); hashRelate.put(aux, new Integer(index));
			 * index++; System.out.println("Row " + index + " clave=" + aux); }
			 * numReg = index;
			 */
			// rs.beforeFirst();
			/*
			 * } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) { //
			 * TODO Auto-generated catch block e.printStackTrace();
			 */
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getSqlTotal() {
		return sqlTotal;
	}

	/**
	 * @return Returns the completeWhere.
	 */
	public String getCompleteWhere() {
		return completeWhere;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.DefaultDBDriver#close()
	 */
	public void close() {
		super.close();
		/*
		 * if (bCursorActivo) { try { // st =
		 * conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,
		 * ResultSet.CONCUR_READ_ONLY); st.execute("CLOSE wkb_cursor_prov"); //
		 * st.close(); } catch (SQLException e) { // TODO Auto-generated catch
		 * block e.printStackTrace(); } bCursorActivo = false; }
		 */

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver#getFeatureIterator(java.awt.geom.Rectangle2D,
	 *      java.lang.String, java.lang.String[])
	 */
	public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG,
			String[] alphaNumericFieldsNeeded) throws DriverException {
		try {
			if (workingArea != null)
				r = r.createIntersection(workingArea);
			// if (getLyrDef()==null){
			// load();
			// throw new DriverException("Fallo de la conexin");
			// }
			String strAux = getGeometryField(getLyrDef().getFieldGeometry());

			boolean found = false;
			if (alphaNumericFieldsNeeded != null) {
				for (int i = 0; i < alphaNumericFieldsNeeded.length; i++) {
					strAux = strAux + ", " + alphaNumericFieldsNeeded[i];
					if (alphaNumericFieldsNeeded[i].equals(getLyrDef()
							.getFieldID()))
						found = true;
				}
			}
			// Nos aseguramos de pedir siempre el campo ID
			if (found == false)
				strAux = strAux + ", " + getLyrDef().getFieldID();

			String sqlProv = "SELECT " + strAux + " FROM "
					+ getLyrDef().getTableName() + " ";
					// + getLyrDef().getWhereClause();

			String sqlAux;
			if (canReproject(strEPSG)) {
				sqlAux = sqlProv + getCompoundWhere(sqlProv, r, strEPSG);
			} else {
				sqlAux = sqlProv + getCompoundWhere(sqlProv, r, originalEPSG);
			}

			System.out.println("SqlAux getFeatureIterator = " + sqlAux);

			return getFeatureIterator(sqlAux);
		} catch (Exception e) {
			throw new DriverException(e);
		}
	}

	/* public void preProcess() throws EditionException {
		writer.preProcess();
	}

	public void process(IRowEdited row) throws EditionException {
		writer.process(row);
	}

	public void postProcess() throws EditionException {
		writer.postProcess();
	}

	public String getCapability(String capability) {
		return writer.getCapability(capability);
	}

	public void setCapabilities(Properties capabilities) {
		writer.setCapabilities(capabilities);
	}

	public boolean canWriteAttribute(int sqlType) {
		return writer.canWriteAttribute(sqlType);
	}

	public boolean canWriteGeometry(int gvSIGgeometryType) {
		return writer.canWriteGeometry(gvSIGgeometryType);
	}

	public void initialize(ITableDefinition layerDef) throws EditionException {
		writer.setCreateTable(false);
		writer.setWriteAll(false);
		// Obtenemos el DBLayerDefinition a partir del driver

		DBLayerDefinition dbLyrDef = getLyrDef();


		writer.initialize(dbLyrDef);
	}
*/
	public boolean isWritable() {
		return true;
	} 

	public IWriter getWriter() {
		return writer;
	}


}
