/*
 * Created on 25-oct-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 * 
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 * 
 *    or
 * 
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 * 
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.jdbc_spatial;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;
import javax.swing.Timer;

import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver;
import com.iver.cit.gvsig.fmap.drivers.jdbc.postgis.PostGIS;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.jdbc_spatial.gui.jdbcwizard.ConnectionSettings;
import com.iver.utiles.swing.threads.SwingWorker;

public class ImportLayer extends Extension {

    private ProgressMonitor progressMonitor;
    private Timer timer;
    WriteTask task;

    /**
     * The actionPerformed method in this class
     * is called each time the Timer "goes off".
     */
    class TimerListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            int pos = task.getCurrent();
            int hasta = task.getLengthOfTask();
            progressMonitor.setProgress(pos);
            // System.out.println("Progreso:" + task.getCurrent());
            progressMonitor.setNote(pos + " de " + hasta);
            if (progressMonitor.isCanceled() || task.isDone()) {
                progressMonitor.close();
                task.stop();
                timer.stop();
            }
        }
    }


    public class WriteTask {
        private int lengthOfTask;
        private int current = 0;
        private boolean done = false;
        private boolean canceled = false;
        private String statMessage;
        ReadableVectorial adapter = null;
        Statement st = null;
        DBLayerDefinition dbLayerDef = new DBLayerDefinition();
        Connection conex;

        public WriteTask(FLyrVect lyrVect) {
            adapter = lyrVect.getSource();
            try {
                adapter.start();

                SelectableDataSource rsSel = lyrVect.getRecordset();

                int sc = adapter.getShapeCount();

                String tableName = JOptionPane.showInputDialog(PluginServices.getText(this,"intro_tablename"));
                if (tableName == null)
                    return;
                ArrayList newList = new ArrayList();
                DlgConnection dlg = new DlgConnection();
                dlg.setModal(true);
                dlg.setVisible(true);
                ConnectionSettings cs = dlg.getConnSettings();
                if (cs == null)
                    return;
                conex = DriverManager.getConnection(cs
                        .getConnectionString(), cs.getUser(), cs.getPassw());

                st = conex.createStatement();

                dbLayerDef.setCatalogName(cs.getDb());
                dbLayerDef.setTableName(tableName);
                String strGeometryFieldType = "GEOMETRY";
                /*
                 * switch (lyrVect.getShapeType()) { case FShape.POINT:
                 * strGeometryFieldType = XTypes.fieldTypeToString(XTypes.POINT2D);
                 * break; case FShape.LINE: strGeometryFieldType =
                 * XTypes.fieldTypeToString(XTypes.LINE2D); break; case
                 * FShape.POLYGON: strGeometryFieldType =
                 * XTypes.fieldTypeToString(XTypes.POLYGON2D); break; case
                 * FShape.MULTI: strGeometryFieldType =
                 * XTypes.fieldTypeToString(XTypes.MULTI2D); break; }
                 */
                dbLayerDef.setFieldGeometry("the_geom");
                FieldDescription[] fieldsDescrip = new FieldDescription[rsSel
                        .getFieldNames().length];
                dbLayerDef.setFieldNames(rsSel.getFieldNames());
                for (int i = 0; i < rsSel.getFieldNames().length; i++) {
                    fieldsDescrip[i] = new FieldDescription();
                    fieldsDescrip[i].setFieldType(rsSel.getFieldType(i));
                    fieldsDescrip[i].setFieldName(rsSel.getFieldName(i));
                    // TODO: Por ahora le ponemos 200, a falta
                    // de recompilar GDBMS con la posibilidad
                    // de obtener el ancho de un campo.
                    fieldsDescrip[i].setFieldLength(200);
                }
                String strSRID = lyrVect.getProjection().getAbrev().substring(5);
                dbLayerDef.setSRID_EPSG(strSRID);
                

                // PluginServices.getMDIManager().setWaitCursor();
                try
                {
                    st.execute("DROP TABLE " + dbLayerDef.getTableName() + ";");
                }
                catch (SQLException e1)
                {
                    // Si no existe la tabla, no hay que borrarla.
                }
        		boolean bExistGID = false;
        		for (int i = 0; i < dbLayerDef.getFieldNames().length; i++) {
        			if (dbLayerDef.getFieldNames()[i].equalsIgnoreCase("gid")) {
        				bExistGID = true;
        				break;
        			}
        		}

                String sqlCreate = PostGIS.getSqlCreateSpatialTable(dbLayerDef,
                        fieldsDescrip, !bExistGID);
                System.out.println("sqlCreate =" + sqlCreate);
                st.execute(sqlCreate);
                dbLayerDef.setDimension(2);

                String sqlAlter = PostGIS.getSqlAlterTable(dbLayerDef);
                System.out.println("sqlAlter =" + sqlAlter);
                st.execute(sqlAlter);
                // CREATE TABLE PARKS ( PARK_ID int4, PARK_NAME varchar(128),
                // PARK_DATE date, PARK_TYPE varchar(2) );
                // SELECT AddGeometryColumn('parks_db', 'parks', 'park_geom', 128,
                // 'MULTIPOLYGON', 2 );

                /*
                 * BEGIN; INSERT INTO ROADS_GEOM (ID,GEOM,NAME ) VALUES
                 * (1,GeometryFromText('LINESTRING(191232 243118,191108
                 * 243242)',-1),'Jeff Rd'); INSERT INTO ROADS_GEOM (ID,GEOM,NAME )
                 * VALUES (2,GeometryFromText('LINESTRING(189141 244158,189265
                 * 244817)',-1),'Geordie Rd'); COMMIT;
                 */                            
                
                conex.setAutoCommit(false);
                
                lengthOfTask = sc;
                
                // ATENTION: We will transform (in PostGIS class; doubleQuote())
                // to UTF-8 strings. Then, we tell the PostgreSQL server
                // that we will use UTF-8, and it can translate
                // to its charset
                // Note: we have to translate to UTF-8 because
                // the server cannot manage UTF-16
                ResultSet rsAux = st.executeQuery("SHOW server_encoding;");
                rsAux.next();
                String serverEncoding = rsAux.getString(1);
                System.out.println("Server encoding = " + serverEncoding);
                // st.execute("SET CLIENT_ENCODING TO 'UNICODE';");
                // Intentamos convertir nuestras cadenas a ese encode.
                PostGIS.toEncode = serverEncoding;
                
                go();
                
            } catch (Exception e) {
                NotificationManager.addError("Error en la carga.", e);
            }

        }

        /**
         * Called from ProgressBarDemo to start the task.
         */
        public void go() {
            final SwingWorker worker = new SwingWorker() {
                public Object construct() {
                    current = 0;
                    done = false;
                    canceled = false;
                    statMessage = null;
                    try {
                        return new ActualTask(adapter, st, dbLayerDef);
                    } catch (Exception e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                    return null;
                }
            };
            worker.start();
        }

        /**
         * Called from ProgressBarDemo to find out how much work needs
         * to be done.
         */
        public int getLengthOfTask() {
            return lengthOfTask;
        }

        /**
         * Called from ProgressBarDemo to find out how much has been done.
         */
        public int getCurrent() {
            return current;
        }

        public void stop() {
            canceled = true;
            statMessage = null;
        }

        /**
         * Called from ProgressBarDemo to find out if the task has completed.
         */
        public boolean isDone() {
            return done;
        }

        /**
         * Returns the most recent status message, or null
         * if there is no current status message.
         */
        public String getMessage() {
            return statMessage;
        }

        /**
         * The actual long running task.  This runs in a SwingWorker thread.
         */
        class ActualTask {
            ActualTask(ReadableVectorial adapter, Statement st, DBLayerDefinition dbLayerDef) throws Exception {
                int sc = adapter.getShapeCount();
                String sqlInsert = null;
                for (int i = 0; i < sc; i++) {
                    // IGeometry geom = adapter.getShape(i);
                    IFeature feat = adapter.getFeature(i);
                    if (feat == null) continue;
                    if (feat.getGeometry() == null) continue;
                    try 
                    {
                        
                        sqlInsert = PostGIS
                                .getSqlInsertFeature(dbLayerDef, feat);
                        // if (i<5)
                            // System.out.println("sqlInsert =" + sqlInsert); 
                        st.execute(sqlInsert);
                    }
                    catch (Exception e)
                    {
                        // Si alguna entidad da problemas, no la importamos y 
                        // lo damos como un Error.
                        NotificationManager.addError(
                                "No se ha podido guardar el registro " + i + 
                                ". SqlInsert=" + sqlInsert, e);
                        canceled = true;
                    }
                    current = i;
                    if (canceled)
                        break;
                }
                done = true;
                statMessage = "Completed " + current +
                " out of " + lengthOfTask + ".";
                conex.setAutoCommit(true);
                adapter.stop();
                
                // PluginServices.getMDIManager().restoreCursor();
                
                int bCreateIndex = JOptionPane.showConfirmDialog(null, PluginServices.getText(this,"capa_importada"), "Index Creation", JOptionPane.YES_NO_OPTION);
                if (bCreateIndex == JOptionPane.YES_OPTION)
                {
                    String sqlCreateIndex = PostGIS.getSqlCreateIndex(dbLayerDef);
                    System.out.println(sqlCreateIndex);
                    st.execute(sqlCreateIndex);
                }                
                conex.close();                
            }
        }
    }

    public void initialize() {
        // TODO Auto-generated method stub

    }

    public void execute(String actionCommand) {
        View vista = (View) PluginServices.getMDIManager().getActiveWindow();
        MapControl mapCtrl = vista.getMapControl();
        FLayer[] activeLayers = mapCtrl.getMapContext().getLayers()
                .getActives();
        FLyrVect lyrVect = (FLyrVect) activeLayers[0];
        
        task = new WriteTask(lyrVect);
        progressMonitor = new ProgressMonitor(null,
                "Importando registros...",
                "", 0, task.getLengthOfTask());
        progressMonitor.setProgress(0);
        progressMonitor.setMillisToDecideToPopup(500);

//      Create a timer.
        timer = new Timer(500, new TimerListener());
        timer.start();
        
    }

    private String[] getDriverNames() {
        Class[] classes = new Class[] { VectorialJDBCDriver.class };

        ArrayList ret = new ArrayList();
        String[] driverNames = LayerFactory.getDM().getDriverNames();

        for (int i = 0; i < driverNames.length; i++) {
            boolean is = false;

            for (int j = 0; j < classes.length; j++) {
                if (LayerFactory.getDM().isA(driverNames[i], classes[j])) {
                    ret.add(driverNames[i]);
                }
            }
        }

        return (String[]) ret.toArray(new String[0]);

    }

    public boolean isEnabled() {
        com.iver.andami.ui.mdiManager.IWindow f = PluginServices.getMDIManager()
                .getActiveWindow();

        if (f == null) {
            return false;
        }

        if (f  instanceof View) {
            MapContext mapa = ((View) f).getModel().getMapContext();

            FLayer[] activeLayers = mapa.getLayers().getActives();
            if (activeLayers.length == 1)
            {
                if (activeLayers[0] instanceof FLyrVect)
                    return true;    
            }
        }
        return false;
    }

    public boolean isVisible() {
        com.iver.andami.ui.mdiManager.IWindow f = PluginServices.getMDIManager()
                .getActiveWindow();

        if (f == null) {
            return false;
        }

        if (f instanceof View) {
            MapContext mapa = ((View) f).getModel().getMapContext();

            return mapa.getLayers().getLayersCount() > 0;
        } else {
            return false;
        }

    }

}
