/*
 * Creado el 7-marzo-2005
 */
package com.iver.cit.gvsig.gui.dialog;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;

import org.cresques.cts.IProjection;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.filter.BrightnessContrast.BrightnessContrastStackManager;
import org.cresques.io.GeoRasterFile;
import org.cresques.px.Extent;
import org.cresques.px.PxRaster;
import org.cresques.ui.BrightnessContrast.EnhancedBrightnessContrastPanel;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.InfoPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.StatusLayerRaster;

/**
 * <P>
 * Dialog for the properties of a WCS layer. It manages the avants and aplies
 * filters to the raster through the filter stack manager according to the user's
 * selection. This dialog contains some panels.
 * </P>
 * <UL>
 * <LI>Propierties</LI>
 * <LI>Band selection</LI>
 * <LI>Transparency</LI>
 * <LI>Enhancement</LI>
 * </UL>
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class WCSRasterPropsDialog extends FilterRasterDialogPanel implements IWindow, MouseListener, KeyListener, ActionListener{
	
	private FilterRasterDialogPanel 				contentPane = null;  	
	private JPanel									propPanel = null;
	private IProjection 							currentProjection = null;
	private FLyrWCS		 							fLayer = null;
	private static final int						nprops = 13;
	private Object[][]								props = null;
	private RasterFilterStackManager 				stackManager = null;
	private Status									status = null;
	private StatusLayerRaster						rasterStatus = null;
	private JFileChooser							fileChooser = null;
	private String									lastPath = new String("./");
	private	PxRaster								px = null;
	private GeoRasterFile							grf = null;
	
	/**
	 * Class that holds the dialog state and restores the initial state if it
	 * is cancelled.
	 * @author Nacho Brodin <brodin_ign@gva.es>
	 */
	class Status{
		public String					inicAlpha;
		public int 						bandR;
		public int 						bandG;
		public int 						bandB;
		private ArrayList				filters = new ArrayList();
		
		public Status(String alpha, int bandR, int bandG, int bandB){
			this.inicAlpha = alpha;
			this.bandR = bandR;
			this.bandG = bandG;
			this.bandB = bandB;
			filters = stackManager.getStringsFromStack();
		}
				
		/**
		 * Restaura el Estado salvado 
		 * @param status Estado
		 */
		public void restoreStatus(WCSRasterPropsDialog props){
			//Devolvemos la pila de filtros al estado inicial
			/*if(stackManager != null)
				stackManager.deleteTempFilters();*/
			
			//Devolvemos el alpha al estado inicial
			int opac = Integer.parseInt(status.inicAlpha);
			opac = (int)((opac*255)/100);
			fLayer.setTransparency(255-opac);
			
			if (fLayer != null) {
				fLayer.setBandR(bandR);
				fLayer.setBandG(bandG);
				fLayer.setBandB(bandB);
			}
			
			//Restauramos los filtros
			if(filters!=null)
				stackManager.createStackFromStrings(filters);
			
			fLayer.getMapContext().invalidate();
			
		}
	}
	
	public class DriverFileFilter extends FileFilter{
		
		private Driver driver;
		
		public DriverFileFilter(String driverName) throws DriverLoadException{
			driver = LayerFactory.getDM().getDriver(driverName);
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
		 */
		public boolean accept(File f) {
			if (f.isDirectory()) return true;
			if (driver instanceof RasterDriver){
				return ((RasterDriver) driver).fileAccepted(f);
			}else{
				throw new RuntimeException("Tipo no reconocido");
			}
		}

		/**
		 * @see javax.swing.filechooser.FileFilter#getDescription()
		 */
		public String getDescription() {
			return ((Driver) driver).getName();
		}
	}
	
	/**
	 * Dialog window constructor.
	 * 
	 * @param app
	 */
	public WCSRasterPropsDialog(FLyrWCS layer, ArrayList ranges){
		super();
		fLayer = layer;
		this.px = layer.getPxRaster();
		this.grf = layer.getGeoRasterFile();
		if(fLayer.getStatus()==null){
			rasterStatus = new StatusLayerRaster();
			fLayer.setStatus(rasterStatus);
		}else
			rasterStatus = (StatusLayerRaster)fLayer.getStatus();
		init();
		this.setRanges(ranges);
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		BandSetupPanel bandSetup = (BandSetupPanel)fr.getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().setEnabled(false);
		bandSetup.getFileList().getJButtonRemove().setEnabled(false);
		this.setTranslation();
		
		EnhancedBrightnessContrastPanel ep = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		ep.lstBrightness.getJSlider().addMouseListener(this);
		ep.lstContrast.getJSlider().addMouseListener(this);
		ep.getCheckSliderText().getJTextField().addKeyListener(this);
		ep.getLabelSliderText().getJTextField().addActionListener(this);
		ep.getLabelSliderText1().getJTextField().addActionListener(this);
	}
	
	/**
	 * Asigna los textos a los paneles
	 */
	private void setTranslation(){
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		BandSetupPanel bandSetup = (BandSetupPanel)fr.getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().setText(PluginServices.getText(this,"Anadir"));
		bandSetup.getFileList().getJButtonRemove().setText(PluginServices.getText(this,"Eliminar"));
		bandSetup.getFileList().lbandasVisibles.setText(PluginServices.getText(this,"bandas"));
		
		RasterTransparencyPanel tpan = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
				
		tpan.getTransparencyCheck().setText(PluginServices.getText(this,"transparencia"));
		tpan.getOpacityCheck().setText(PluginServices.getText(this,"opacidad"));
						
		EnhancedBrightnessContrastPanel ep = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		
		ep.lLineal.setText(PluginServices.getText(this, "lineal_directo"));
		ep.lRemove.setText(PluginServices.getText(this, "eliminar_extremos"));
		ep.cstEnhanced.setName(PluginServices.getText(this, "recorte_colas")+" ( % )");
		ep.lBrightC.setText(PluginServices.getText(this, "brillo_y_contraste"));
		ep.lstBrightness.setName(PluginServices.getText(this, "brillo"));
		ep.lstContrast.setName(PluginServices.getText(this, "contraste"));
		ep.lpreview.setText(PluginServices.getText(this, "previsualizacion"));
		ep.getPBrightCont().setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1), (PluginServices.getText(this, "brillo_y_contraste")), javax.swing.border.TitledBorder.LEFT, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, java.awt.Color.black));
		ep.getPEnhanced().setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1), (PluginServices.getText(this, "realce")), javax.swing.border.TitledBorder.LEFT, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, java.awt.Color.black));
		
		InfoPanel ip = (InfoPanel)super.getPanelByClassName("InfoPanel");
		
		ip.cabInfo = new String(PluginServices.getText(this, "Informacion"));
		ip.cabCoord = new String(PluginServices.getText(this, "coor_geograficas"));
		ip.cabProy = new String(PluginServices.getText(this, "Proyecciones"));
		ip.cabOrig = new String(PluginServices.getText(this, "origen_de_datos"));
		ip.cabMeta = new String(PluginServices.getText(this, "metadatos"));
		ip.refresh();
		
		
		
		//Recorremos los Tab y traducimos el nombre
		for(int i=0;i<this.getTab().getTabCount();i++){
			if(this.getTab().getTitleAt(i).equals("Info"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"info"));
			if(this.getTab().getTitleAt(i).equals("Transparencia"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"transparencia"));
			if(this.getTab().getTitleAt(i).equals("Bandas"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"bandas"));
			if(this.getTab().getTitleAt(i).equals("Realce"))
				this.getTab().setTitleAt(i,PluginServices.getText(this,"realce"));
		}
		
		this.getAcceptButton().setText(PluginServices.getText(this,"Aceptar"));
		this.getApplyButton().setText(PluginServices.getText(this,"Aplicar"));
		this.getCancelButton().setText(PluginServices.getText(this,"Cancelar"));
	}
	
	/**
	 * Assigns a FLayerRaster
	 * @param layer	capa a asignar
	 */
	public void setFLyrWCS(FLyrWCS layer){
		fLayer = layer;
	}
	
	/**
	 * Dialog window constructor.
	 */
	public WCSRasterPropsDialog() {
		init();
	}
			
	/**
	 * Loads the info panel data.
	 */
	private void loadInfoData(){
		Rectangle2D r = fLayer.getFullExtent();
		if(fLayer != null){
			props = new Object[nprops][2];
			props[0][0] = new String(PluginServices.getText(this,"Fichero")+":");
			props[0][1] = fLayer.getHost();
			props[1][0] = new String(PluginServices.getText(this,"tamano")+":");
			props[1][1] = "";
			props[2][0] = new String(PluginServices.getText(this,"ancho_alto")+":");
			props[2][1] = fLayer.getWidth()+" , "+fLayer.getHeight();
			props[3][0] = new String(PluginServices.getText(this,"formato")+":");
			props[3][1] = "WCS";
			props[4][0] = new String(PluginServices.getText(this,"tipo_dato")+":");
			String type = null;
			switch(grf.getDataType()){
				case 0: type = new String("BYTE");break;
				case 1: type = new String("USHORT");break;
				case 2: type = new String("SHORT");break;
				case 3: type = new String("INT");break;
				case 4: type = new String("FLOAT");break;
				case 5: type = new String("DOUBLE");break;
				default: type = new String("UNDEFINED");break;
			}
		    props[4][1] = type;
		    props[4][0] = new String(PluginServices.getText(this, "georeferenciado")+":");
		    props[4][1] = new String(PluginServices.getText(this, "si"));
		    props[5][0] = new String(PluginServices.getText(this,"num_bandas")+":");
			props[5][1] = new String(String.valueOf(grf.getBandCount()));
		    props[6][0] = new String(PluginServices.getText(this,"coor_geograficas"));
			props[7][0] = new String(PluginServices.getText(this,"xmin")+":");
			props[7][1] = String.valueOf(fLayer.getMinX());
			props[8][0] = new String(PluginServices.getText(this,"ymin")+":");
			props[8][1] = String.valueOf(fLayer.getMinY());
			props[9][0] = new String(PluginServices.getText(this,"xmax")+":");
			props[9][1] = String.valueOf(fLayer.getMaxX());
			props[10][0] = new String(PluginServices.getText(this,"ymax")+":");
			props[10][1] = String.valueOf(fLayer.getMaxY());
			
			double tamRealX = fLayer.getMaxX()-fLayer.getMinX();
			double tamRealY = fLayer.getMaxY()-fLayer.getMinY();
			double tamX = Math.round((tamRealX/fLayer.getWidth())*10000000);
			double tamY = Math.round((tamRealY/fLayer.getHeight())*10000000);
			
			String tamPixX = String.valueOf(tamX/10000000);
			String tamPixY = String.valueOf(tamY/10000000);
			
			props[11][0] = new String(PluginServices.getText(this, "tamPixX"));
			props[11][1] = tamPixX + " " + new String(PluginServices.getText(this, "m/pixel"));
			props[12][0] = new String(PluginServices.getText(this, "tamPixY"));
			props[12][1] = tamPixY + " " + new String(PluginServices.getText(this, "m/pixel"));
			
		}else{
			props = new Object[1][2];
			props[0][0] = new String("No props");
			props[0][1] = new String("-");
		}
		
	}
	
	/**
	 * A�ade bandas al contador de bandas del FilterRasterDialogPanel
	 * @param numBands N�mero de bandas a a�adir
	 */
	public void addNumBands(int numBands){
		nbands += numBands;
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
		if(rasterTrans != null && rasterTrans.getPTranspByPixel().isControlEnabled())
			rasterTrans.setActiveTransparencyControl(true); 
	}
	
	/**
	 * Inits the jDialog	
	 */    
	public void init() {
		
		//this.setLayout(new FlowLayout());
			
		setName("filterRaster");
			   		
   		this.loadInfoData();
		super.init(props);
		this.setTabVisible("Pansharpening", false);
	
		//this.add(getContentPane());
	
		this.setSize(486, 352);
		
		//contentPane.getAcceptButton().setEnabled(false);
		this.getAcceptButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getCancelButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
				closeJDialog();
			}
		});
		this.getApplyButton().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				acceptButtonActionPerformed(evt);
			}
		});
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		bandSetup.getFileList().getJButtonAdd().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				addFileBand(evt);
				
			}
		});
		bandSetup.getFileList().getJButtonRemove().addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt){
				delFileBand(evt);
			}
		});

	
	}
	
	/**
	 * Saves the initial state to restore it if cancel.
	 */
	public void readStat(){
		FilterRasterDialogPanel fr = ((FilterRasterDialogPanel)this.getContentPane());
		RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)fr.getPanelByClassName("RasterTransparencyPanel");
		status = new Status(rasterTrans.getOpacityText().getText(),
							getAssignedBand(GeoRasterFile.RED_BAND),
							getAssignedBand(GeoRasterFile.GREEN_BAND),
							getAssignedBand(GeoRasterFile.BLUE_BAND)
							);				
	}
	
	/**
	 * This method initializes jContentPane		
	 */    
	public Container getContentPane() {
		return this;
	}
	
	/**
	 * Sets a projection.
	 * 
	 * @param prj
	 */
	public void setProjection(IProjection prj) {
		this.currentProjection = prj;
	}
	
	
	
	public void closeJDialog() {
		PluginServices.getMDIManager().closeWindow(WCSRasterPropsDialog.this);
	}
	
	/**
	 * Sets the RasterFilterStackManager
	 * @param stackManager
	 */
	public void setRasterFilterStackManager(RasterFilterStackManager stackManager){
		this.stackManager = stackManager;
		stackManager.resetTempFilters();
	}
	
	/**
	 * 
	 * @param flag
	 * @return
	 */
	public int getAssignedBand(int flag) {
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		return bandSetup.getAssignedBand(flag);
	}
	
	/**
	 * Pressing OK with the bands panel selected makes its values to be
	 * processed.
	 * 
	 * @return true if the bands panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processBandPanel(){
		fLayer.setBandR(getAssignedBand(GeoRasterFile.RED_BAND));
		fLayer.setBandG(getAssignedBand(GeoRasterFile.GREEN_BAND));
		fLayer.setBandB(getAssignedBand(GeoRasterFile.BLUE_BAND));
		rasterStatus.bandR = getAssignedBand(GeoRasterFile.RED_BAND);
		rasterStatus.bandG = getAssignedBand(GeoRasterFile.GREEN_BAND);
		rasterStatus.bandB = getAssignedBand(GeoRasterFile.BLUE_BAND);
			
		//Comprobamos si hay alguna banda que no est� asignada y aplicamos el filtro
		StringBuffer sb = new StringBuffer();
		if(getAssignedBand(GeoRasterFile.RED_BAND) == -1)
			sb.append("R");
		if(getAssignedBand(GeoRasterFile.GREEN_BAND) == -1)
			sb.append("G");
		if(getAssignedBand(GeoRasterFile.BLUE_BAND) == -1)
			sb.append("B");
			
		if(!sb.toString().equals(""))
			stackManager.addRemoveBands(sb.toString());
		else
			stackManager.removeFilter(stackManager.getTypeFilter("removebands"));
			
		fLayer.getMapContext().invalidate();
		
		InfoPanel pInfo = (InfoPanel)super.getPanelByClassName("InfoPanel");
		pInfo.setBands(getAssignedBand(GeoRasterFile.RED_BAND),getAssignedBand(GeoRasterFile.GREEN_BAND),getAssignedBand(GeoRasterFile.BLUE_BAND));
		
		return true;
	}
	
	/**
	 * Pressing OK with the transparency panel selected makes its values to be processed.
	 * @return true if the transparency panel was selected and the action is processed, false it
	 * was not selected.
	 */
	public boolean processTransparencyPanel(){
		//OPACIDAD
		RasterTransparencyPanel tpan = (RasterTransparencyPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("RasterTransparencyPanel");
		String sOpac = tpan.getOpacityText().getText();
		if(!sOpac.equals("") && tpan.getOpacityCheck().isSelected()){
			int opac = Integer.parseInt(sOpac);
			opac = (int)((opac*255)/100);
			px.setTransparency(true);
			//px.setTransparency(255-opac);
			fLayer.setTransparency(255-opac);
			rasterStatus.transparency = 255-opac;
		}else{
			px.setTransparency(false);
			fLayer.setTransparency(0);
			rasterStatus.transparency = 0;
		}
				
		//TRANSPARENCIA
		if(	tpan.getTransparencyCheck().isSelected()){
			stackManager.addTransparencyFilter(	tpan.getPTranspByPixel().getEntries(),
													0x10,	//Transparencia
													0xff,	//Color Transparencia R
													0xff,	//Color Transparencia G
													0xff);	//Color Transparencia B
													
		}else
			px.filterStack.removeFilter(stackManager.getTypeFilter("transparency"));
			
		fLayer.getMapContext().invalidate();
		return true;	
	}
	
	/**
	 * Pressing OK with the BrightnessContrastPanel selected makes its values to be processed.
	 * @return true if the BrightnessContrastPanel was selected and the action is processed, false it
	 * was not selected.
	 */
	private void processBrightnessContrastPanel(){
		EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		// Si est� activo el panel de brillo y contraste tomamos los valores y cargamos un filtro de 
		// brillo y contraste
		
		BrightnessContrastStackManager bcStackManager = (BrightnessContrastStackManager)stackManager.getManagerByClass(BrightnessContrastStackManager.class);
		if(ebcPanel.getCBrightC().isSelected()){
			int incrBrillo = (int)Math.round(Double.valueOf(ebcPanel.lstBrightness.getTextValue()).doubleValue());
			int incrContraste = (int)Math.round(Double.valueOf(ebcPanel.lstContrast.getTextValue()).doubleValue());
			bcStackManager.addBrightnessFilter(incrBrillo);
			bcStackManager.addContrastFilter(incrContraste);
		}else{
			stackManager.removeFilter(bcStackManager.brightness);
			stackManager.removeFilter(bcStackManager.contrast);
		}	
		fLayer.getMapContext().invalidate();
	}
			
	/**
	 * Pulsar aceptar con el panel de realce seleccionado hace que se procesen los valores
	 * introducidos en este.
	 * @return true si estaba seleccionado el panel de realce y se ha procesado la
	 * acci�n y false si no lo estaba.
	 */
	public void processEnhancedPanel(){
		EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		
		//Filtro de realce lineal seleccionado
		if(ebcPanel.getCEnhanced().isSelected()){
			if((ebcPanel.getJCheckBox().isSelected()) && (!ebcPanel.getCheckSliderText().getJCheckBox().isSelected()))
				stackManager.addEnhancedFilter(true);
			else
				stackManager.addEnhancedFilter(false);
				
			//Recorte de colas seleccionado
			if(ebcPanel.getCheckSliderText().getJCheckBox().isSelected()){
				stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
				double recorte = Double.parseDouble(ebcPanel.getCheckSliderText().getTextValue())/100;
				if(ebcPanel.getJCheckBox().isSelected())
					stackManager.addTailFilter(recorte, 0D, true);
				else
					stackManager.addTailFilter(recorte, 0D, false);
			}else{
				stackManager.removeFilter(stackManager.getTypeFilter("tail"));
				stackManager.addComputeMinMaxFilter();
			}
			
		}
		// Sin filtro lineal seleccionado
		if(!ebcPanel.getCEnhanced().isSelected()){
			stackManager.removeFilter(stackManager.getTypeFilter("computeminmax"));
			stackManager.removeFilter(stackManager.getTypeFilter("tail"));
			stackManager.removeFilter(stackManager.getTypeFilter("enhanced"));
		}
		fLayer.getMapContext().invalidate();
	}
	
	/**
	 * Manages the action perfomed when Apply/OK is pressed or Apply at the raster
	 * properties control.
	 * @param e
	 */
	private void acceptButtonActionPerformed(ActionEvent e) {
		processBandPanel();
		processTransparencyPanel();	
		processEnhancedPanel();	
		processBrightnessContrastPanel();
	}
	
	/**
	 * Adds a band to the raster.
	 * @param e
	 */
	private void addFileBand(ActionEvent e){
		String[] driverNames = null;
		String rasterDriver = null;
				
		//Creaci�n del dialogo para selecci�n de ficheros
		
		fileChooser = new JFileChooser(lastPath);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setAcceptAllFileFilterUsed(false);
        try {
			driverNames = LayerFactory.getDM().getDriverNames();
			FileFilter defaultFileFilter = null, auxF;
			for (int i = 0; i < driverNames.length; i++) {
				
				if (driverNames[i].endsWith("gvSIG Image Driver")){
					rasterDriver = driverNames[i];
				    auxF = new DriverFileFilter(driverNames[i]);
					fileChooser.addChoosableFileFilter(auxF);
					defaultFileFilter = auxF;
				}
			}
		} catch (DriverLoadException e1) {
			NotificationManager.addError("No se pudo acceder a los drivers", e1);
		}
		int result = fileChooser.showOpenDialog(WCSRasterPropsDialog.this);
		
		if(result == JFileChooser.APPROVE_OPTION){
			File[] files = fileChooser.getSelectedFiles();
		 	FileFilter filter = fileChooser.getFileFilter();
		 	BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
		 	lastPath = files[0].getPath();
		 	
		 	//Lo a�adimos a la capa si no esta
		 	
		 	Vector v = new Vector();
            for(int i=0;i<files.length;i++){
            	boolean exist = false;
            	for(int j=0;j<px.getFiles().length;j++){
            		if(px.getFiles()[j].getName().endsWith(files[i].getName()))
            			exist = true;
            	}
            	if(!exist){
            		try{
            			Rectangle2D extentOrigin = fLayer.getFullExtent();
            			
            			Extent extentNewFile = GeoRasterFile.openFile(fLayer.getProjection(), files[i].getAbsolutePath()).getExtent();
            			
						//Comprobamos que el extent y tama�o del fichero a�adido sea igual al 
						//fichero original. Si no es as� no abrimos la capa y mostramos un aviso
						
            			double widthNewFile = (extentNewFile.getMax().getX()-extentNewFile.getMin().getX());
            			double heightNewFile = (extentNewFile.getMax().getY()-extentNewFile.getMin().getY());
            				                                                                                                
            			if( (widthNewFile-extentOrigin.getWidth()) > 1.0 ||
            				(widthNewFile-extentOrigin.getWidth()) < -1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) > 1.0 ||
            				(heightNewFile-extentOrigin.getHeight()) < -1.0){       
            				JOptionPane.showMessageDialog(	null,
            												PluginServices.getText(this, "extents_no_coincidentes"), 
															"",
															JOptionPane.ERROR_MESSAGE);
            				return;
            			}
            				                                
            			if(	(extentNewFile.getMax().getX()-extentNewFile.getMin().getX())!=extentOrigin.getWidth() ||
								(extentNewFile.getMax().getY()-extentNewFile.getMin().getY())!=extentOrigin.getHeight()	){
            				JOptionPane.showMessageDialog(null, 
									PluginServices.getText(this, "extents_no_coincidentes"), "", JOptionPane.ERROR_MESSAGE);
							return;
            			}
														
            		}catch(Exception exc){
            			exc.printStackTrace();
            		}
            		
            		//Lo a�adimos a la capa
            		px.addFile(files[i].getAbsolutePath());

            	}else{
            		JOptionPane.showMessageDialog(null, 
							PluginServices.getText(this, "fichero_existe")+" "+files[i].getAbsolutePath(), "", JOptionPane.ERROR_MESSAGE);
            	}
            }
		 	            
            //A�adimos los georasterfile a la tabla del Panel
            
            v = new Vector();
            Vector vInfo = new Vector();
            for(int i=0;i<px.getFiles().length;i++){
            	boolean exist = false;
            	for(int j=0;j<bandSetup.getNBands();j++){
            		if(px.getFiles()[i].getName().endsWith(bandSetup.getBandName(j)))
            			exist = true;
            	}
            	if(!exist){
            		v.add(px.getFiles()[i]);
            		vInfo.add(px.getFiles()[i]);
            	}
            }
            
            GeoRasterFile[] grf = new GeoRasterFile[v.size()];
            for(int i=0;i<grf.length;i++){
            	grf[i] = (GeoRasterFile)v.get(i);
            }
            GeoRasterFile[] infoGrf = new GeoRasterFile[vInfo.size()];
            for(int i = 0; i<infoGrf.length ; i++){
            	infoGrf[i] = (GeoRasterFile)vInfo.get(i);
            }
            bandSetup.addFiles(grf);
            this.addInfoFiles(infoGrf);
		}
	}
	
	/**
	 * Deletes a band from the raster. If there is only a file or no band is
	 * selected then it does nothing.
	 * 
	 * @param e
	 */
	private void delFileBand(ActionEvent e){
		BandSetupPanel bandSetup = (BandSetupPanel)((FilterRasterDialogPanel)this.getContentPane()).getPanelByClassName("BandSetupPanel");
	
		if(	bandSetup.getFileList().getJList().getSelectedValue()!=null &&
			bandSetup.getFileList().getNFiles() > 1){
			String pathName = bandSetup.getFileList().getJList().getSelectedValue().toString();
			px.delFile(pathName);
			String file = pathName.substring(pathName.lastIndexOf("/")+1);
			file = file.substring(file.lastIndexOf("\\")+1);
			bandSetup.removeFile(file);
			
		}	
		
		Vector vInfo = new Vector();
        for(int i=0;i<px.getFiles().length;i++){
        	vInfo.add(px.getFiles()[i]);
        }
		
		GeoRasterFile[] infoGrf = new GeoRasterFile[vInfo.size()];
        for(int i = 0; i<infoGrf.length ; i++){
        	infoGrf[i] = (GeoRasterFile)vInfo.get(i);
        }
        
        this.addInfoFiles(infoGrf);
        
	}
	
	/**
	 * The cancel button restores the previous state to the loading of this dialog
	 * @param e	Evento
	 */
	private void cancelButtonActionPerformed(ActionEvent e) {
		this.status.restoreStatus(this);
		fLayer.getMapContext().invalidate();
	}
	
	/**
	 * @see com.iver.mdiApp.ui.MDIManager.IWindow#getWindowInfo()
	 */
	public WindowInfo getWindowInfo() {
		WindowInfo m_viewinfo=new WindowInfo(WindowInfo.MODALDIALOG | WindowInfo.RESIZABLE);
    		m_viewinfo.setTitle(PluginServices.getText(this, "propiedades_raster"));
		return m_viewinfo;
	}
	
	//********************************************************************
	//***********************EVENTOS DE RAT�N*****************************
	
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		if (((e.getSource() == ebcPanel.lstBrightness.getJSlider()) || (e.getSource() == ebcPanel.lstContrast.getJSlider())) &&
			 (ebcPanel.getJCheckBox1().isSelected() == true))
			processBrightnessContrastPanel();		
	}

	public void keyPressed(KeyEvent e) {
		EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		if(e.getSource() == ebcPanel.getCheckSliderText().getJTextField()){
			ebcPanel.getCheckSliderText().getJSlider().setValue((int) Math.round(Double.valueOf(ebcPanel.getCheckSliderText().getTextValue()).doubleValue()));
		}
		
	}

	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	public void actionPerformed(ActionEvent e){
		EnhancedBrightnessContrastPanel ebcPanel = (EnhancedBrightnessContrastPanel)super.getPanelByClassName("EnhancedBrightnessContrastPanel");
		if((e.getSource() == ebcPanel.getLabelSliderText().getJTextField()) ||
			(e.getSource() == ebcPanel.getLabelSliderText1().getJTextField())){
				
			if(ebcPanel.getJCheckBox1().isSelected()){		
				processBrightnessContrastPanel();
			}
		
		}
	}
	
}  
