package com.iver.cit.gvsig.fmap.layers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;

import org.cresques.cts.ProjectionPool;
import org.gvsig.remoteClient.wfs.WFSAttribute;
import org.gvsig.remoteClient.wfs.WFSStatus;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.WFSException;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriver;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriverFactory;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FLyrWFS.java,v 1.10.2.4 2006/10/02 12:46:17 jorpiell Exp $
 * $Log: FLyrWFS.java,v $
 * Revision 1.10.2.4  2006/10/02 12:46:17  jorpiell
 * Aadidos los mtodos para escribirse y recuperarse del XMLEntity
 *
 * Revision 1.10.2.2  2006/09/28 08:54:01  jorpiell
 * Ya se puede reproyectar
 *
 * Revision 1.10.2.1  2006/09/26 07:36:24  jorpiell
 * El WFS no reproyectaba porque no se le asignaba a la capa un sistema de referencia. Ahora ya se hace.
 *
 * Revision 1.10  2006/09/05 15:41:52  jorpiell
 * Aadida la funcionalidad de cargar WFS desde el catlogo
 *
 * Revision 1.9  2006/07/05 12:05:41  jorpiell
 * Se ha modificado para que avise si se han recuperado las mismas features que marca el campo buffer
 *
 * Revision 1.8  2006/06/21 12:52:10  jorpiell
 * Aadido un icono para la capa WFS
 *
 * Revision 1.7  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.6  2006/06/15 11:17:06  jorpiell
 * Se ha encontrado la forma de comprobar cuando se prodicia un error de parseo al hacer un hasnext (en la feature). Se atrapa y se lanza la excepcin hacia arriba
 *
 * Revision 1.5  2006/06/14 07:57:19  jorpiell
 * Ya no se usa la estrategia ni se usa geotools para hacer el getFeature. Ahora se usa nicamente para el parseo de GML
 *
 * Revision 1.4  2006/06/05 16:49:31  caballero
 * poder editar y exportar
 *
 * Revision 1.3  2006/05/25 10:31:55  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.2  2006/05/23 13:21:59  jorpiell
 * Si hay algn problema en la carga se muestra un mensaje de error
 *
 * Revision 1.1  2006/05/19 12:54:11  jorpiell
 * Creada la capa WFS
 *
 *
 */
/**
 *  FMap's WFS Layer class.
 *
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FLyrWFS extends FLyrVect{
	private URL host = null;
	private String onlineResource = null;
	private String name = null;
	private String layerName = null;
    private String userName = null;
    private String password = null;
    private WFSAttribute[] fields = null;
    private int numfeatures = 100;
    private int timeout = 10000;
    private String srs = null;
    private FMapWFSDriver wfsDriver = null;
    private VectorialAdapter vectorialAdapter = null;
    private WFSLayerNode wfsLayerNode = null;
    
    /**
     * Constructor
     */
    public FLyrWFS(){
    	super();
    }
    
    /**
     * Constructor to load a new layer from a catalog
     * search
     * @param args
     * Params to load a new layer
     * @throws WFSException 
     * @throws IOException 
     * @throws MalformedURLException 
     * @throws ConnectException 
     * @throws DriverIOException 
     */
    public FLyrWFS(Map args) throws WFSException, ConnectException, MalformedURLException, IOException, DriverIOException{
    	String[] sLayer = (String[])args.get("layer");
		String user = (String)args.get("user");
		String pwd = (String)args.get("pwd");
		String host = (String)args.get("host");	
		String projection = (String)args.get("projection");
		
		FMapWFSDriver driver = FMapWFSDriverFactory.getFMapDriverForURL(new URL(host));
		WFSLayerNode[] layers = driver.getLayerList();
		WFSLayerNode layer = driver.getLayerInfo(sLayer[0]);
		
		layer.setSelectedFields(layer.getFields());
		
		WFSAttribute[] atts = new WFSAttribute[layer.getFields().size()];
		for (int i=0 ; i<atts.length ; i++){
			atts[i] = (WFSAttribute)layer.getFields().get(i);
		}
	
		setHost(host);
		setName(sLayer[0]);
		setLayerName(sLayer[0]);
		setWfsLayerNode(layer);
		setFields(atts);
		setUserName(user);
		setPassword(pwd);
		setNumfeatures(10000);
		setTimeout(10000);
		setWfsDriver(driver);			
		setProjection(CRSFactory.getCRS(projection));
		
    	load();    	
    }
    

    /**
     * Loads the features from the server
     */
    public void load() throws DriverIOException{
    	WFSAdapter adapter = new WFSAdapter();
    	try {
    		wfsDriver.getFeature(getWFSStatus());

    		adapter.setDriver((VectorialDriver) wfsDriver);
    		setSource(adapter);

    		setLegend(LegendFactory.createSingleSymbolLegend(
    				getShapeType()));
    	} catch (Exception e){
    		e.printStackTrace();
    		throw new DriverIOException(e.getMessage());
    	}
    }

    /**
     * Gets the WFS Status
     * @return
     */
    private WFSStatus getWFSStatus(){
    	WFSStatus status = new WFSStatus(getLayerName());
    	status.setUserName(getUserName());
    	status.setPassword(getPassword());
    	status.setBuffer(getNumfeatures());
    	status.setTimeout(getTimeout());
    	status.setFields(getFieldNames());
    	status.setOnlineResource(getOnlineResource());
    	status.setSrs(getSrs());
	   	return status;
    }

    /**
     * Adds the vectorial adapter to the layer
     * @param vectorialAdapter
     */
    public void setSource(VectorialAdapter vectorialAdapter) {
    	this.vectorialAdapter = vectorialAdapter;
        super.setSource(vectorialAdapter);
    }

	/**
	 * @return Returns the layerName.
	 */
	public String getLayerName() {
		return layerName;
	}
	/**
	 * @param layerName The layerName to set.
	 */
	public void setLayerName(String layerName) {
		this.layerName = layerName;
	}
	/**
	 * @return Returns the numfeatures.
	 */
	public int getNumfeatures() {
		return numfeatures;
	}
	/**
	 * @param numfeatures The numfeatures to set.
	 */
	public void setNumfeatures(int numfeatures) {
		this.numfeatures = numfeatures;
	}

	/**
	 * @return Returns the pwd.
	 */
	public String getPassword() {
		return password;
	}
	/**
	 * @param pwd The pwd to set.
	 */
	public void setPassword(String password) {
		this.password = password;
	}
	/**
	 * @return Returns the timeout.
	 */
	public int getTimeout() {
		return timeout;
	}
	/**
	 * @param timeout The timeout to set.
	 */
	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}
	/**
	 * @return Returns the user.
	 */
	public String getUserName() {
		return userName;
	}
	/**
	 * @param user The user to set.
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}
	/**
	 * @return Returns the fields.
	 */
	public WFSAttribute[] getFields() {
		if (fields == null){
			return new WFSAttribute[0];
		}
		return fields;
	}

	/**
	 * Return the attributes name
	 * @return
	 */
	public String[] getFieldNames(){
		String[] attributes = new String[getFields().length];
		for (int i=0 ; i<getFields().length ; i++){
			attributes[i] = getFields()[i].getName();
		}
		return attributes;
	}

	/**
	 * @param fields The fields to set.
	 */
	public void setFields(WFSAttribute[] fields) {
		this.fields = fields;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the host.
	 */
	public URL getHost() {
		return host;
	}
	/**
	 * @param host The host to set.
	 */
	public void setHost(URL host) {
		this.host = host;
	}

	/**
	 * @return Returns the wfs.
	 */
	public FMapWFSDriver getWfsDriver() {
		return wfsDriver;
	}

	/**
	 * @param wfs The wfs to set.
	 */
	public void setWfsDriver(FMapWFSDriver wfs) {
		this.wfsDriver = wfs;
	}


	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrDefault#getStrategy()
	 */
//    public Strategy getStrategy() {
//        if (wfsStrategy == null){
//        	wfsStrategy = new WFSStrategy(this);
//        }
//    	return wfsStrategy;
//    }

    /*
     *  (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
     */
    public Rectangle2D getFullExtent() throws DriverException {
    	try {
    		Rectangle2D extent = wfsDriver.getFullExtent();
    		if (getCoordTrans() != null){
    			return getCoordTrans().convert(extent);
    		}
			return extent;
		} catch (IOException e) {
			throw new DriverException(e.toString());
		}
	}

	/**
	 * @return Returns the onlineResource.
	 */
	public String getOnlineResource() {
		return onlineResource;
	}

	/**
	 * @param onlineResource The onlineResource to set.
	 */
	public void setOnlineResource(String onlineResource) {
		this.onlineResource = onlineResource;
	}

	public HashMap getProperties() {
		HashMap info = new HashMap();
		info.put("name", getName());
		info.put("layerName", getLayerName());
		info.put("layerText", getLayerText());
		info.put("attributes", getFields());
		info.put("host", getHost());	
		info.put("wfsLayerNode", getWfsLayerNode());	
		WFSStatus status = new WFSStatus(getLayerName());
		status.setBuffer(getNumfeatures());
		status.setTimeout(getTimeout());
		status.setUserName(getUserName());
		status.setPassword(getPassword());
		info.put("status",status);			
		return info;

	}

	/**
	 * @return Returns the wfsLayerNode.
	 */
	public WFSLayerNode getWfsLayerNode() {
		return wfsLayerNode;
	}

	/**
	 * @param wfsLayerNode The wfsLayerNode to set.
	 */
	public void setWfsLayerNode(WFSLayerNode wfsLayerNode) {
		this.wfsLayerNode = wfsLayerNode;
	}

	public void setHost(String host2) {
		try {
			setHost(new URL(host2));
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	public ImageIcon getTocImageIcon() {			
		return new ImageIcon(PluginServices.getPluginServices("com.iver.cit.gvsig.wfs2").getClassLoader().getResource("images/icoLayer.png"));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.layers.FLyrVect#isPropertiesMenuVisible()
	 */
	public boolean isPropertiesMenuVisible(){
		return false;
	}

	public String getSrs() {
		return srs;
	}

	public void setSrs(String srs) {
		this.srs = srs;
	}
	
	/**
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() throws XMLException {
		XMLEntity xml = super.getXMLEntity();		
		
		// Full extent
		try {
			xml.putProperty("fullExtent", StringUtilities.rect2String(getFullExtent()));
		} catch (DriverException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// Host
		xml.putProperty("host", host.toExternalForm());
		
		// Layer name
		xml.putProperty("layerName", getLayerName());

		//Layer fields
		String strFields = "";
		WFSAttribute[] fields = getFields();
		for (int i=0 ; i<fields.length ; i++){
			strFields = strFields + fields[i].getName();
			if (i < fields.length - 1){
				strFields = strFields + "~##SEP1##~";
			}
		}
		xml.putProperty("fields", strFields);
	        
		// User name
		xml.putProperty("user", getUserName());
	
		// SRS
		xml.putProperty("srs", getSrs());		

        // OnlineResources
        xml.putProperty("onlineResource", getOnlineResource());
      
    	// TimeOut
		xml.putProperty("timeout", getTimeout());		

        // Buffer
        xml.putProperty("buffer", getNumfeatures());
        
        //Projection
        xml.putProperty("projection",getProjection().getAbrev());
        
        return xml;
	}
	
	/**
	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public void setXMLEntity(XMLEntity xml)throws XMLException {
		super.setXMLEntity(xml);
		
		// Host
		try {
			host = new URL(xml.getStringProperty("host"));
		} catch (MalformedURLException e) {
			throw new XMLException(e);
		}
		
		// Layer name
		setLayerName(xml.getStringProperty("layerName"));		
		
		// User name
		if (xml.contains("user")){
			setUserName(xml.getStringProperty("user"));
		}
		
		// SRS
		if (xml.contains("srs")){
			setSrs(xml.getStringProperty("srs"));
		}
		
		// OnlineResources
		if (xml.contains("onlineResource")){
			setOnlineResource(xml.getStringProperty("onlineResource"));
		}
		
		// TimeOut
		if (xml.contains("timeout")){
			setTimeout(xml.getIntProperty("timeout"));
		}
		
		// Buffer
		if (xml.contains("buffer")){
			setNumfeatures(xml.getIntProperty("buffer"));
		}
		
		 //Projection
		if (xml.contains("projection")){
			setProjection(CRSFactory.getCRS(xml.getStringProperty("projection")));
		}              
		
		// Layer fields
		FMapWFSDriver driver;
		try {
			driver = FMapWFSDriverFactory.getFMapDriverForURL(host);
		} catch (Exception e) {
			throw new XMLException(e);	
		}
		WFSLayerNode[] layers = driver.getLayerList();
		WFSLayerNode layer = driver.getLayerInfo(getLayerName());			
		layer.setSelectedFields(layer.getFields());					
		WFSAttribute[] atts = null;
		
		if (xml.contains("fields")) {
			String[] fields = xml.getStringProperty("fields").split("~##SEP1##~");
			atts = new WFSAttribute[fields.length];
			for (int i=0 ; i<fields.length ; i++){
				for (int j=0 ; j<layer.getFields().size() ; j++){
					WFSAttribute attribute = (WFSAttribute)layer.getFields().get(j);
					if (attribute.getName().equals(fields[i])){
						atts[i] = attribute;
						break;
					}
				}					
			}
		}else{
			atts = new WFSAttribute[0];
		}
		layer.setSelectedFields(atts);
		setWfsDriver(driver);	
		setWfsLayerNode(layer);
		setFields(atts);	
		setAvailable(true);
	}
}
