package com.iver.cit.gvsig.gui.panels;

import java.util.Vector;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import org.gvsig.remoteClient.wfs.WFSAttribute;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.wfs.WFSUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: AttributesTable.java,v 1.3.2.1 2006/09/19 12:28:11 jorpiell Exp $
 * $Log: AttributesTable.java,v $
 * Revision 1.3.2.1  2006/09/19 12:28:11  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.4  2006/09/18 12:07:31  jorpiell
 * Se ha sustituido geotools por el driver de remoteservices
 *
 * Revision 1.3  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 * Revision 1.2  2006/05/25 16:01:43  jorpiell
 * Se ha aadido la funcionalidad para eliminar el namespace de los tipos de atributos
 *
 * Revision 1.1  2006/05/25 10:29:28  jorpiell
 * Aadida la clase
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class AttributesTable extends JTable {
	
	public AttributesTable() {
		super();
		setModel(new AttributesTableModel());
	}
	
	/**
	 * Returns all the attributes
	 * @return
	 */
	public Object[] getAllValues(){
		AttributesTableModel model = (AttributesTableModel)getModel();
		return model.getAllValues();
	}
	
	/**
	 * Gets the selected layer
	 * @return
	 */
	public Object[] getSelectedValues(){
		AttributesTableModel model = (AttributesTableModel)getModel();
		int[] selectedRows = getSelectedRows();
		Object[] attributes = new Object[selectedRows.length];
		for (int i=0 ; i<attributes.length ; i++){
			attributes[i] = model.getAttributeAt(selectedRows[i]);
		}
		return attributes;
	}
	
	/**
	 * Adds a vector of features
	 * @param features
	 */
	public void addAttributes(Object[] attributes){
		AttributesTableModel model = (AttributesTableModel)getModel();
		model.deleteAllRows();
		for (int i=0 ; i<attributes.length ; i++){
			model.addRow((WFSAttribute)attributes[i]);			
		}		
	}	
	
	
	/**
	 * Table model
	 * @author Jorge Piera Llodr (piera_jor@gva.es)
	 *
	 */
	public class AttributesTableModel extends AbstractTableModel{  
		private Vector attributes = new Vector();
			
		/**
		 Constructs an investment table model.
		 */
		public AttributesTableModel(){  
			super();
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount(){  
			return attributes.size();		
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		public int getColumnCount(){  
			return 2;
		}
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int rowNumber, int columnNumber){  
			if (rowNumber < attributes.size()){
				WFSAttribute attribute = (WFSAttribute)attributes.get(rowNumber);
				if (columnNumber == 0){
					return attribute.getName();
				}else{
					return PluginServices.getText(this,WFSUtils.getAttributeType(attribute)); 
				}
			}else{
				return "";
			}
		}
		
		/**
		 * Returns all the attributes
		 * @return
		 */
		public Object[] getAllValues(){
			return attributes.toArray();
		}
		
		/**
		 * Return the attribute at the specified position
		 * @param rowNumber
		 * Row position
		 * @return
		 */
		public WFSAttribute getAttributeAt(int rowNumber){  
			try{
				return (WFSAttribute)attributes.get(rowNumber);
			}catch (ArrayIndexOutOfBoundsException e){
				return null;
			}
		}	
		
		/**
		 * It adds a new attribute
		 * @param values
		 * Array of column values
		 */
		public void addRow(WFSAttribute attribute){
			attributes.add(attribute);		
			fireTableRowsInserted(getRowCount(),getRowCount());
			fireTableRowsUpdated(0,getRowCount());
		}
		
		/**
		 * Delete all the table rows
		 */
		public void deleteAllRows(){
			attributes.clear();
			int rows = getRowCount();
			if (rows >= 1){
				fireTableRowsDeleted(0, rows - 1);
			}		
		}	
		
		/**
		 * Delete all the table rows
		 */
		public void deleteRow(int rowPosition){
			attributes.remove(rowPosition);
			fireTableRowsDeleted(rowPosition, rowPosition);
			fireTableRowsUpdated(0,getRowCount());		
		}	
		
		/*
		 *  (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnName(int)
		 */
		public String getColumnName(int columnIndex){
			if (columnIndex == 0){
				return PluginServices.getText(this,"attributeName");
			}else{
				return PluginServices.getText(this,"attributeType");
			}
		}		
	}

}
