package org.gvsig.remoteClient.arcims.styling.renderers;

import java.io.File;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.gvsig.remoteClient.arcims.ArcImsProtFeatureHandler;
import org.gvsig.remoteClient.arcims.exceptions.ArcImsException;
import org.gvsig.remoteClient.arcims.utils.ServiceInformation;
import org.gvsig.remoteClient.arcims.utils.ServiceInformationLayerFeatures;

import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.SingleSymbolLegend;
import com.iver.cit.gvsig.fmap.rendering.VectorialUniqueValueLegend;

public class ArcImsFLegendFactoryTest extends TestCase {
	String baseFilePath = "D:/04_ECLIPSE/gvSIG_10RC1/libArcIMS/test";
	private static Logger logger = Logger.getLogger(ArcImsFLegendFactoryTest.class.getName());
	String log4jprops = baseFilePath+"/test-data/log4j.properties";
	private ArcImsFLegendFactory aiff;
	
    
    String xmlFilePath = baseFilePath;
    File f;
    
    ArcImsProtFeatureHandler aipf;
    ServiceInformation my_si;
    
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(ArcImsFLegendFactoryTest.class);
	}

	public ArcImsFLegendFactoryTest(String arg0) {
		super(arg0);
		//Configurar el log4j
		PropertyConfigurator.configure(log4jprops);
	}

	protected void setUp() throws Exception {
		super.setUp();
		aipf = new ArcImsProtFeatureHandler();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'org.gvsig.remoteClient.arcims.styling.renderers.ArcImsFLegendFactory.getMainLegend()'
	 */
	public final void testBugCTH() {
		xmlFilePath+="/test-data/serviceInfo-bug-CTH.xml";
		f = new File(xmlFilePath);
		try {
			my_si = aipf.parseServiceInfo(new ServiceInformation(),f);
			
	    	//Get the ServiceInformationLayer object
			ServiceInformationLayerFeatures silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(0);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
			
			/* If org.gvsig.remoteClient.arcims.styling.renderers.ArcImsRendererFactoryTest
			 * doesn't fail, we can continue testing the FMap objects creation as we are sure
			 * that ArcIMS objetcts have been correctly created
			*/
			
			//At this point we can test the 
			Legend leg;
			leg = aiff.getMainLegend();
			
			//Check colors
			assertEquals(leg.getDefaultSymbol().getColor().getRed(),192);
			assertEquals(leg.getDefaultSymbol().getColor().getBlue(),192);
			assertEquals(leg.getDefaultSymbol().getColor().getGreen(),192);
			
			//Check font
			logger.debug(leg.getDefaultSymbol().getFont().getFontName());
			assertEquals(leg.getDefaultSymbol().getFont().getFontName(),"Arial");
			assertEquals(Float.toString(leg.getDefaultSymbol().getFontSize()),Float.toString((float) 10.0));
		} catch (ArcImsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	/*
	 * Test method for 'org.gvsig.remoteClient.arcims.styling.renderers.ArcImsFLegendFactory.getMainLegend()'
	 */
	public final void testNormalScaleDependentRenderer() {
		xmlFilePath+="/test-data/serviceInfo-scale-dep.xml";
		f = new File(xmlFilePath);
		try {
			my_si = aipf.parseServiceInfo(new ServiceInformation(),f);
			
	    	//Get the FIRST ServiceInformationLayer object
			ServiceInformationLayerFeatures silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(0);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
			
			/* If org.gvsig.remoteClient.arcims.styling.renderers.ArcImsRendererFactoryTest
			 * doesn't fail, we can continue testing the FMap objects creation as we are sure
			 * that ArcIMS objetcts have been correctly created
			*/
			
			//At this point we can test the 
			Legend leg;
			leg = aiff.getMainLegend();
			
			//Check colors
			assertEquals(leg.getDefaultSymbol().getColor().getRed(),129);
			assertEquals(leg.getDefaultSymbol().getColor().getGreen(),129);
			assertEquals(leg.getDefaultSymbol().getColor().getBlue(),204);

			
	    	//Get the SECOND ServiceInformationLayer object
			silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(1);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
			
			/* If org.gvsig.remoteClient.arcims.styling.renderers.ArcImsRendererFactoryTest
			 * doesn't fail, we can continue testing the FMap objects creation as we are sure
			 * that ArcIMS objetcts have been correctly created
			*/
			
			//At this point we can test the 
			leg = aiff.getMainLegend();
			
			//Check types
			assertEquals(leg.getClass(),VectorialUniqueValueLegend.class);
			
		} catch (ArcImsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/*
	 * Test method for 'org.gvsig.remoteClient.arcims.styling.renderers.ArcImsFLegendFactory.getMainLegend()'
	 */
	public final void testLabels() {
		ServiceInformationLayerFeatures silf;
		
		xmlFilePath+="/test-data/serviceInfo-labels-tests.xml";
		f = new File(xmlFilePath);
		try {
			my_si = aipf.parseServiceInfo(new ServiceInformation(),f);
			
	    	//Get the FIRST ServiceInformationLayer object
			silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(0);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
			
			/* If org.gvsig.remoteClient.arcims.styling.renderers.ArcImsRendererFactoryTest
			 * doesn't fail, we can continue testing the FMap objects creation as we are sure
			 * that ArcIMS objetcts have been correctly created
			*/
			//At this point we can test the 
			Legend leg;
			leg = aiff.getMainLegend();
			
			//Check colors
			assertEquals(leg.getDefaultSymbol().getColor().getRed(),0);
			assertEquals(leg.getDefaultSymbol().getColor().getGreen(),204);
			assertEquals(leg.getDefaultSymbol().getColor().getBlue(),204);

			
	    	//Get the SECOND ServiceInformationLayer object
			silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(1);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
					
			//At this point we can test the 
			leg = aiff.getMainLegend();
			
			//Check types and label field
			assertEquals(leg.getClass(),SingleSymbolLegend.class);
			assertEquals(((SingleSymbolLegend) leg).getLabelField(),"CURR_TYPE");
			assertEquals(leg.getDefaultSymbol().isShapeVisible(),false);
			
			
	    	//Get the THIRD ServiceInformationLayer object
			silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(2);
			assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
			
			//Build the ArcImsFLegendFactory object
			aiff = new ArcImsFLegendFactory(my_si,silf.getId());
			assertNotNull(aiff);
					
			//At this point we can test the 
			leg = aiff.getMainLegend();
			
			//Check types and label field
			assertEquals(leg.getClass(),SingleSymbolLegend.class);
			assertEquals(((SingleSymbolLegend) leg).getLabelField(),"NAME");
			assertEquals(leg.getDefaultSymbol().isShapeVisible(),false);
		} catch (ArcImsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
