package org.gvsig.remoteClient.arcims.styling.renderers;

import java.io.File;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.gvsig.remoteClient.arcims.ArcImsProtFeatureHandler;
import org.gvsig.remoteClient.arcims.exceptions.ArcImsException;
import org.gvsig.remoteClient.arcims.styling.symbols.SimplePolygonSymbol;
import org.gvsig.remoteClient.arcims.styling.symbols.TextSymbol;
import org.gvsig.remoteClient.arcims.utils.ServiceInformation;
import org.gvsig.remoteClient.arcims.utils.ServiceInformationLayerFeatures;

import junit.framework.TestCase;

public class ArcImsRendererFactoryTest extends TestCase {
	 String baseFilePath = "D:/04_ECLIPSE/gvSIG_10RC1/libArcIMS/test";
	    String xmlFilePath = baseFilePath+"/test-data/serviceInfo-bug-CTH.xml";
	    String log4jprops = baseFilePath+"/test-data/log4j.properties";
	    File f;
	    private static Logger logger = Logger.getLogger(ArcImsFLegendFactoryTest.class.getName());
	    ServiceInformation my_si;
	    
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(ArcImsRendererFactoryTest.class);
	}

	protected void setUp() throws Exception {
		super.setUp();
		//Configurar el log4j
		PropertyConfigurator.configure(log4jprops);
		f = new File(xmlFilePath);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/*
	 * Test method for 'org.gvsig.remoteClient.arcims.styling.renderers.ArcImsRendererFactory.getRenderer(KXmlParser)'
	 */
	public void testGetRenderer() {
		ArcImsProtFeatureHandler aipf = new ArcImsProtFeatureHandler();
		try {
			my_si = aipf.parseServiceInfo(new ServiceInformation(),f);
	    } catch (ArcImsException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	    
		//Get the ServiceInformationLayer object
		ServiceInformationLayerFeatures silf =  (ServiceInformationLayerFeatures) my_si.getLayers().get(0);
		assertEquals(silf.getClass(),ServiceInformationLayerFeatures.class);
		
	    //Test if renderers have been parsed
		Renderer mainRenderer = silf.getLayerMainRenderer();
		assertNotNull(mainRenderer);
		
		logger.debug("\n"+mainRenderer);
				
		//Test structure of renderer, symbols and so on
		//If it is a group
		assertEquals(mainRenderer.getClass(),GroupRenderer.class);
		GroupRenderer group = (GroupRenderer) mainRenderer;
		
		//First element of the group and it symbol
		assertEquals(group.getRenderer(0).getClass(),SimpleRenderer.class);
		SimpleRenderer simple = (SimpleRenderer) group.getRenderer(0);
		assertEquals(simple.getSymbol().getClass(),SimplePolygonSymbol.class);
		
		//Second element of the group and it symbol (a label renderer)
		assertEquals(group.getRenderer(1).getClass(),ScaleDependentRenderer.class);
		ScaleDependentRenderer scale = (ScaleDependentRenderer) group.getRenderer(1);
		assertEquals(scale.getRenderer(0).getClass(),SimpleLabelRenderer.class);
		SimpleLabelRenderer label = (SimpleLabelRenderer) scale.getRenderer(0); 
		assertEquals(label.getTextSymbol().getClass(),TextSymbol.class);
		TextSymbol text = (TextSymbol) label.getTextSymbol();
		assertEquals(text.getFont(),"Arial");
	}

}
