/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.geo.cover;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;
import org.cresques.cts.gt2.CoordSys;
//import org.cresques.cts.gt2.CoordTrans;

import org.cresques.geo.Projected;
import org.cresques.geo.Projection;
import org.cresques.geo.ReProjection;

import org.cresques.io.GeoRasterFile;

import org.cresques.px.*;

import org.cresques.ui.CQApp;

import java.awt.Color;

import java.io.File;
import java.io.FilenameFilter;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;


/**
 * Cobertura espacial
 *
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class Coverage implements Projected {
    boolean debug = true;

    /*        protected FileSupport [] fSupport = {
                    new FileSupport("ecw", EcwFile.class)
            };*/

    // Nombre de la cobertura
    protected String name = "Orto ICV 1:5.000";
    protected String fName = "cover";

    // Direccin raiz de los datos
    protected String dName = null; //"//sercart/cartografia/ortofotos/Orto_ICV/00009";
    protected int nLevels = 1;
    protected String hojaPrefix = "";
    protected Hashtable hojas = null;
    private Hashtable files = null;
    protected PxObjList marcos;
    public Color color = null;
    public Color fillColor = null;
    protected boolean loadable = true;
    private String fType = "";
    protected IProjection proj = null;
    private boolean available = true;

    public Coverage() {
        hojas = new Hashtable();
        files = new Hashtable();
    }

    public String getName() {
        return name;
    }

    public Hashtable getHojas() {
        return hojas;
    }

    public IProjection getProjection() {
        return proj;
    }

    public void setProjection(IProjection p) {
        proj = p;
    }

    public void reProject(ICoordTrans rp) {
        // TODO metodo reProject pendiente de implementar
    }

    /**
     * Asigna el estado de disponibilidad.
     * Cuando una cobertura no esta disponible (la unidad est
     * inactiva, no tiene ficheros, etc...) deber ponerse a false.
     * Es true por defecto
     * @param av Disponibilidad.
     */
    public void setAvailable(boolean av) {
        available = av;
    }

    /**
     * Devuelve el estado de disponibilidad.
     * @return Disponibilidad.
     */
    public boolean getAvailable() {
        return available;
    }

    /**
     * Especifica el tipo de los ficheros que forman la cobertura.
     * Es un string que ser procesado por la aplicacin. La clase
     * Coverage no hace ningn tratamiento de este valor.
     * @param fType Tipo de ficheros.
     */
    public void setFileType(String fType) {
        this.fType = fType;
    }

    /**
     * Devuelve el tipo de fichero.
     * @return Tipo de fichero.
     */
    public String getFileType() {
        return fType;
    }

    public void catalogue(String[] files) {
        String hName = null;

        for (int i = 0; i < files.length; i++) {
            hName = hojaPrefix + files[i];
            hojas.put(hName, files[i]);

            if (debug) {
                System.out.println("Coverage: hojas.put('" + hName + "'");
            }
        }
    }

    public void catalogue() {
        catalogue(dName);
    }

    /**
     * Hace el catlogo a partir de la lista de ficheros.
     * @param files
     */
    public void catalogueList(String[][] files) {
        IProjection prj = proj;
        String hName = null;

        for (int i = 0; i < files.length; i++) {
            if (files[i][2] != null) {
                if (proj instanceof Projection) {
                    prj = (Projection) Projection.getProjectionByName(proj.getDatum(),
                                                                      files[i][2]);
                } else if (proj instanceof CoordSys) {
                    prj = new ProjectionPool().get(files[i][2]);
                }
            }

            hName = files[i][0];
            hojas.put(hName, new Hoja(prj, files[i][1], files[i][0]));

            if (debug) {
                System.out.println("Coverage: hojas.put('" + hName + "'");
            }
        }
    }

    /**
     * Hace el catlogo recorriendo un directorio.
     * @param dName
     */
    public void catalogue(String dName) {
        System.out.println("Catalogando '" + dName + "' ...");

        File dir = new File(dName);

        if (!dir.exists()) {
            System.err.println("Que dice que '" + dName + "'no existe");

            return;
        }

        String[] files = dir.list();

        if (files != null) {
            catalogue(files);
        }
    }

    public IObjList[] loadMinuteo() {
        IObjList[] minuteo = null;

        return minuteo;
    }

    public void saveMinuteo(IObjList[] minuteo) {
        for (int i = 0; i < minuteo.length; i++) {
        }
    }

    /**
     * Genera el minuteo, reproyectado.
     * @param targetProj proyeccion de destino
     * @return array de minuteos
     */
    public IObjList[] generateMinuteo(IProjection targetProj) {
        IObjList[] minuteo = loadMinuteo();

        if (minuteo != null) {
            return minuteo;
        }

        minuteo = new IObjList[nLevels];

        for (int i = 0; i < minuteo.length; i++)
            minuteo[i] = new PxObjList();

        if (hojas.size() == 0) {
            catalogue(this.dName);
        }

        marcos = new PxObjList();

        IObjList layer = minuteo[0];
        IObjList layer1 = null;

        if (nLevels > 1) {
            layer1 = minuteo[1];
        }

        Enumeration enum = hojas.keys();
        String hName;
        String hCode;
        Hoja h;
        Object obj = null;

        while (enum.hasMoreElements()) {
            hName = (String) enum.nextElement();

            try {
                obj = hojas.get(hName);

                if (obj instanceof String) {
                    hCode = (String) obj;
                    generateMinuteoHoja(dName + "/" + hCode, hName, layer,
                                        layer1);
                } else if (obj instanceof Hoja) {
                    h = (Hoja) obj;
                    generateMinuteoHoja(dName + "/" + h.getCode(), h, layer,
                                        layer1);
                }
            } catch (Exception e) {
                // TODO Tengo que tratar los errores de una forma mejor.
                System.err.println("Error al cargar la hoja " + hName);
                e.printStackTrace();
            }
        }

        saveMinuteo(minuteo);

        //app.setCurrentMinuteo(layer);
        if (proj != targetProj) {
            ICoordTrans rp = null;

            if (proj instanceof Projection) {
                rp = new ReProjection(((Projection) proj),
                                      ((Projection) targetProj));
            } else if (proj instanceof CoordSys) {
                //rp = new CoordTrans((CoordSys) proj, (CoordSys) targetProj);
            }

            layer.reProject(rp);

            if (layer1 != null) {
                layer1.reProject(rp);
            }
        }

        return minuteo;
    }

    public void generateMinuteo(CQApp app, String lName, String lName1) {
        PxLayer layer1 = null;

        if (color == null) {
            color = new Color(64, 220, 64, 255);
        }

        if (fillColor == null) {
            fillColor = new Color(128, 220, 128, 32);
        }

        if (hojas.size() == 0) {
            catalogue(this.dName);
        }

        marcos = new PxObjList();

        if (nLevels > 0) {
            layer1 = app.createLayer(name + lName1, proj);
            layer1.setColor(color);
            layer1.setFillColor(fillColor);
        }

        PxLayer layer = app.createLayer(name + lName, proj);
        layer.setColor(color);
        layer.setFillColor(fillColor);

        Enumeration enum = hojas.keys();
        String hName;
        String hCode;
        Hoja h;
        Object obj = null;

        while (enum.hasMoreElements()) {
            hName = (String) enum.nextElement();
            obj = hojas.get(hName);

            if (obj.getClass() == String.class) {
                hCode = (String) obj;
                generateMinuteoHoja(dName + "/" + hCode, hName, layer, layer1);
            } else if (obj.getClass() == Hoja.class) {
                h = (Hoja) obj;
                generateMinuteoHoja(dName + "/" + h.getCode(), h, layer, layer1);
            }
        }

        app.setCurrentMinuteo(layer);

        if (proj != app.getCurrentProjection()) {
            ICoordTrans rp = null;

            if (proj instanceof Projection) {
                rp = new ReProjection(((Projection) proj),
                                      ((Projection) app.getCurrentProjection()));
            }

            layer.reProject(rp);

            if (layer1 != null) {
                layer1.reProject(rp);
            }
        }
    }

    public void generateMinuteoHoja(String hFName, Hoja h, IObjList layer,
                                    IObjList layer1) {
        GeoRasterFile eFile = null;
        PxContour contour = null;
        String fName = null;
        File dir = new File(hFName);

        if (!hFName.startsWith("ecwp:") && !dir.exists()) {
            System.err.println("Hoja '" + hFName + "' Inexistente");

            return;
        }

        if (dir.isDirectory()) {
        } else {
            if (!GeoRasterFile.fileIsSupported(hFName)) {
                return;
            }

            fName = hFName;
            System.out.println("Analizo '" + h.getName() + "' ...");
            eFile = GeoRasterFile.openFile(null, fName);
            contour = new PxContour(eFile.getExtent(), fName, h.getName(),
                                    h.getProjection());

            if (proj != h.getProjection()) {
                ICoordTrans rp = null;

                if (proj instanceof Projection) {
                    rp = new ReProjection(((Projection) h.getProjection()),
                                          ((Projection) proj));
                } else if (proj instanceof CoordSys) {
                   /* rp = new CoordTrans((CoordSys) h.getProjection(),
                                        (CoordSys) proj);*/
                }

                contour.reProject(rp);
            }

            eFile.close();
            contour.c(color);
            contour.fillColor(fillColor);
            layer.add(contour);
        }
    }

    public void generateMinuteoHoja(String hName, String name, IObjList layer,
                                    IObjList layer1) {
        GeoRasterFile eFile = null;
        PxContour contour = null;
        String fName = null;
        File dir = new File(hName);

        if (!hName.startsWith("ecwp:") && !dir.exists()) {
            System.err.println("Hoja '" + hName + "' Inexistente");

            return;
        }

        if (dir.isDirectory()) {
            FilenameFilter filter = new FilenameFilter() {
                public boolean accept(File dir, String fName) {
                    return GeoRasterFile.fileIsSupported(fName);
                }
            };

            String[] files = dir.list(filter);
            System.out.print(files.length + " ficheros.");

            if (files.length > 0) {
                Extent extent = new Extent();

                for (int i = 0; i < files.length; i++) {
                    fName = hName + "/" + files[i];
                    System.out.println("Analizo '" + fName + "' ...");
                    eFile = GeoRasterFile.openFile(null, fName);
                    contour = new PxContour(eFile.getExtent(), fName, files[i],
                                            proj);
                    eFile.close();

                    //contour = EcwFile.getContour(fName, files[i], proj);
                    layer1.add(contour);

                    //					mapview.loadECW(hName+"/"+fName);
                    extent.add(contour.getExtent());
                }

                if (contour != null) {
                    contour = new PxContour(extent, hName, name, proj);
                    contour.c(color);
                    contour.fillColor(fillColor);
                    layer.add(contour);
                }
            }
        } else {
            if (!GeoRasterFile.fileIsSupported(hName)) {
                return;
            }

            fName = hName;
            System.out.println("Analizo '" + name + "' ...");

            //contour = EcwFile.getContour(fName, name, proj);
            eFile = GeoRasterFile.openFile(null, fName);
            contour = new PxContour(eFile.getExtent(), fName, name, proj);
            eFile.close();
            contour.c(color);
            contour.fillColor(fillColor);
            layer.add(contour);

            //			mapview.loadECW(hName+"/"+fName);
        }
    }

    /**
     * Devuelve los ficheros que incluye la hoja.
     * @param hoja hoja que se busca.
     * @return nombre(s) de fichero(s).
     */
    public ArrayList getFileNames(String hoja) {
        ArrayList fNames = new ArrayList();

        if (!loadable) {
            return null;
        }

        String fName = null;
        Object obj = hojas.get(hoja);
        String hCode = null;
        String hDName = null;
        File dir = null;

        if (obj != null) {
            if (obj instanceof String) {
                hCode = (String) obj;
            } else if (obj instanceof Hoja) {
                hCode = ((Hoja) obj).getCode();
            }

            hDName = dName + "/" + hCode;
            System.out.print("Explorando '" + hDName + "' (" + hoja + ") ... ");
            dir = new File(hDName);
        }

        if ((obj == null) || (!hDName.startsWith("ecwp:") && !dir.exists())) {
            System.err.println("Que dice que no existe");

            return null;
        }

        String[] files; /* = dir.list();
        /*                if (children == null) {
        // Either dir does not exist or is not a directory
        } else {
        for (int i=0; i<children.length; i++) {
         // Get filename of file or directory
         String filename = children[i];
        }
        } */

        // It is also possible to filter the list of returned files.
        // This example does not return any files that start with `.'.
        if (dir.isDirectory()) {
            FilenameFilter filter = new FilenameFilter() {
                public boolean accept(File dir, String fName) {
                    return GeoRasterFile.fileIsSupported(fName);
                }
            };

            files = dir.list(filter);
            System.out.print(files.length + " ficheros.");

            if (files.length > 0) {
                //PxLayer layer = app.createLayer(name+hoja, proj);
                for (int i = 0; i < files.length; i++) {
                    fName = files[i];
                    System.out.println("Cargo ...'" + fName + "'");
                    fNames.add(hDName + "/" + fName);

                    //mapview.canvas.zoomExtents();
                    //app.getCanvas().repaint();
                }
            }
        } else {
            //PxLayer layer = app.getLayerByName(name);
            //if (layer == null) {
            //	layer = app.createLayer(name, proj);
            //}
            //app.setCurrentLayer(layer);
            fName = hDName;

            if (!GeoRasterFile.fileIsSupported(fName)) {
                return null;
            }

            System.out.println("Cargo ...'" + fName + "'");
            fNames.add(fName);

            //mapview.canvas.zoomExtents();
            //app.getCanvas().repaint();
        }

        return fNames;
    }

    public void loadHoja(CQApp app, String hoja) {
        if (!loadable) {
            return;
        }

        String fName = null;
        Object obj = hojas.get(hoja);
        String hCode = null;

        if (obj.getClass() == String.class) {
            hCode = (String) obj;
        } else if (obj.getClass() == Hoja.class) {
            hCode = ((Hoja) obj).getCode();
        }

        String hDName = dName + "/" + hCode;
        System.out.print("Explorando '" + hDName + "' (" + hoja + ") ... ");

        File dir = new File(hDName);

        if ((!hDName.startsWith("ecwp:") && !dir.exists()) ||
                (hojas.get(hoja) == null)) {
            System.err.println("Que dice que no existe");

            return;
        }

        String[] files; /* = dir.list();
        /*                if (children == null) {
        // Either dir does not exist or is not a directory
        } else {
        for (int i=0; i<children.length; i++) {
         // Get filename of file or directory
         String filename = children[i];
        }
        } */

        // It is also possible to filter the list of returned files.
        // This example does not return any files that start with `.'.
        if (dir.isDirectory()) {
            FilenameFilter filter = new FilenameFilter() {
                public boolean accept(File dir, String fName) {
                    return GeoRasterFile.fileIsSupported(fName);
                }
            };

            files = dir.list(filter);
            System.out.print(files.length + " ficheros.");

            if (files.length > 0) {
                PxLayer layer = app.createLayer(name + hoja, proj);

                for (int i = 0; i < files.length; i++) {
                    fName = files[i];
                    System.out.println("Cargo ...'" + fName + "'");
                    app.loadFile(hDName + "/" + fName, proj);

                    //mapview.canvas.zoomExtents();
                    app.getCanvas().repaint();
                }
            }
        } else {
            PxLayer layer = app.getLayerByName(name);

            if (layer == null) {
                layer = app.createLayer(name, proj);
            }

            app.setCurrentLayer(layer);
            fName = hDName;

            if (!GeoRasterFile.fileIsSupported(fName)) {
                return;
            }

            System.out.println("Cargo ...'" + fName + "'");
            app.loadFile(fName, proj);

            //mapview.canvas.zoomExtents();
            app.getCanvas().repaint();
        }
    }
}
