/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.data;


/**
* Opciones soportadas por los drivers de escritura
* @author Nacho Brodin (brodin_ign@gva.es)
*/
public class WriterSupportOptions {
    private String extension = null;
    private String[] blockSizeList = {
                                         "1", "8", "16", "32", "64", "128",
                                         "256", "512"
                                     };
    private boolean georef = true;
    private int blockSizeDefault = 1;

    /**
     * Constructor
     * @param extension        Extensin del driver
     */
    public WriterSupportOptions(String extension) {
        this.extension = extension;
    }

    /**
     * Asigna el tamao de bloque que se lee de una vez en la imagen origen
     * @param blockSize        Tamao de bloque
     */
    public void setBlockSize(int blockSize) {
        this.blockSizeDefault = blockSize;
    }

    /**
     * Se asigna true o false si se desea o no que el fichero de salida tenga georeferenciacin.
     * @param geor true salva con georeferenciacin
     */
    public void setWriteGeoref(boolean geor) {
        this.georef = geor;
    }

    /**
     * Obtiene el valor del flag que dice si se desea o no salvar con georeferenciacin
     * @return        true si se desea salvar con georeferenciacin
     */
    public boolean getGeoref() {
        return georef;
    }

    /**
     * Obtiene el tamao de bloque seleccionado
     * @return        Tamao de bloque
     */
    public int getBlockSize() {
        return blockSizeDefault;
    }

    /**
     * Obtiene la extensin del driver
     * @return        Extensin del driver
     */
    public String getExtension() {
        return extension;
    }

    /**
     * Obtiene la lista de tamaos de bloque posible
     * @return        Lista de tamaos de bloque
     */
    public String[] getBlockSizeList() {
        return blockSizeList;
    }
}
