/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap;

import com.iver.utiles.XMLEntity;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;


/**
 * Clase que gestiona los diferentes extent por los que ha pasado la vista.
 *
 * @author Vicente Caballero Navarro
 */
public class ExtentHistory {
	private int NUMREC;
	private Rectangle2D[] extents;
	private int num = 0;

	/**
	 * Creates a new ExtentsHistory object.
	 */
	public ExtentHistory() {
		NUMREC = 10;
		extents = new Rectangle2D[NUMREC];
	}

	/**
	 * Creates a new ExtentsHistory object.
	 *
	 * @param numEntries Numero de entradas que se guardan en el historico de
	 * 		  rectngulos, por defecto 20
	 */
	public ExtentHistory(int numEntries) {
		NUMREC = numEntries;
	}

	/**
	 * Pone un nuevo rectngulo al final del array
	 *
	 * @param ext Rectngulo que se aade al histrico
	 */
	public void put(Rectangle2D ext) {
		if ((ext != null) && ((num < 1) || (ext != extents[num - 1]))) {
			if (num < (NUMREC)) {
				extents[num] = ext;
				num = num + 1;
			} else {
				for (int i = 0; i < (NUMREC - 1); i++) {
					extents[i] = extents[i + 1];
				}

				extents[num - 1] = ext;
			}
		}
	}

	/**
	 * Devuelve true si hay algn rectngulo en el histrico
	 *
	 * @return true o false en caso de que haya o no haya rectngulos
	 */
	public boolean hasPrevious() {
		return num > 0;
	}

	/**
	 * Obtiene el ltimo rectngulo que se aadi al histrico
	 *
	 * @return Ultimo rectngulo aadido
	 */
	public Rectangle2D get() {
		Rectangle2D ext = extents[num - 1];

		return ext;
	}

	/**
	 * Devuelve el ltimo rectngulo del histrico y lo elimina del mismo
	 *
	 * @return Ultimo rectngulo aadido
	 */
	public Rectangle2D removePrev() {
		Rectangle2D ext = extents[num - 1];
		num = num - 1;

		return ext;
	}

	/**
	 * Devuelve el XMLEntity con la informacin para poder reproducir de nuevo
	 * el objeto actual.
	 *
	 * @return XMLEntity.
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className",this.getClass().getName());
		xml.putProperty("num", num);
		xml.putProperty("numrec", NUMREC);

		for (int i = 0; i < NUMREC; i++) {
			if (extents[i] != null) {
				xml.putProperty("extent" + i + "X", extents[i].getX());
				xml.putProperty("extent" + i + "Y", extents[i].getY());
				xml.putProperty("extent" + i + "W", extents[i].getWidth());
				xml.putProperty("extent" + i + "H", extents[i].getHeight());
			}
		}

		return xml;
	}

	/**
	 * Crea un nuevo ExtentHistory a partir del XMLEntity.
	 *
	 * @param xml XMLEntity
	 *
	 * @return Nuevo ExtentHistory.
	 */
	public static ExtentHistory createFromXML03(XMLEntity xml) {
		ExtentHistory eh = new ExtentHistory();
		eh.num = xml.getIntProperty("num");
		eh.NUMREC = xml.getIntProperty("numrec");

		for (int i = 0; i < eh.NUMREC; i++) {
			try {
				eh.extents[i] = new Rectangle2D.Double(xml.getDoubleProperty(
							"extent" + i + "X"),
						xml.getDoubleProperty("extent" + i + "Y"),
						xml.getDoubleProperty("extent" + i + "W"),
						xml.getDoubleProperty("extent" + i + "H"));
			} catch (Exception e) {
				///System.out.println("En las ExtentHistory =" + e); //TODO o se captura de alguna forma o se mete un nuevo parametro en el xml para saber exactamente cuantos rectgulos se han aadido.
			}
		}

		return eh;
	}

	/**
	 * Crea un nuevo ExtentHistory a partir del XMLEntity.
	 *
	 * @param xml XMLEntity
	 *
	 * @return Nuevo ExtentHistory.
	 */
	public static ExtentHistory createFromXML(XMLEntity xml) {
		ExtentHistory eh = new ExtentHistory();
		eh.num = xml.getIntProperty("num");
		eh.NUMREC = xml.getIntProperty("numrec");

		for (int i = 0; i < eh.NUMREC; i++) {
			try {
				eh.extents[i] = new Rectangle2D.Double(xml.getDoubleProperty(
							"extent" + i + "X"),
						xml.getDoubleProperty("extent" + i + "Y"),
						xml.getDoubleProperty("extent" + i + "W"),
						xml.getDoubleProperty("extent" + i + "H"));
			} catch (Exception e) {
				///System.out.println("En las ExtentHistory =" + e); //TODO o se captura de alguna forma o se mete un nuevo parametro en el xml para saber exactamente cuantos rectgulos se han aadido.
			}
		}

		return eh;
	}
}
