package com.iver.cit.gvsig.fmap.edition.writers.gml;

import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;

import com.iver.cit.gvsig.fmap.crs.CRSFactory;
import com.iver.utiles.StringUtilities;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLBoundedBy.java,v 1.2.4.1 2006/09/29 14:12:57 luisw2 Exp $
 * $Log: GMLBoundedBy.java,v $
 * Revision 1.2.4.1  2006/09/29 14:12:57  luisw2
 * CRSFactory.getCRS substitutes ProjectionPool.get
 *
 * Revision 1.2  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 * Revision 1.1  2006/07/19 12:29:39  jorpiell
 * Aadido el driver de GML
 *
 *
 */
/**
 * This class is used to get a GML projection tag
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLBoundedBy {
	private static final String EPSG_URL = "http://www.opengis.net/gml/srs/epsg.xml#";
	
	public static String fromGvSigToGML(IProjection proj){
		if (proj == null){
			return EPSG_URL + "23030";
		}	
		
		String srs = proj.getAbrev();
		if (srs.compareTo("EPSG") == 0){
			if (srs.split(":").length > 1){
				return EPSG_URL + srs.split(":")[1];
			}
		}
		return "http://www.opengis.net/gml/srs/epsg.xml#23030";
	}
	
	public static IProjection fromGMLToGvSIG(String srs){
		srs = StringUtilities.replace(srs,EPSG_URL,"");
		srs = "EPSG:" + srs;
		IProjection projection = CRSFactory.getCRS(srs);
		if (projection != null){
			return projection;
		}
		return CRSFactory.getCRS("EPSG:23030");
		
	}

}
