/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D;

import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;

import java.awt.Color;


/**
 * Rampa de colores para el dibujado de Raster.
 *
 * @author Vicente Caballero Navarro
 */
public class ColorRamp implements RasterLegend {
	private Color firstColor;
	private Color lastColor;
	private double first;
	private double last;

	/**
	 * Inserta el nmero inicial y el color inicial.
	 *
	 * @param d Nmero inicial.
	 * @param c Color inicial.
	 */
	public void setFirstPoint(double d, Color c) {
		firstColor = c;
		first = d;
	}

	/**
	 * Inserta el Nmero fianl y el color final.
	 *
	 * @param d Nmero final.
	 * @param c Color final.
	 */
	public void setLastPoint(double d, Color c) {
		lastColor = c;
		last = d;
	}

	/**
	 * Devuelve el color a partir del nmero.
	 *
	 * @param d Nmero.
	 *
	 * @return Color.
	 */
	public Color getColor(double d) {
		int fr = firstColor.getRed();
		int fg = firstColor.getGreen();
		int fb = firstColor.getBlue();

		int lr = lastColor.getRed();
		int lg = lastColor.getGreen();
		int lb = lastColor.getBlue();

		double total = last - first;
		double tr = lr - fr;
		double tg = lg - fg;
		double tb = lb - fb;

		double por = (d - first) / total;
		int r = (int) ((por * tr) + fr);
		int g = (int) ((por * tg) + fg);
		int b = (int) ((por * tb) + fb);

		return new Color(r, g, b);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo#getDescriptions()
	 */
	public String[] getDescriptions() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo#getSymbols()
	 */
	public FSymbol[] getSymbols() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo#getValues()
	 */
	public Object[] getValues() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getDefaultSymbol()
	 */
	public FSymbol getDefaultSymbol() {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.RasterLegend#getSymbol(double)
	 */
	public FStyle2D getSymbol(double d) {
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.RasterLegend#setDefaultSymbol(com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D)
	 */
	public void setDefaultSymbol(FStyle2D s) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getSLDString()
	 */
	public String getSLDString(String layerName)
	{
		StringBuffer sld = new StringBuffer();
//		xml.putProperty("className",this.getClass().getName());
//		xml.putProperty("first", first);
//		xml.putProperty("firstColor", StringUtilities.color2String(firstColor));
//		xml.putProperty("last", last);
//		xml.putProperty("lastColor", StringUtilities.color2String(lastColor));

		return sld.toString();
	}	
	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getXMLEntity()
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.putProperty("className",this.getClass().getName());
		xml.putProperty("first", first);
		xml.putProperty("firstColor", StringUtilities.color2String(firstColor));
		xml.putProperty("last", last);
		xml.putProperty("lastColor", StringUtilities.color2String(lastColor));

		return xml;
	}

	/**
	 * Crea un ColorRamp a partir del XMLEntity.
	 *
	 * @param xml XMLEntity.
	 *
	 * @return Nuevo ColorRamp.
	 */
	public static ColorRamp createFromXML(XMLEntity xml) {
		ColorRamp cr = new ColorRamp();
		cr.first = xml.getDoubleProperty("first");
		cr.firstColor = StringUtilities.string2Color(xml.getStringProperty(
					"fistColor"));
		cr.last = xml.getDoubleProperty("last");
		cr.lastColor = StringUtilities.string2Color(xml.getStringProperty(
					"lastColor"));

		return cr;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#cloneLegend()
	 */
	public Legend cloneLegend() {
		return createFromXML(getXMLEntity());
	}
}
