/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.utiles;

import com.iver.utiles.xmlEntity.generate.Property;
import com.iver.utiles.xmlEntity.generate.XmlTag;

/**
 * Adaptador de las llamadas a sus mtodos sobre los generados por castor para
 * generar despus los XML.
 *
 * @author Vicente Caballero Navarro
 */
public class XMLEntity {
	private XmlTag xmltag;

	/**
	 * Crea un nuevo XMLEntity.
	 *
	 * @param tag
	 *            DOCUMENT ME!
	 */
	public XMLEntity(XmlTag tag) {
		xmltag = tag;
	}

	/**
	 * Crea un nuevo XMLEntity.
	 */
	public XMLEntity() {
		xmltag = new XmlTag();
	}

	/**
	 * Aade una propiedad con un String como clave y un String como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, String value) {
		if ((key == null)) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(value);
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un Object como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, Object value) {
		String ret = "";

		if (key == null) {
			return;
		}

		Class valueClass = value.getClass();

		if (valueClass.isArray()) {
			Class compType = valueClass.getComponentType();

			if (compType == byte.class) {
				byte[] array = (byte[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == short.class) {
				short[] array = (short[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == int.class) {
				int[] array = (int[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == long.class) {
				long[] array = (long[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == float.class) {
				float[] array = (float[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == double.class) {
				double[] array = (double[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == boolean.class) {
				boolean[] array = (boolean[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						ret += (" ," + ("" + array[i]).replaceAll("[,]",
								"\\\\,"));
					}
				}
			} else if (compType == String.class) {
				String[] array = (String[]) value;

				if (!(array.length == 0)) {
					if (array[0] == null) {
						array[0] = "null";
					}

					ret += array[0].replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						if (array[i] == null) {
							array[i] = "null";
						}

						array[i] = array[i].replaceAll("[,]", "\\\\,");
						ret += (" ," + array[i]);
					}
				}
			} else {
				Object[] array = (Object[]) value;

				if (!(array.length == 0)) {
					ret += ("" + array[0]).replaceAll("[,]", "\\\\,");

					for (int i = 1; i < array.length; i++) {
						array[i] = ("" + array[i]).replaceAll("[,]", "\\\\,");
						ret += (" ," + array[i]);
					}
				}
			}
		} else {
			ret = value.toString();
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(ret);
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un entero como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, int value) {
		if (key == null) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(new Integer(value).toString());
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un long como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, long value) {
		if (key == null) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(new Long(value).toString());
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un boolean como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, boolean value) {
		if (key == null) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(new Boolean(value).toString());
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un float como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, float value) {
		if (key == null) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(new Float(value).toString());
		putProperty(p);
	}

	/**
	 * Aade una propiedad con un String como clave y un double como valor.
	 *
	 * @param key
	 *            clave.
	 * @param value
	 *            valor.
	 */
	public void putProperty(String key, double value) {
		if (key == null) {
			return;
		}

		Property p = new Property();
		p.setKey(key);
		p.setValue(new Double(value).toString());
		putProperty(p);
	}

	/**
	 * Devuelve el String que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public String getStringProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String res = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = properties[i].getValue();
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el double que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public double getDoubleProperty(String key) {
		Property[] properties = xmltag.getProperty();
		double res = 0;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = Double.parseDouble(properties[i].getValue());
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el array de doubles que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public double[] getDoubleArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new double[0];
		}

		String[] aux = (String[]) value.split(" ,");
		double[] ret = new double[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Double.parseDouble(aux[i].replaceAll("\\\\,", ","));
		}

		return ret;
	}

	/**
	 * Devuelve el Object que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public Object getObjectProperty(String key) {
		Property[] properties = xmltag.getProperty();
		Object res = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = properties[i].getValue();
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el array de float que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public float[] getFloatArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new float[0];
		}

		String[] aux = (String[]) value.split(" ,");
		float[] ret = new float[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Float.parseFloat(aux[i].replaceAll("\\\\,", ","));
		}

		return ret;
	}

	/**
	 * Devuelve el array de long que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public long[] getLongArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new long[0];
		}

		String[] aux = (String[]) value.split(" ,");
		long[] ret = new long[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Long.parseLong(aux[i].replaceAll("\\\\,", ","));
		}

		return ret;
	}

	/**
	 * Devuelve el array de bytes que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public byte[] getByteArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new byte[0];
		}

		String[] aux = (String[]) value.split(" ,");
		byte[] ret = new byte[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Byte.parseByte(aux[i].replaceAll("\\\\,", ","));
		}

		return ret;
	}

	/**
	 * Devuelve el array de enteros que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public int[] getIntArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new int[0];
		}

		String[] aux = (String[]) value.split(" ,");
		int[] ret = new int[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Integer.parseInt(aux[i].replaceAll("\\\\,", ","));
		}

		return ret;
	}

	/**
	 * Devuelve el array de boolean que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public boolean[] getBooleanArrayProperty(String key) {
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new boolean[0];
		}

		String[] aux = (String[]) value.split(" ,");
		boolean[] ret = new boolean[aux.length];

		for (int i = 0; i < aux.length; i++) {
			ret[i] = Boolean.valueOf(aux[i].replaceAll("\\\\,", ","))
					.booleanValue();
		}

		return ret;
	}

	/**
	 * Devuelve el array de String que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public String[] getStringArrayProperty(String key) {
		// /String value = (String) properties.get(key);
		Property[] properties = xmltag.getProperty();
		String value = null;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				value = properties[i].getValue();
				exists = true;
			}
		}

		if (!exists) {
			throw new NotExistInXMLEntity();
		}

		if (value.compareTo("") == 0) {
			return new String[0];
		}

		String[] aux = (String[]) value.split(" ,");

		for (int i = 0; i < aux.length; i++) {
			aux[i] = aux[i].replaceAll("\\\\,", ",");
		}

		return aux;
	}

	/**
	 * Devuelve el boolean que corresponda a la clave que se pasa como
	 * parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public boolean getBooleanProperty(String key) {
		Property[] properties = xmltag.getProperty();
		boolean res = false;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = (boolean) Boolean.valueOf(properties[i].getValue())
						.booleanValue();
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el entero que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public int getIntProperty(String key) {
		Property[] properties = xmltag.getProperty();
		int res = 0;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = Integer.parseInt(properties[i].getValue());
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el long que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public long getLongProperty(String key) {
		Property[] properties = xmltag.getProperty();
		long res = 0;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = Long.valueOf(properties[i].getValue()).longValue();
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Devuelve el float que corresponda a la clave que se pasa como parmetro.
	 *
	 * @param key
	 *            clave
	 *
	 * @return valor.
	 *
	 * @throws NotExistInXMLEntity
	 *             Lanza esta excepcin si no se encuentra ningn elemento con
	 *             esa clave.
	 */
	public float getFloatProperty(String key) {
		Property[] properties = xmltag.getProperty();
		float res = 0;
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				res = Float.valueOf(properties[i].getValue()).floatValue();
				exists = true;
			}
		}

		if (exists) {
			return res;
		}

		throw new NotExistInXMLEntity();
	}

	/**
	 * Aade el nombre con clave name y valor el String que se pasa como
	 * parmetro.
	 *
	 * @param name
	 *            nombre.
	 */
	public void setName(String name) {
		Property p = new Property();
		p.setKey("name");
		p.setValue(name);
		xmltag.addProperty(p);
	}

	/**
	 * Devuelve el nombre.
	 *
	 * @return nombre.
	 */
	public String getName() {
		Property p = getProperty("name");

		return (String) p.getValue();
	}

	/**
	 * Devuelve la clase que implementa.
	 *
	 * @return clase.
	 */
	public Class getImplementingClass() {
		Property p = getProperty("class");
		Object ob = p.getValue();

		return (Class) ob;
	}

	/**
	 * Aade la clase que implementa
	 *
	 * @param c
	 */
	/*
	 * public void setImplementingClass(Class c) { Property p = new Property();
	 * p.setKey("class"); p.setValue(c.toString()); xmltag.addProperty(p); }
	 */

	/**
	 * Aade un hijo al XMLEntity.
	 *
	 * @param entity
	 *            xml para aadir.
	 */
	public void addChild(XMLEntity entity) {
		xmltag.addXmlTag(entity.getXmlTag());
	}

	/**
	 * Devuelve un hijo a partir de un indice.
	 *
	 * @param i
	 *            indice.
	 *
	 * @return hijo.
	 */
	public XMLEntity getChild(int i) {
		return new XMLEntity(xmltag.getXmlTag(i));
	}

	/**
	 * Devuelve el nmero de hijos que contiene el XMLEntity.
	 *
	 * @return nmero de hijos.
	 */
	public int getChildrenCount() {
		return xmltag.getXmlTagCount();
	}

	/**
	 * Devuelve el Property a partir de la clave.
	 *
	 * @param key
	 *            clave.
	 *
	 * @return Property.
	 */
	private Property getProperty(String key) {
		Property[] ps = xmltag.getProperty();

		for (int i = 0; i < ps.length; i++) {
			if (ps[i].getKey().compareTo(key) == 0) {
				return ps[i];
			}
		}

		return null;
	}

	/**
	 * Devuelve el xmltag.
	 *
	 * @return xmltag.
	 */
	public XmlTag getXmlTag() {
		return xmltag;
	}

	/**
	 * Aade el property que se pasa como parmetro.
	 *
	 * @param p
	 *            property.
	 */
	private void putProperty(Property p) {
		for (int i = 0; i < xmltag.getPropertyCount(); i++) {
			if (xmltag.getProperty(i).getKey().compareTo(p.getKey()) == 0) {
				xmltag.getProperty(i).setValue(p.getValue());

				return;
			}
		}

		xmltag.addProperty(p);
	}

	public boolean contains(String key) {
		Property[] properties = xmltag.getProperty();
		boolean exists = false;

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				exists = true;
			}
		}
		return exists;
	}

	public int getPropertyCount() {
		return xmltag.getPropertyCount();
	}

	public String getPropertyValue(int index) {
		return xmltag.getProperty(index).getValue();
	}

	public String getPropertyName(int index) {
		return xmltag.getProperty(index).getKey();
	}

	/**
	 * Removes a property of this XML-Entity.
	 *
	 * @param p
	 *            property.
	 */
	public void remove(String key) {
		Property[] properties = xmltag.getProperty();

		for (int i = 0; i < properties.length; i++) {
			if (properties[i].getKey().compareTo(key) == 0) {
				Property[] newProperties = new Property[properties.length - 1];
				int k = 0;
				for (int j = 0; j < newProperties.length; j++) {
					if (j == i)
						k = 1;
					newProperties[j] = properties[j + k];
				}
				xmltag.setProperty(newProperties);
				return;
			}
		}
	}

	/**
	 * Elimina el hijo n del XMLEntity.
	 *
	 * @param indice
	 *            del hijo a eliminar.
	 */
	public void removeChild(int index) {
		xmltag.removeXmlTag(index);
	}

	/**
	 * Elimina todos los hijos de XMLEntity.
	 *
	 */
	public void removeAllChilds() {
		xmltag.removeAllXmlTag();
	}

	// Jaume (en proves)
	public boolean equals(Object obj) {
		if (obj instanceof XMLEntity) {
			XMLEntity other = (XMLEntity) obj;

			Property[] thisProperties;
			Property[] otherProperties;

			thisProperties = xmltag.getProperty();
			otherProperties = other.xmltag.getProperty();

			if (thisProperties.length != otherProperties.length)
				return false;

			for (int i = 0; i < thisProperties.length; i++) {
				if (!thisProperties[i].getKey().equals(
						otherProperties[i].getKey()))
					return false;
				if (!thisProperties[i].getValue().equals(
						otherProperties[i].getValue()))
					return false;
			}

			if (this.getChildrenCount() != other.getChildrenCount())
				return false;

			for (int i = 0; i < this.getChildrenCount(); i++) {
				if (!this.getChild(i).equals(other.getChild(i)))
					return false;
			}

			return true;
		}

		return false;
	}

	// jaume (en proves)
	public int hashCode() {
		int result = 17;

		if (xmltag.getName()!=null) {
			char[] name = xmltag.getName().toCharArray();

			for (int i = 0; i<name.length; i++) {
				result = 37 + result + (int) name[i];
			}
		}

		Property[] properties = xmltag.getProperty();
		for (int i = 0; i < properties.length; i++) {
			char[] key = properties[i].getKey().toCharArray();
			for (int j = 0; j < key.length; j++) {
				result = 37 + result + (int) key[j];
			}

			String value = properties[i].getValue();
			char[] chars = (value!=null)? value.toCharArray() : new char[0];
			for (int j = 0; j < chars.length; j++) {
				result = 37 + result + (int) chars[j];
			}
		}

		for (int i = 0; i < this.getChildrenCount(); i++) {
			result = 37 + result + this.getChild(i).hashCode();
		}
		return result;
	}
}
