package org.gvsig.remoteClient.gml.factories;

import java.util.Hashtable;
import java.util.Set;

import org.gvsig.remoteClient.gml.schemas.GMLGeometryType;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLComplexType;
import org.gvsig.remoteClient.gml.schemas.XMLSimpleType;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLTypesFactory.java,v 1.1.2.1 2006/09/19 12:23:15 jorpiell Exp $
 * $Log: XMLTypesFactory.java,v $
 * Revision 1.1.2.1  2006/09/19 12:23:15  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.2  2006/09/18 12:08:55  jorpiell
 * Se han hecho algunas modificaciones que necesitaba el WFS
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * GML and XML types factory. All the types that are
 * located in the schemas must be registered here
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class XMLTypesFactory {
	private static Hashtable types = new Hashtable();
		
	static{
		types.put(XMLSimpleType.STRING,new XMLSimpleType(XMLSimpleType.STRING));
		types.put(XMLSimpleType.INTEGER,new XMLSimpleType(XMLSimpleType.INTEGER));
		types.put(XMLSimpleType.INT,new XMLSimpleType(XMLSimpleType.INT));
		types.put(XMLSimpleType.DOUBLE,new XMLSimpleType(XMLSimpleType.DOUBLE));
		types.put(XMLSimpleType.FLOAT,new XMLSimpleType(XMLSimpleType.FLOAT));
		types.put(XMLSimpleType.BOOLEAN,new XMLSimpleType(XMLSimpleType.BOOLEAN));
		types.put(XMLSimpleType.LONG,new XMLSimpleType(XMLSimpleType.LONG));
		types.put(GMLGeometryType.POINT,new GMLGeometryType(GMLGeometryType.POINT));
		types.put(GMLGeometryType.MULTIPOINT,new GMLGeometryType(GMLGeometryType.MULTIPOINT));
		types.put(GMLGeometryType.LINE,new GMLGeometryType(GMLGeometryType.LINE));
		types.put(GMLGeometryType.POLYGON,new GMLGeometryType(GMLGeometryType.POLYGON));
		types.put(GMLGeometryType.GEOMETRY,new GMLGeometryType(GMLGeometryType.GEOMETRY));
	}
	
	/**
	 * Gets a type by name
	 * @param type
	 * Type name
	 * @return
	 */
	public static IXMLType getType(String type){
		return (IXMLType)types.get(type.toUpperCase());
	}
	
	/**
	 * Adds a new type
	 * @param type
	 * type to add 
	 */
	private static void addType(IXMLType type){
		types.put(type.getName().toUpperCase(),type);
	}	
	
	/**
	 * Adds a complex type
	 * @param nameSpace
	 * NameSpace where is located
	 * @param name
	 * Complex type name
	 * @return
	 */
	public static XMLComplexType addCompleyType(String nameSpace,String name){
		XMLComplexType complexType = new XMLComplexType(nameSpace + ":" + name);
		addType(complexType);
		return complexType;
	}

	/**
	 * Just for degug. It prints all the registred components.
	 */
	public static void printTypes(){
		System.out.println("*** TIPOS ***");
		Object[] keys = types.keySet().toArray();
		for (int i=0 ; i<types.size() ; i++){
			IXMLType type = (IXMLType)types.get(keys[i]);
			System.out.print("NAME: " + type.getName());
			System.out.print(" TYPE: " + type.getType() + "\n");
		}
	}
	
}
