package org.gvsig.remoteClient.gml.v2;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.gml.IGMLFeaturesIterator;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.factories.XMLElementsFactory;
import org.gvsig.remoteClient.gml.factories.XMLTypesFactory;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLComplexType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.gml.schemas.XMLSimpleType;
import org.gvsig.remoteClient.gml.utils.GMLUtilsParser;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesIterator_v2.java,v 1.1.2.2 2006/09/25 11:35:15 jorpiell Exp $
 * $Log: GMLFeaturesIterator_v2.java,v $
 * Revision 1.1.2.2  2006/09/25 11:35:15  jorpiell
 * Se tienen en cuanta tablas a distintos niveles. En caso de anidamiento se cogen los resultados de la primera tabla que aparezca.
 *
 * Revision 1.1.2.1  2006/09/19 12:23:15  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.2  2006/09/18 12:08:55  jorpiell
 * Se han hecho algunas modificaciones que necesitaba el WFS
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLFeaturesIterator_v2 extends IGMLFeaturesIterator {
	
	public GMLFeaturesIterator_v2(XMLSchemaParser parser, IGeometriesFactory factory) {
		super(parser, factory);		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.IGMLFeaturesIterator#hasNext()
	 */
	public boolean hasNext() throws GMLException {
		try {
			if ((getParser().getName().compareTo(getFeatureRoot()) == 0) &&
				(getParser().getEventType() == KXmlParser.START_TAG)){
				return true;
			}
		} catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		}		
		return false;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.IGMLFeaturesIterator#next()
	 */
	public Object next() throws GMLException {
		boolean endFeature = false;
		Object feature = null;
		int currentTag;		
		
		try {	
			currentTag = getParser().getEventType();	
			while (!endFeature){
				switch(currentTag){
				case KXmlParser.START_TAG:
					feature = parseFeature();
					break;
				case KXmlParser.END_TAG:				
					if (getParser().getName().compareTo(getFeatureRoot()) == 0){
						endFeature = true;	
						currentTag = getParser().nextTag();
					}
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!endFeature){
					currentTag = getParser().next();
				}
			}
		}catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_READ_FILE);
		} 		
		return feature;
	}
	
	/**
	 * It parses a Feature
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private Object parseFeature() throws GMLException {
		int currentTag;
		boolean end = false;
				
		try {
			currentTag = getParser().getEventType();
			String featureType = null;
			
			if ((getParser().getName() != null) && (featureType == null)){
				featureType = getParser().getName();
				return parseSpecificFeature(featureType);
			}
		}catch (XmlPullParserException e) {
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		}
		return null;
				
	}
	
	private Object parseSpecificFeature(String elementName) throws GMLException {
		int currentTag;
		boolean end = false;
		ArrayList params = new ArrayList();
		ArrayList values = new ArrayList();
		Hashtable duplicates = new Hashtable();
		Object geom = null;
		
		XMLElement element = XMLElementsFactory.getElement(elementName);
		if (element != null){
			XMLComplexType elementType = (XMLComplexType)element.getType();
			try{		
				currentTag = getParser().next();					
				while (!end){			
					switch(currentTag){
					case KXmlParser.START_TAG:
						String attName = getParser().getName();
						XMLElement attribute = elementType.getElement(attName);
						if (attribute != null){
							if (attribute.getType() != null){
								if (attribute.getType().getType() == IXMLType.SIMPLE){
									if (duplicates.get(attName) == null){
										params.add(attName);
										duplicates.put(attName,attName);
										getParser().next();
										values.add(((XMLSimpleType)attribute.getType()).getObject(getParser().getText()));
									}
								}else if (attribute.getType().getType() == IXMLType.GML_GEOMETRY){
									geom = GMLUtilsParser.parseGeometry(getParser(),attName,getFactory());
								}else if (attribute.getType().getType() == IXMLType.COMPLEX){
									//Not in GML v2
								}
							}
						}
						break;
					case KXmlParser.END_TAG:
						if ((getParser().getName().compareTo(elementName) == 0))
							end = true;
						break;
					case KXmlParser.TEXT:                   
						break;
					}
					if (!end){
						currentTag = getParser().next();
					}	
				}
			}catch (XmlPullParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new GMLException(GMLException.EXC_PARSE);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				throw new GMLException(GMLException.EXC_READ_FILE);
			} 		
		}
		return getFactory().createSimpleFeature(elementName,geom,params,values);
	}
	
}
